@@ -97,16 +97,23 @@ static void update_nr_buttons(
 	return;
 }
 
-static void activate_binding(
-	Binding *binding, BindingType type, Bool do_grab, Bool do_grab_root)
+static int activate_binding(Binding *binding, binding_t type, Bool do_grab)
 {
 	FvwmWindow *t;
+	Bool rc = 0;
 
 	if (binding == NULL)
 	{
-		return;
+		return rc;
 	}
-	if (do_grab_root)
+	if (BIND_IS_KEY_RELEASE(type))
+	{
+		/* Release a passive grab on the key if any.  Actually this may
+		 * release the grab for the wrong key, but it will prevent the
+		 * keyboard from staying frozen forever. */
+		MyXUngrabKey(dpy);
+	}
+	if (BIND_IS_PKEY_BINDING(type) || binding->Context == C_ALL)
 	{
 		/* necessary for key bindings that work over unfocused windows
 		 */
@@ -114,10 +121,19 @@ static void activate_binding(
 			dpy, Scr.Root, binding,
 			C_WINDOW | C_DECOR | C_ROOT | C_ICON | C_EWMH_DESKTOP,
 			GetUnusedModifiers(), None, do_grab);
+		if (do_grab == False)
+		{
+			rc = 1;
+		}
+	}
+	if (do_grab == False && BIND_IS_KEY_BINDING(type) &&
+	    (binding->Context & C_ROOT))
+	{
+		rc = 1;
 	}
 	if (fFvwmInStartup == True)
 	{
-		return;
+		return rc;
 	}
 
 	/* grab keys immediately */
@@ -125,7 +141,7 @@ static void activate_binding(
 	{
 		if (!IS_EWMH_DESKTOP(FW_W(t)) &&
 		    (binding->Context & (C_WINDOW | C_DECOR)) &&
-		    binding->type == KEY_BINDING)
+		    BIND_IS_KEY_BINDING(type))
 		{
 			GrabWindowKey(
 				dpy, FW_W_FRAME(t), binding,
@@ -158,7 +174,7 @@ static void activate_binding(
 		}
 	}
 
-	return;
+	return rc;
 }
 
 static int bind_get_bound_button_contexts(
@@ -173,8 +189,8 @@ static int bind_get_bound_button_contexts(
 	}
 	for (b = *pblist; b != NULL; b = b->NextBinding)
 	{
-		if (b->type != MOUSE_BINDING
-		    STROKE_CODE(&& b->type != STROKE_BINDING))
+		if (!BIND_IS_MOUSE_BINDING(b->type) &&
+		    !BIND_IS_STROKE_BINDING(b->type))
 		{
 			continue;
 		}
@@ -199,11 +215,29 @@ static int bind_get_bound_button_contexts(
 
 	return bcontext;
 }
+
+static void __rebind_global_key(Binding **pblist, int Button_Key)
+{
+	Binding *b;
+
+	for (b = *pblist; b != NULL; b = b->NextBinding)
+	{
+		if (b->Button_Key == Button_Key &&
+		    (BIND_IS_PKEY_BINDING(b->type) || b->Context == C_ALL))
+		{
+			activate_binding(b, b->type, True);
+			return;
+		}
+	}
+
+	return;
+}
+
 /* Parses a mouse or key binding */
 static int ParseBinding(
-	Display *dpy, Binding **pblist, char *tline, BindingType type,
+	Display *dpy, Binding **pblist, char *tline, binding_t type,
 	int *nr_left_buttons, int *nr_right_buttons,
-	unsigned short *buttons_grabbed, Bool do_ungrab_root, Bool is_silent)
+	unsigned short *buttons_grabbed, Bool is_silent)
 {
 	char *action, context_string[20], modifier_string[20], *ptr, *token;
 	char key_string[201] = "";
@@ -214,6 +248,9 @@ static int ParseBinding(
 	int modifier;
 	Bool is_unbind_request = False;
 	int rc;
+	Bool is_binding_removed = False;
+	Binding *b;
+	Binding *rmlist = NULL;
 	STROKE_CODE(char stroke[STROKE_MAX_SEQUENCE + 1] = "");
 	STROKE_CODE(int n4=0);
 	STROKE_CODE(int i);
@@ -222,13 +259,19 @@ static int ParseBinding(
 	token = PeekToken(tline, &ptr);
 	if (token != NULL)
 	{
-		if (type == KEY_BINDING)
+		if (BIND_IS_KEY_BINDING(type))
 		{
+			if (token[0] == '-')
+			{
+				/* it's a key release binding */
+				type++;
+				token++;
+			}
 			/* see len of key_string above */
 			n1 = sscanf(token,"%200s", key_string);
 		}
 #ifdef HAVE_STROKE
-		else if (type == STROKE_BINDING)
+		else if (BIND_IS_STROKE_BINDING(type))
 		{
 			int num = 0;
 			int j;
@@ -313,7 +356,7 @@ static int ParseBinding(
 	}
 
 #ifdef HAVE_STROKE
-	if (type == STROKE_BINDING)
+	if (BIND_IS_STROKE_BINDING(type))
 	{
 		token = PeekToken(ptr, &ptr);
 		if (token != NULL)
@@ -335,7 +378,7 @@ static int ParseBinding(
 	}
 
 	if (n1 != 1 || n2 != 1 || n3 != 1
-	    STROKE_CODE(|| (type == STROKE_BINDING && n4 != 1)))
+	    STROKE_CODE(|| (BIND_IS_STROKE_BINDING(type) && n4 != 1)))
 	{
 		if (!is_silent)
 		{
@@ -359,7 +402,7 @@ static int ParseBinding(
 			tline);
 	}
 
-	if (type == KEY_BINDING)
+	if (BIND_IS_KEY_BINDING(type))
 	{
 		keysym = FvwmStringToKeysym(dpy, key_string);
 		/* Don't let a 0 keycode go through, since that means AnyKey
@@ -392,46 +435,48 @@ static int ParseBinding(
 	/*
 	** Remove the "old" bindings if any
 	*/
+	/* BEGIN remove */
+	CollectBindingList(
+		dpy, pblist, &rmlist, type, STROKE_ARG((void *)stroke)
+		button, keysym, modifier, context);
+	if (rmlist != NULL)
 	{
-		Bool is_binding_removed = False;
-		Binding *b;
-		Binding *rmlist = NULL;
-
-		CollectBindingList(
-			dpy, pblist, &rmlist, type, STROKE_ARG((void *)stroke)
-			button, keysym, modifier, context);
-		if (rmlist != NULL)
-		{
-			is_binding_removed = True;
-			if (is_unbind_request)
-			{
-				/* remove the grabs for the key for unbind
-				 * requests */
-				for (b = rmlist; b != NULL; b = b->NextBinding)
-				{
-					/* release the grab */
-					activate_binding(
-						b, type, False, do_ungrab_root);
-				}
-			}
-			FreeBindingList(rmlist);
-		}
-		if (is_binding_removed)
-		{
-			int bcontext;
-
-			bcontext = bind_get_bound_button_contexts(
-				pblist, buttons_grabbed);
-			update_nr_buttons(
-				bcontext, nr_left_buttons, nr_right_buttons,
-				True);
-		}
-		/* return if it is an unbind request */
+		is_binding_removed = True;
 		if (is_unbind_request)
 		{
-			return 0;
+			int rc = 0;
+
+			/* remove the grabs for the key for unbind
+			 * requests */
+			for (b = rmlist; b != NULL; b = b->NextBinding)
+			{
+				/* release the grab */
+				rc |= activate_binding(b, type, False);
+			}
+			if (rc)
+			{
+				__rebind_global_key(pblist, rmlist->Button_Key);
+			}
 		}
+		FreeBindingList(rmlist);
 	}
+	if (is_binding_removed)
+	{
+		int bcontext;
+
+		bcontext = bind_get_bound_button_contexts(
+			pblist, buttons_grabbed);
+		update_nr_buttons(
+			bcontext, nr_left_buttons, nr_right_buttons,
+			True);
+	}
+	/* return if it is an unbind request */
+	if (is_unbind_request)
+	{
+		return 0;
+	}
+	/* END remove */
+
 	update_nr_buttons(context, nr_left_buttons, nr_right_buttons, False);
 	if ((modifier & AnyModifier)&&(modifier&(~AnyModifier)))
 	{
@@ -441,9 +486,8 @@ static int ParseBinding(
 			" ignored.");
 		modifier = AnyModifier;
 	}
-	if ((type == MOUSE_BINDING STROKE_CODE(|| type == STROKE_BINDING)) &&
-	    (context & (C_WINDOW | C_EWMH_DESKTOP)) &&
-	    buttons_grabbed != NULL)
+	if ((BIND_IS_MOUSE_BINDING(type) || BIND_IS_STROKE_BINDING(type)) &&
+	    (context & (C_WINDOW | C_EWMH_DESKTOP)) && buttons_grabbed != NULL)
 	{
 		if (button == 0)
 		{
@@ -463,7 +507,7 @@ static int ParseBinding(
 	return rc;
 }
 
-static void binding_cmd(F_CMD_ARGS, BindingType type, Bool do_grab_root)
+static void binding_cmd(F_CMD_ARGS, binding_t type)
 {
 	Binding *b;
 	int count;
@@ -471,8 +515,7 @@ static void binding_cmd(F_CMD_ARGS, BindingType type, Bool do_grab_root)
 
 	count = ParseBinding(
 		dpy, &Scr.AllBindings, action, type, &Scr.nr_left_buttons,
-		&Scr.nr_right_buttons, &btg, do_grab_root,
-		Scr.flags.silent_functions);
+		&Scr.nr_right_buttons, &btg, Scr.flags.silent_functions);
 	if (btg != Scr.buttons2grab)
 	{
 		Scr.flags.do_need_window_update = 1;
@@ -481,7 +524,7 @@ static void binding_cmd(F_CMD_ARGS, BindingType type, Bool do_grab_root)
 	}
 	for (b = Scr.AllBindings; count > 0; count--, b = b->NextBinding)
 	{
-		activate_binding(b, type, True, do_grab_root);
+		activate_binding(b, type, True);
 	}
 
 	return;
@@ -504,21 +547,21 @@ unsigned int GetUnusedModifiers(void)
 
 void CMD_Key(F_CMD_ARGS)
 {
-	binding_cmd(F_PASS_ARGS, KEY_BINDING, False);
+	binding_cmd(F_PASS_ARGS, BIND_KEYPRESS);
 
 	return;
 }
 
 void CMD_PointerKey(F_CMD_ARGS)
 {
-	binding_cmd(F_PASS_ARGS, KEY_BINDING, True);
+	binding_cmd(F_PASS_ARGS, BIND_PKEYPRESS);
 
 	return;
 }
 
 void CMD_Mouse(F_CMD_ARGS)
 {
-	binding_cmd(F_PASS_ARGS, MOUSE_BINDING, False);
+	binding_cmd(F_PASS_ARGS, BIND_BUTTONPRESS);
 
 	return;
 }
@@ -526,7 +569,7 @@ void CMD_Mouse(F_CMD_ARGS)
 #ifdef HAVE_STROKE
 void CMD_Stroke(F_CMD_ARGS)
 {
-	binding_cmd(F_PASS_ARGS, STROKE_BINDING, False);
+	binding_cmd(F_PASS_ARGS, BIND_STROKE);
 
 	return;
 }
