@@ -120,6 +120,7 @@ enum
 	F_RESTART,
 	F_SAVE_SESSION,
 	F_SAVE_QUIT_SESSION,
+	F_SCANFORWINDOW,
 	F_SCHEDULE,
 	F_SCROLL,
 	F_SETDESK,
@@ -187,8 +188,8 @@ enum
 	F_SEND_STRING,
 	F_STATE,
 	F_STICK,
-	F_STICKDESK,
-	F_STICKPAGE,
+	F_STICKACROSSDESKS,
+	F_STICKACROSSPAGES,
 	F_UPDATE_DECOR,
 	F_WARP,
 	F_WINDOWID,
@@ -277,10 +278,8 @@ void CMD_Focus(F_CMD_ARGS);
 void CMD_FocusStyle(F_CMD_ARGS);
 void CMD_Function(F_CMD_ARGS);
 void CMD_GlobalOpts(F_CMD_ARGS);
-#ifdef GNOME
 void CMD_GnomeButton(F_CMD_ARGS);
 void CMD_GnomeShowDesks(F_CMD_ARGS);
-#endif /* GNOME */
 void CMD_GotoDesk(F_CMD_ARGS);
 void CMD_GotoDeskAndPage(F_CMD_ARGS);
 void CMD_GotoPage(F_CMD_ARGS);
@@ -314,6 +313,7 @@ void CMD_Next(F_CMD_ARGS);
 void CMD_None(F_CMD_ARGS);
 void CMD_Nop(F_CMD_ARGS);
 void CMD_NoWindow(F_CMD_ARGS);
+void CMD_On(F_CMD_ARGS);
 void CMD_OpaqueMoveSize(F_CMD_ARGS);
 void CMD_Pick(F_CMD_ARGS);
 void CMD_PipeRead(F_CMD_ARGS);
@@ -345,6 +345,7 @@ void CMD_RestackTransients(F_CMD_ARGS);
 void CMD_Restart(F_CMD_ARGS);
 void CMD_SaveQuitSession(F_CMD_ARGS);
 void CMD_SaveSession(F_CMD_ARGS);
+void CMD_ScanForWindow(F_CMD_ARGS);
 void CMD_Schedule(F_CMD_ARGS);
 void CMD_Scroll(F_CMD_ARGS);
 void CMD_Send_ConfigInfo(F_CMD_ARGS);
@@ -360,8 +361,8 @@ void CMD_SnapAttraction(F_CMD_ARGS);
 void CMD_SnapGrid(F_CMD_ARGS);
 void CMD_State(F_CMD_ARGS);
 void CMD_Stick(F_CMD_ARGS);
-void CMD_StickDesk(F_CMD_ARGS);
-void CMD_StickPage(F_CMD_ARGS);
+void CMD_StickAcrossDesks(F_CMD_ARGS);
+void CMD_StickAcrossPages(F_CMD_ARGS);
 #ifdef HAVE_STROKE
 void CMD_Stroke(F_CMD_ARGS);
 void CMD_StrokeFunc(F_CMD_ARGS);
