@@ -43,6 +43,8 @@
 #include "focus.h"
 #include "geometry.h"
 #include "stack.h"
+#include "commands.h"
+#include "decorations.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -415,26 +417,54 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 		}
 		else if (StrEquals(cond,"Sticky"))
 		{
-			SET_STICKY_ON_PAGE(mask, on);
-			SET_STICKY_ON_DESK(mask, on);
-			SETM_STICKY_ON_PAGE(mask, 1);
-			SETM_STICKY_ON_DESK(mask, 1);
+			SET_STICKY_ACROSS_PAGES(mask, on);
+			SET_STICKY_ACROSS_DESKS(mask, on);
+			SETM_STICKY_ACROSS_PAGES(mask, 1);
+			SETM_STICKY_ACROSS_DESKS(mask, 1);
 		}
-		else if (StrEquals(cond,"StickyPage"))
+		else if (StrEquals(cond,"StickyAcrossPages"))
 		{
-			SET_STICKY_ON_PAGE(mask, on);
-			SETM_STICKY_ON_PAGE(mask, 1);
+			SET_STICKY_ACROSS_PAGES(mask, on);
+			SETM_STICKY_ACROSS_PAGES(mask, 1);
 		}
-		else if (StrEquals(cond,"StickyDesk"))
+		else if (StrEquals(cond,"StickyAcrossDesks"))
 		{
-			SET_STICKY_ON_DESK(mask, on);
-			SETM_STICKY_ON_DESK(mask, 1);
+			SET_STICKY_ACROSS_DESKS(mask, on);
+			SETM_STICKY_ACROSS_DESKS(mask, 1);
 		}
 		else if (StrEquals(cond,"Maximized"))
 		{
 			SET_MAXIMIZED(mask, on);
 			SETM_MAXIMIZED(mask, 1);
 		}
+		else if (StrEquals(cond,"FixedSize"))
+		{
+		        /* don't set mask here, because we make the test here
+			   (and don't compare against window's mask)
+			   by checking allowed function */
+		        SET_SIZE_FIXED(mask, on);
+			SETM_SIZE_FIXED(mask, 1);
+		}		
+		else if (StrEquals(cond,"HasHandles"))
+		{
+		        SET_HAS_HANDLES(mask,on);
+			SETM_HAS_HANDLES(mask,1);
+		}
+		else if (StrEquals(cond,"Iconifiable"))
+		{
+		        SET_IS_UNICONIFIABLE(mask,!on);
+		        SETM_IS_UNICONIFIABLE(mask,1);
+		}
+		else if (StrEquals(cond,"Maximizable"))
+		{
+		        SET_IS_UNMAXIMIZABLE(mask,!on);
+		        SETM_IS_UNMAXIMIZABLE(mask,1);
+		}
+		else if (StrEquals(cond,"Closable"))
+		{
+		        SET_IS_UNCLOSABLE(mask,!on);
+		        SETM_IS_UNCLOSABLE(mask,1);
+		}
 		else if (StrEquals(cond,"Shaded"))
 		{
 			SET_SHADED(mask, on);
@@ -570,12 +600,77 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 	Bool fMatchesResource;
 	Bool fMatches;
 	FvwmWindow *sf = get_focus_window();
+	
+	/* match FixedSize conditional */
+	/* special treatment for FixedSize, because more than just
+	   the is_size_fixed flag makes a window unresizable (width and height
+	   hints etc.) */
+	if(IS_SIZE_FIXED(mask) && 
+	   mask->flag_mask.common.s.is_size_fixed && 
+	   is_function_allowed(F_RESIZE,NULL,fw,True,False))
+	{
+	        return False;
+	}
+	if(!IS_SIZE_FIXED(mask) &&
+           mask->flag_mask.common.s.is_size_fixed &&
+	   !is_function_allowed(F_RESIZE,NULL,fw,True,False))
+	{
+	        return False;
+        }
+        SETM_SIZE_FIXED(mask, 0);
+	
+	if(IS_UNICONIFIABLE(mask) &&
+	   mask->flag_mask.common.s.is_uniconifiable &&
+	   is_function_allowed(F_ICONIFY,NULL,fw,True,False))
+	{
+	        return False;
+	}
+	if(!IS_UNICONIFIABLE(mask) &&
+	   mask->flag_mask.common.s.is_uniconifiable &&
+	   !is_function_allowed(F_ICONIFY,NULL,fw,True,False))
+	{
+	        return False;
+	}
+	SETM_IS_UNICONIFIABLE(mask, 0);
+
+	if(IS_UNMAXIMIZABLE(mask) &&
+	   mask->flag_mask.common.s.is_unmaximizable &&
+	   is_function_allowed(F_MAXIMIZE,NULL,fw,True,False))
+	{
+	        return False;
+	}
+	if(!IS_UNMAXIMIZABLE(mask) &&
+	   mask->flag_mask.common.s.is_unmaximizable &&
+	   !is_function_allowed(F_MAXIMIZE,NULL,fw,True,False))
+	{
+	        return False;
+	}
+	SETM_IS_UNMAXIMIZABLE(mask, 0);
+
+	if(IS_UNCLOSABLE(mask) &&
+	   mask->flag_mask.common.s.is_unclosable &&
+	   (is_function_allowed(F_CLOSE,NULL,fw,True,False) ||
+	    is_function_allowed(F_DELETE,NULL,fw,True,False) ||
+	    is_function_allowed(F_DESTROY,NULL,fw,True,False)))
+	{
+	        return False;
+	}
+	if(!IS_UNCLOSABLE(mask) &&
+	   mask->flag_mask.common.s.is_unclosable &&
+	   (!is_function_allowed(F_CLOSE,NULL,fw,True,False) &&
+	    !is_function_allowed(F_DELETE,NULL,fw,True,False) &&
+	    !is_function_allowed(F_DESTROY,NULL,fw,True,False)))
+	{
+	        return False;
+	}
+	SETM_IS_UNCLOSABLE(mask, 0);
 
 	if (!blockcmpmask((char *)&(fw->flags), (char *)&(mask->flags),
 			  (char *)&(mask->flag_mask), sizeof(fw->flags)))
 	{
 		return False;
 	}
+	
 	if (!mask->my_flags.use_circulate_hit && DO_SKIP_CIRCULATE(fw))
 	{
 		return False;
@@ -832,13 +927,9 @@ void CMD_All(F_CMD_ARGS)
 	return;
 }
 
-/**********************************************************************
- * Execute a function to the closest window in the given
- * direction.
- **********************************************************************/
-void CMD_Direction(F_CMD_ARGS)
+static void direction_cmd(F_CMD_ARGS,Bool is_scan)
 {
-	/* The rectangles are inteded for a future enhancement and are not used
+	/* The rectangles are intended for a future enhancement and are not used
 	 * yet. */
 	rectangle my_g;
 	rectangle his_g;
@@ -858,6 +949,8 @@ void CMD_Direction(F_CMD_ARGS)
 	FvwmWindow *window;
 	FvwmWindow *best_window;
 	int dir;
+	int dir2;
+	Bool right_handed=False;
 	char *flags;
 	char *restofline;
 	char *tmp;
@@ -878,11 +971,6 @@ void CMD_Direction(F_CMD_ARGS)
 	{
 		is_pointer_relative = False;
 	}
-	if(!strncmp(tmp,"Cycle",5))
-	{
-		cycle=True;
-		tmp+=5;
-	}
 	dir = gravity_parse_dir_argument(tmp, NULL, -1);
 	if (dir == -1 || dir > DIR_ALL_MASK)
 	{
@@ -894,6 +982,28 @@ void CMD_Direction(F_CMD_ARGS)
 		}
 		return;
 	}
+	if(is_scan)
+	{
+		cycle=True;
+		tmp = PeekToken(action, &action);
+		dir2 = gravity_parse_dir_argument(tmp, NULL, -1);
+		/* if enum direction_type changes, this is trashed. */
+		if (dir2 == -1 || dir2 > DIR_NW ||
+			(dir<4) != (dir2<4) || (abs(dir-dir2)&1)!=1)
+		{
+			fvwm_msg(ERR, "Direction", "Invalid minor direction %s",
+				(tmp) ? tmp : "");
+			if (cond_rc != NULL)
+			{
+				*cond_rc = COND_RC_ERROR;
+			}
+			return;
+		}
+		else if(dir2-dir==1 || dir2-dir== -3)
+		{
+			right_handed=True;
+		}
+	}
 
 	/* Create the mask for flags */
 	flags = CreateFlagString(action, &restofline);
@@ -982,7 +1092,7 @@ void CMD_Direction(F_CMD_ARGS)
 			forward=True;
 		case DIR_N:
 		case DIR_NE:
-			cross = his_cx;
+			cross = -his_cx;
 			offset = (his_cx < 0) ? -his_cx : his_cx;
 			distance = (dir == DIR_N || dir == DIR_NE) ? -his_cy : his_cy;
 			break;
@@ -1005,17 +1115,21 @@ void CMD_Direction(F_CMD_ARGS)
 
 		if(cycle)
 			offset=0;
-		else if (distance <= 0)		/* Target must be in given direction. */
+		else if (distance <= 0)	/* Target must be in given direction. */
 			continue;
 
 		/* Calculate score for this window.  The smaller the better. */
 		score = distance + offset;
 
+		if(!right_handed)
+			cross= -cross;
+
 		if(cycle)
 		{
 			int ordered=(forward == (cross<best_cross));
 
-			if (distance<0 && best_score == -1 && (score < worst_score ||
+			if (distance<0 && best_score == -1 &&
+				(score < worst_score ||
 				(score==worst_score && ordered)))
 			{
 				best_window = window;
@@ -1026,7 +1140,8 @@ void CMD_Direction(F_CMD_ARGS)
 			if(score==0 && forward==(cross<0))
 				continue;
 
-			if (distance>=0 && (best_score == -1 || score < best_score ||
+			if (distance>=0 &&
+				(best_score == -1 || score < best_score ||
 				(score==best_score && ordered)))
 			{
 				best_window = window;
@@ -1069,6 +1184,20 @@ void CMD_Direction(F_CMD_ARGS)
 	return;
 }
 
+/**********************************************************************
+ * Execute a function to the closest window in the given
+ * direction.
+ **********************************************************************/
+void CMD_Direction(F_CMD_ARGS)
+{
+	direction_cmd(F_PASS_ARGS,False);
+}
+
+void CMD_ScanForWindow(F_CMD_ARGS)
+{
+	direction_cmd(F_PASS_ARGS,True);
+}
+
 void CMD_WindowId(F_CMD_ARGS)
 {
 	FvwmWindow *t;
@@ -1300,3 +1429,103 @@ void CMD_NoWindow(F_CMD_ARGS)
 {
 	execute_function_override_window(NULL, exc, action, 0, NULL);
 }
+
+void CMD_On(F_CMD_ARGS)
+{
+	char *restofline;
+	char *flags;
+	char *condition;
+	char *tmp;
+	Bool match, error;
+
+	flags = CreateFlagString(action, &restofline);
+
+	/* Next parse the flags in the string. */
+	tmp = flags;
+	tmp = GetNextSimpleOption(tmp, &condition);
+
+	match = True;
+	error = False;
+	while (condition)
+	{
+		char *cond;
+		Bool on;
+
+		cond = condition;
+		on = 1;
+		if (*cond == '!')
+		{
+			on = 0;
+			cond++;
+		}
+		if (StrEquals(cond, "True"))
+		{
+			match = True;
+		}
+		else if (StrEquals(cond, "Version"))
+		{
+			char *pattern;
+			tmp = GetNextSimpleOption(tmp, &pattern);
+			if (pattern)
+			{
+				match = matchWildcards(pattern, VERSION);
+				free(pattern);
+			}
+			else
+			{
+				error = True;
+			}
+		}
+		else if (StrEquals(cond, "Start"))
+		{
+			match = exc->type == EXCT_INIT ||
+				exc->type == EXCT_RESTART;
+		}
+		else if (StrEquals(cond, "Init"))
+		{
+			match = exc->type == EXCT_INIT;
+		}
+		else if (StrEquals(cond, "Restart"))
+		{
+			match = exc->type == EXCT_RESTART;
+		}
+		else if (StrEquals(cond, "Exit"))
+		{
+			match = exc->type == EXCT_QUIT ||
+				exc->type == EXCT_TORESTART;
+		}
+		else if (StrEquals(cond, "Quit"))
+		{
+			match = exc->type == EXCT_QUIT;
+		}
+		else if (StrEquals(cond, "ToRestart"))
+		{
+			match = exc->type == EXCT_TORESTART;
+		}
+
+		if (!on)
+		{
+			match = !match;
+		}
+		if (error || !match)
+		{
+			break;
+		}
+		free(condition);
+		tmp = GetNextSimpleOption(tmp, &condition);
+	}
+	if (flags != NULL)
+	{
+		free(flags);
+	}
+	if (!error && match)
+	{
+		execute_function(cond_rc, exc, restofline, 0);
+	}
+	if (cond_rc != NULL)
+	{
+		*cond_rc =
+			error ? COND_RC_ERROR :
+			match ? COND_RC_OK : COND_RC_NO_MATCH;
+	}
+}
