@@ -17,8 +17,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_EWMH
-
 #include <stdio.h>
 
 #include <X11/Xlib.h>
@@ -117,7 +115,7 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
     }
     else if (d >= 0)
     {
-      if (IS_STICKY(fwin))
+      if (IS_STICKY_ACROSS_PAGES(fwin) || IS_STICKY_ACROSS_DESKS(fwin))
 	 execute_function_override_window(NULL, NULL, "Stick off", 0, fwin);
       if (fwin->Desk != d)
 	do_move_window_to_desk(fwin, (int)d);
@@ -153,9 +151,12 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
     {
       if ((val[0] == 0xFFFFFFFE || val[0] == 0xFFFFFFFF))
       {
-	S_SET_IS_STICKY(SCF(*style), 1);
-	S_SET_IS_STICKY(SCM(*style), 1);
-	S_SET_IS_STICKY(SCC(*style), 1);
+	S_SET_IS_STICKY_ACROSS_PAGES(SCF(*style), 1);
+	S_SET_IS_STICKY_ACROSS_PAGES(SCM(*style), 1);
+	S_SET_IS_STICKY_ACROSS_PAGES(SCC(*style), 1);
+	S_SET_IS_STICKY_ACROSS_DESKS(SCF(*style), 1);
+	S_SET_IS_STICKY_ACROSS_DESKS(SCM(*style), 1);
+	S_SET_IS_STICKY_ACROSS_DESKS(SCC(*style), 1);
       }
       else if (val[0] < 256)
       {
@@ -816,7 +817,7 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
 	return True;
       return False;
     }
-    return IS_STICKY(fwin);
+    return (IS_STICKY_ACROSS_PAGES(fwin) && IS_STICKY_ACROSS_DESKS(fwin));
   }
 
   if (ev == NULL && style != NULL)
@@ -836,9 +837,12 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
     }
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
-      S_SET_IS_STICKY(SCF(*style), 1);
-      S_SET_IS_STICKY(SCM(*style), 1);
-      S_SET_IS_STICKY(SCC(*style), 1);
+      S_SET_IS_STICKY_ACROSS_PAGES(SCF(*style), 1);
+      S_SET_IS_STICKY_ACROSS_PAGES(SCM(*style), 1);
+      S_SET_IS_STICKY_ACROSS_PAGES(SCC(*style), 1);
+      S_SET_IS_STICKY_ACROSS_DESKS(SCF(*style), 1);
+      S_SET_IS_STICKY_ACROSS_DESKS(SCM(*style), 1);
+      S_SET_IS_STICKY_ACROSS_DESKS(SCC(*style), 1);
     }
     SET_HAS_EWMH_INIT_STICKY_STATE(fwin, EWMH_STATE_HAS_HINT);
     return 0;
@@ -847,7 +851,8 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
   {
     /* client message */
     int bool_arg = ev->xclient.data.l[0];
-    if ((bool_arg == NET_WM_STATE_TOGGLE && !IS_STICKY(fwin)) ||
+    if ((bool_arg == NET_WM_STATE_TOGGLE &&
+	 (!IS_STICKY_ACROSS_PAGES(fwin) || !IS_STICKY_ACROSS_DESKS(fwin))) ||
 	bool_arg == NET_WM_STATE_ADD)
     {
       execute_function_override_window(NULL, NULL, "Stick on", 0, fwin);
@@ -1008,5 +1013,3 @@ void EWMH_ProcessPropertyNotify(const exec_context_t *exc)
   }
 
 }
-
-#endif /* HAVE_EWMH */
