@@ -35,29 +35,21 @@
 #include <stdio.h>
 
 #include <X11/keysym.h>
-#include <X11/Intrinsic.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/Parse.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
-#include "commands.h"
 #include "externs.h"
 #include "cursor.h"
+#include "execcontext.h"
 #include "functions.h"
+#include "commands.h"
 #include "functable.h"
 #include "events.h"
 #include "modconf.h"
 #include "module_interface.h"
 #include "misc.h"
 #include "screen.h"
-#include "geometry.h"
 #include "repeat.h"
-#include "read.h"
-#include "virtual.h"
-#include "colorset.h"
-#include "ewmh.h"
-#include "schedule.h"
 #include "expand.h"
 #include "menus.h"
 
@@ -67,8 +59,6 @@
 
 /* ---------------------------- imports ------------------------------------- */
 
-extern FvwmWindow *Fw;
-
 /* ---------------------------- included code files ------------------------- */
 
 /* ---------------------------- local types --------------------------------- */
@@ -106,7 +96,9 @@ typedef enum
 
 /* ---------------------------- forward declarations ------------------------ */
 
-static void execute_complex_function(F_CMD_ARGS, Bool *desperate);
+static void execute_complex_function(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	Bool *desperate);
 
 /* ---------------------------- local variables ----------------------------- */
 
@@ -114,6 +106,182 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate);
 
 /* ---------------------------- local functions ----------------------------- */
 
+/***********************************************************************
+ *
+ *  Procedure:
+ *      DeferExecution - defer the execution of a function to the
+ *          next button press if the context is C_ROOT
+ *
+ *  Inputs:
+ *      cursor  - the cursor to display while waiting
+ *      finishEvent - ButtonRelease or ButtonPress; tells what kind of event to
+ *                    terminate on.
+ *
+ ***********************************************************************/
+static Bool DeferExecution(
+	exec_context_changes_t *ret_ecc, exec_context_change_mask_t *ret_mask,
+	cursor_t cursor, int trigger_evtype, int FinishEvent,
+	int do_allow_unmanaged)
+{
+	int done;
+	int finished = 0;
+	Window dummy;
+	Window original_w;
+	static XEvent e;
+	Window w;
+	int wcontext;
+	FvwmWindow *fw;
+
+	fw = ret_ecc->w.fw;
+	w = ret_ecc->w.w;
+	original_w = w;
+	wcontext = ret_ecc->w.wcontext;
+	if (wcontext == C_UNMANAGED && do_allow_unmanaged)
+	{
+		return False;
+	}
+	if (wcontext != C_ROOT && wcontext != C_NO_CONTEXT && fw != NULL &&
+	    wcontext != C_EWMH_DESKTOP)
+	{
+		if (FinishEvent == ButtonPress ||
+		    (FinishEvent == ButtonRelease &&
+		     trigger_evtype != ButtonPress))
+		{
+			return False;
+		}
+		else if (FinishEvent == ButtonRelease)
+		{
+			/* We are only waiting until the user releases the
+			 * button. Do not change the cursor. */
+			cursor = CRS_NONE;
+		}
+	}
+	if (Scr.flags.silent_functions)
+	{
+		return True;
+	}
+	if (!GrabEm(cursor, GRAB_NORMAL))
+	{
+		XBell(dpy, 0);
+		return True;
+	}
+	MyXGrabKeyboard(dpy);
+	while (!finished)
+	{
+		done = 0;
+		/* block until there is an event */
+		FMaskEvent(
+			dpy, ButtonPressMask | ButtonReleaseMask |
+			ExposureMask | KeyPressMask | VisibilityChangeMask |
+			ButtonMotionMask | PointerMotionMask
+			/* | EnterWindowMask | LeaveWindowMask*/, &e);
+
+		if (e.type == KeyPress)
+		{
+			KeySym keysym = XLookupKeysym(&e.xkey, 0);
+			if (keysym == XK_Escape)
+			{
+				ret_ecc->x.etrigger = &e;
+				*ret_mask |= ECC_ETRIGGER;
+				UngrabEm(GRAB_NORMAL);
+				MyXUngrabKeyboard(dpy);
+				return True;
+			}
+			Keyboard_shortcuts(&e, NULL, NULL, NULL, FinishEvent);
+		}
+		if (e.type == FinishEvent)
+		{
+			finished = 1;
+		}
+		switch (e.type)
+		{
+		case KeyPress:
+		case ButtonPress:
+			if (e.type != FinishEvent)
+			{
+				original_w = e.xany.window;
+			}
+			done = 1;
+			break;
+		case ButtonRelease:
+			done = 1;
+			break;
+		default:
+			break;
+		}
+		if (!done)
+		{
+			dispatch_event(&e);
+		}
+	}
+	MyXUngrabKeyboard(dpy);
+	UngrabEm(GRAB_NORMAL);
+
+	w = e.xany.window;
+	ret_ecc->x.etrigger = &e;
+	*ret_mask |= ECC_ETRIGGER | ECC_W | ECC_WCONTEXT;
+	if ((w == Scr.Root || w == Scr.NoFocusWin) &&
+	    e.xbutton.subwindow != None)
+	{
+		w = e.xbutton.subwindow;
+		e.xany.window = w;
+	}
+	if (w == Scr.Root || IS_EWMH_DESKTOP(w))
+	{
+		ret_ecc->w.w = w;
+		ret_ecc->w.wcontext = C_ROOT;
+		XBell(dpy, 0);
+		return True;
+	}
+	*ret_mask |= ECC_FW;
+	if (XFindContext(dpy, w, FvwmContext, (caddr_t *)&fw) == XCNOENT)
+	{
+		ret_ecc->w.fw = NULL;
+		ret_ecc->w.w = w;
+		ret_ecc->w.wcontext = C_ROOT;
+		XBell(dpy, 0);
+		return (True);
+	}
+	if (w == FW_W_PARENT(fw))
+	{
+		w = FW_W(fw);
+	}
+	if (original_w == FW_W_PARENT(fw))
+	{
+		original_w = FW_W(fw);
+	}
+	/* this ugly mess attempts to ensure that the release and press
+	 * are in the same window. */
+	if (w != original_w && original_w != Scr.Root &&
+	    original_w != None && original_w != Scr.NoFocusWin &&
+	    !IS_EWMH_DESKTOP(original_w))
+	{
+		if (w != FW_W_FRAME(fw) || original_w != FW_W(fw))
+		{
+			ret_ecc->w.fw = fw;
+			ret_ecc->w.w = w;
+			ret_ecc->w.wcontext = C_ROOT;
+			XBell(dpy, 0);
+			return True;
+		}
+	}
+
+	if (IS_EWMH_DESKTOP(FW_W(fw)))
+	{
+		ret_ecc->w.fw = fw;
+		ret_ecc->w.w = w;
+		ret_ecc->w.wcontext = C_ROOT;
+		XBell(dpy, 0);
+		return True;
+	}
+	wcontext = GetContext(NULL, fw, &e, &dummy);
+	ret_ecc->w.fw = fw;
+	ret_ecc->w.w = w;
+	ret_ecc->w.wcontext = C_ROOT;
+
+	return False;
+}
+
 /* dummies */
 void CMD_Silent(F_CMD_ARGS)
 {
@@ -181,6 +349,294 @@ static const func_type *find_builtin_function(char *func)
 	return ret_func;
 }
 
+static void __execute_function(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags, char *args[])
+{
+	static unsigned int func_depth = 0;
+	Window w;
+	int j;
+	char *function;
+	char *taction;
+	char *trash;
+	char *trash2;
+	char *expaction = NULL;
+	char *arguments[11];
+	const func_type *bif;
+	Bool set_silent;
+	Bool must_free_string = False;
+	Bool must_free_function = False;
+	fvwm_cond_func_rc dummy_rc;
+
+	if (!action)
+	{
+		/* impossibly short command */
+		return;
+	}
+	/* ignore whitespace at the beginning of all config lines */
+	action = SkipSpaces(action, NULL, 0);
+	if (!action || action[0] == 0)
+	{
+		/* impossibly short command */
+		return;
+	}
+	if (action[0] == '#')
+	{
+		/* a comment */
+		return;
+	}
+	if (cond_rc == NULL)
+	{
+		cond_rc = &dummy_rc;
+		memset(cond_rc, 0, sizeof(*cond_rc));
+	}
+
+	func_depth++;
+	if (args)
+	{
+		for (j = 0; j < 11; j++)
+		{
+			arguments[j] = args[j];
+		}
+	}
+	else
+	{
+		for (j = 0; j < 11; j++)
+		{
+			arguments[j] = NULL;
+		}
+	}
+
+	if (exc->w.fw == NULL || IS_EWMH_DESKTOP(FW_W(exc->w.fw)))
+	{
+		if (exec_flags & FUNC_IS_UNMANAGED)
+		{
+			w = exc->w.w;
+		}
+		else
+		{
+			w = Scr.Root;
+		}
+	}
+	else
+	{
+		w = GetSubwindowFromEvent(dpy, exc->x.elast);
+		if (w == None)
+		{
+			w = exc->x.elast->xany.window;
+		}
+		if (w == None)
+		{
+			w = FW_W(exc->w.fw);
+		}
+	}
+
+	set_silent = False;
+	if (action[0] == '-')
+	{
+		exec_flags |= FUNC_DONT_EXPAND_COMMAND;
+		action++;
+	}
+
+	taction = action;
+	/* parse prefixes */
+	trash = PeekToken(taction, &trash2);
+	while (trash)
+	{
+		if (StrEquals(trash, PRE_SILENT))
+		{
+			if (Scr.flags.silent_functions == 0)
+			{
+				set_silent = 1;
+				Scr.flags.silent_functions = 1;
+			}
+			taction = trash2;
+			trash = PeekToken(taction, &trash2);
+		}
+		else
+		{
+			break;
+		}
+	}
+	if (taction == NULL)
+	{
+		if (set_silent)
+		{
+			Scr.flags.silent_functions = 0;
+		}
+		func_depth--;
+		return;
+	}
+
+	function = PeekToken(taction, NULL);
+	if (function)
+	{
+		function = expand_vars(
+			function, arguments, exc->w.fw, False, False, cond_rc);
+	}
+	if (function && function[0] != '*')
+	{
+#if 1
+		/* DV: with this piece of code it is impossible to have a
+		 * complex function with embedded whitespace that begins with a
+		 * builtin function name, e.g. a function "echo hello". */
+		/* DV: ... and without it some of the complex functions will
+		 * fail */
+		char *tmp = function;
+
+		while (*tmp && !isspace(*tmp))
+		{
+			tmp++;
+		}
+		*tmp = 0;
+#endif
+		bif = find_builtin_function(function);
+		must_free_function = True;
+	}
+	else
+	{
+		bif = NULL;
+		if (function)
+		{
+			free(function);
+		}
+		function = "";
+	}
+
+#ifdef USEDECOR
+	if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor &&
+	    (!bif || !(bif->flags & FUNC_DECOR)))
+	{
+		fvwm_msg(
+			ERR, "__execute_function",
+			"Command can not be added to a decor; executing"
+			" command now: '%s'", action);
+	}
+#endif
+
+	if (!(exec_flags & FUNC_DONT_EXPAND_COMMAND))
+	{
+		expaction = expand_vars(
+			taction, arguments, exc->w.fw, (bif) ?
+			!!(bif->flags & FUNC_ADD_TO) :
+			False, (taction[0] == '*'), cond_rc);
+		if (func_depth <= 1)
+		{
+			must_free_string = set_repeat_data(
+				expaction, REPEAT_COMMAND, bif);
+		}
+		else
+		{
+			must_free_string = True;
+		}
+	}
+	else
+	{
+		expaction = taction;
+	}
+
+#ifdef FVWM_COMMAND_LOG
+	fvwm_msg(INFO, "LOG", "%c: %s", (char)exc->type, expaction);
+#endif
+
+	/* Note: the module config command, "*" can not be handled by the
+	 * regular command table because there is no required white space after
+	 * the asterisk. */
+	if (expaction[0] == '*')
+	{
+#ifdef USEDECOR
+		if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
+		{
+			fvwm_msg(
+				WARN, "__execute_function",
+				"Command can not be added to a decor;"
+				" executing command now: '%s'", expaction);
+		}
+#endif
+		/* process a module config command */
+		ModuleConfig(expaction);
+	}
+	else
+	{
+		const exec_context_t *exc2;
+		exec_context_changes_t ecc;
+		exec_context_change_mask_t mask;
+
+		mask = (w != exc->w.w) ? ECC_W : 0;
+		ecc.w.fw = exc->w.fw;
+		ecc.w.w = w;
+		ecc.w.wcontext = exc->w.wcontext;
+		if (bif && bif->func_type != F_FUNCTION)
+		{
+			char *runaction;
+			Bool rc = False;
+
+			runaction = SkipNTokens(expaction, 1);
+			if ((bif->flags & FUNC_NEEDS_WINDOW) &&
+			    !(exec_flags & FUNC_DONT_DEFER))
+			{
+				rc = DeferExecution(
+					&ecc, &mask, bif->cursor,
+					exc->x.elast->type, bif->evtype,
+					(bif->flags & FUNC_ALLOW_UNMANAGED));
+			}
+			if (rc == False)
+			{
+				exc2 = exc_clone_context(exc, &ecc, mask);
+				bif->action(cond_rc, exc2, runaction);
+				exc_destroy_context(exc2);
+			}
+		}
+		else
+		{
+			Bool desperate = 1;
+			char *runaction;
+
+			if (bif)
+			{
+				/* strip "function" command */
+				runaction = SkipNTokens(expaction, 1);
+			}
+			else
+			{
+				runaction = expaction;
+			}
+			exc2 = exc_clone_context(exc, &ecc, mask);
+			execute_complex_function(
+				cond_rc, exc2, runaction, &desperate);
+			if (!bif && desperate)
+			{
+				if (executeModuleDesperate(
+					    cond_rc, exc, runaction) == -1 &&
+				    *function != 0 && !set_silent)
+				{
+					fvwm_msg(
+						ERR, "__execute_function",
+						"No such command '%s'",
+						function);
+				}
+			}
+			exc_destroy_context(exc2);
+		}
+	}
+
+	if (set_silent)
+	{
+		Scr.flags.silent_functions = 0;
+	}
+	if (must_free_string)
+	{
+		free(expaction);
+	}
+	if (must_free_function)
+	{
+		free(function);
+	}
+	func_depth--;
+
+	return;
+}
+
 /* find_complex_function expects a token as the input. Make sure you have used
  * GetNextToken before passing a function name to remove quotes */
 static FvwmFunction *find_complex_function(const char *function_name)
@@ -222,17 +678,15 @@ static cfunc_action_type CheckActionType(
 	int xcurrent,ycurrent,total = 0;
 	Time t0;
 	int dist;
-	XEvent old_event;
-	extern Time lastTimestamp;
 	Bool do_sleep = False;
 
 	xcurrent = x;
 	ycurrent = y;
-	t0 = lastTimestamp;
+	t0 = fev_get_evtime();
 	dist = Scr.MoveThreshold;
 
-	while ((total < Scr.ClickTime && lastTimestamp - t0 < Scr.ClickTime) ||
-	       !may_time_out)
+	while ((total < Scr.ClickTime &&
+		fev_get_evtime() - t0 < Scr.ClickTime) || !may_time_out)
 	{
 		if (!(x - xcurrent <= dist && xcurrent - x <= dist &&
 		      y - ycurrent <= dist && ycurrent - y <= dist))
@@ -250,12 +704,11 @@ static cfunc_action_type CheckActionType(
 			do_sleep = 1;
 		}
 		total += 20;
-		if (XCheckMaskEvent(
+		if (FCheckMaskEvent(
 			    dpy, ButtonReleaseMask|ButtonMotionMask|
 			    PointerMotionMask|ButtonPressMask|ExposureMask, d))
 		{
 			do_sleep = 0;
-			StashEventTime(d);
 			switch (d->xany.type)
 			{
 			case ButtonRelease:
@@ -286,12 +739,7 @@ static cfunc_action_type CheckActionType(
 			case Expose:
 				/* must handle expose here so that raising a
 				 * window with "I" works */
-				memcpy(&old_event, &Event, sizeof(XEvent));
-				memcpy(&Event, d, sizeof(XEvent));
-				/* note: handling Expose events never modifies
-				 * the global Fw */
-				DispatchEvent(True);
-				memcpy(&Event, &old_event, sizeof(XEvent));
+				dispatch_event(d);
 				break;
 			default:
 				/* can't happen */
@@ -303,7 +751,35 @@ static cfunc_action_type CheckActionType(
 	return (is_button_pressed) ? CF_HOLD : CF_TIMEOUT;
 }
 
-static void cf_cleanup(unsigned int *depth, char **arguments)
+static void __run_complex_function_items(
+	fvwm_cond_func_rc *cond_func_rc, char cond, FvwmFunction *func,
+	const exec_context_t *exc, char *args[])
+{
+	char c;
+	FunctionItem *fi;
+
+	for (fi = func->first_item; fi != NULL && *cond_func_rc != COND_RC_BREAK; )
+	{
+		/* make lower case */
+		c = fi->condition;
+		if (isupper(c))
+		{
+			c = tolower(c);
+		}
+		if (c == cond)
+		{
+			__execute_function(
+				cond_func_rc, exc, fi->action, FUNC_DONT_DEFER,
+				args);
+		}
+		fi = fi->next_item;
+	}
+
+	return;
+}
+
+static void cf_cleanup(
+	unsigned int *depth, char **arguments)
 {
 	int i;
 
@@ -323,7 +799,9 @@ static void cf_cleanup(unsigned int *depth, char **arguments)
 	return;
 }
 
-static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
+static void execute_complex_function(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	Bool *desperate)
 {
 	fvwm_cond_func_rc cond_func_rc = COND_RC_OK;
 	cfunc_action_type type = CF_MOTION;
@@ -334,13 +812,24 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 	Bool HaveHold = False;
 	Bool NeedsTarget = False;
 	Bool ImmediateNeedsTarget = False;
+	int do_allow_unmanaged = FUNC_ALLOW_UNMANAGED;
+	int do_allow_unmanaged_immediate = FUNC_ALLOW_UNMANAGED;
 	char *arguments[11], *taction;
-	char* func_name;
+	char *func_name;
 	int x, y ,i;
-	XEvent d, *ev;
+	XEvent d;
 	FvwmFunction *func;
 	static unsigned int depth = 0;
+	const exec_context_t *exc2;
+	exec_context_changes_t ecc;
+	exec_context_change_mask_t mask;
+	int trigger_evtype;
+	XEvent *te;
 
+	mask = 0;
+	ecc.w.fw = exc->w.fw;
+	ecc.w.w = exc->w.w;
+	ecc.w.wcontext = exc->w.wcontext;
 	/* find_complex_function expects a token, not just a quoted string */
 	func_name = PeekToken(action, &taction);
 	if (!func_name)
@@ -379,26 +868,28 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 			}
 		}
 		/* Get the argument list */
-		for (i=1;i<11;i++)
+		for (i = 1; i < 11; i++)
 		{
-			taction = GetNextToken(taction,&arguments[i]);
+			taction = GetNextToken(taction, &arguments[i]);
 		}
 	}
 	else
 	{
-		for (i=0;i<11;i++)
+		for (i = 0; i < 11; i++)
 		{
 			arguments[i] = NULL;
 		}
 	}
-	/* see functions.c to find out which functions need a window to operate
-	 * on */
-	ev = eventp;
 	/* In case we want to perform an action on a button press, we
 	 * need to fool other routines */
-	if (eventp->type == ButtonPress)
+	te = exc->x.elast;
+	if (te->type == ButtonPress)
 	{
-		eventp->type = ButtonRelease;
+		trigger_evtype = ButtonRelease;
+	}
+	else
+	{
+		trigger_evtype = te->type;
 	}
 	func->use_depth++;
 
@@ -407,8 +898,10 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 		if (fi->flags & FUNC_NEEDS_WINDOW)
 		{
 			NeedsTarget = True;
+			do_allow_unmanaged &= fi->flags;
 			if (fi->condition == CF_IMMEDIATE)
 			{
+				do_allow_unmanaged_immediate &= fi->flags;
 				ImmediateNeedsTarget = True;
 				break;
 			}
@@ -418,7 +911,8 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 	if (ImmediateNeedsTarget)
 	{
 		if (DeferExecution(
-			    eventp, &w, &fw, &context, CRS_SELECT, ButtonPress))
+			    &ecc, &mask, CRS_SELECT, trigger_evtype,
+			    ButtonPress, do_allow_unmanaged_immediate))
 		{
 			func->use_depth--;
 			cf_cleanup(&depth, arguments);
@@ -426,6 +920,11 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 		}
 		NeedsTarget = False;
 	}
+	else
+	{
+		ecc.w.w = (ecc.w.fw) ? FW_W_FRAME(ecc.w.fw) : None;
+		mask |= ECC_W;
+	}
 
 	/* we have to grab buttons before executing immediate actions because
 	 * these actions can move the window away from the pointer so that a
@@ -437,6 +936,10 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 		cf_cleanup(&depth, arguments);
 		return;
 	}
+	exc2 = exc_clone_context(exc, &ecc, mask);
+	__run_complex_function_items(
+		&cond_func_rc, CF_IMMEDIATE, func, exc2, arguments);
+	exc_destroy_context(exc2);
 	for (fi = func->first_item; fi != NULL && cond_func_rc != COND_RC_BREAK;
 	     fi = fi->next_item)
 	{
@@ -445,17 +948,6 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 		switch (c)
 		{
 		case CF_IMMEDIATE:
-			if (fw)
-			{
-				w = FW_W_FRAME(fw);
-			}
-			else
-			{
-				w = None;
-			}
-			old_execute_function(
-				&cond_func_rc, fi->action, fw, eventp, context,
-				-1, 0, arguments);
 			break;
 		case CF_DOUBLE_CLICK:
 			HaveDoubleClick = True;
@@ -484,7 +976,8 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 	if (NeedsTarget)
 	{
 		if (DeferExecution(
-			    eventp, &w, &fw, &context, CRS_SELECT, ButtonPress))
+			    &ecc, &mask, CRS_SELECT, trigger_evtype,
+			    ButtonPress, do_allow_unmanaged))
 		{
 			func->use_depth--;
 			cf_cleanup(&depth, arguments);
@@ -493,18 +986,19 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 		}
 	}
 
-	switch (eventp->xany.type)
+	te = (mask & ECC_ETRIGGER) ? ecc.x.etrigger : exc->x.elast;
+	switch (te->xany.type)
 	{
 	case ButtonPress:
 	case ButtonRelease:
-		x = eventp->xbutton.x_root;
-		y = eventp->xbutton.y_root;
+		x = te->xbutton.x_root;
+		y = te->xbutton.y_root;
 		/* Take the click which started this fuction off the
 		 * Event queue.  -DDN- Dan D Niles dniles@iname.com */
-		XCheckMaskEvent(dpy, ButtonPressMask, &d);
+		FCheckMaskEvent(dpy, ButtonPressMask, &d);
 		break;
 	default:
-		if (XQueryPointer(
+		if (FQueryPointer(
 			    dpy, Scr.Root, &JunkRoot, &JunkChild, &x, &y,
 			    &JunkX, &JunkY, &JunkMask) == False)
 		{
@@ -520,7 +1014,6 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 	type = CheckActionType(x, y, &d, HaveHold, True);
 	if (type == CF_CLICK)
 	{
-		ev = &d;
 		/* If it was a click, wait to see if its a double click */
 		if (HaveDoubleClick)
 		{
@@ -531,7 +1024,6 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 			case CF_HOLD:
 			case CF_MOTION:
 				type = CF_DOUBLE_CLICK;
-				ev = &d;
 				break;
 			case CF_TIMEOUT:
 				type = CF_CLICK;
@@ -547,14 +1039,13 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 		type = CF_HOLD;
 	}
 
-	/* some functions operate on button release instead of
-	 * presses. These gets really weird for complex functions ... */
-	if (ev->type == ButtonPress)
+	/* some functions operate on button release instead of presses. These
+	 * gets really weird for complex functions ... */
+	if (d.type == ButtonPress)
 	{
-		ev->type = ButtonRelease;
+		d.type = ButtonRelease;
 	}
 
-	fi = func->first_item;
 #ifdef BUGGY_CODE
 	/* domivogt (11-Apr-2000): The pointer ***must not*** be ungrabbed
 	 * here.  If it is, any window that the mouse enters during the
@@ -563,30 +1054,15 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
 	 * ButtonMotion as user input to select text. */
 	UngrabEm(GRAB_NORMAL);
 #endif
-	while (fi != NULL)
-	{
-		/* make lower case */
-		c = fi->condition;
-		if (isupper(c))
-		{
-			c=tolower(c);
-		}
-		if (c == type)
-		{
-			if (fw)
-			{
-				w = FW_W_FRAME(fw);
-			}
-			else
-			{
-				w = None;
-			}
-			old_execute_function(
-				&cond_func_rc, fi->action, fw, ev, context, -1,
-				0, arguments);
-		}
-		fi = fi->next_item;
-	}
+	fev_set_evpos(&d, x, y);
+	fev_fake_event(&d);
+	ecc.x.etrigger = &d;
+	ecc.w.w = (ecc.w.fw) ? FW_W_FRAME(ecc.w.fw) : None;
+	mask |= ECC_ETRIGGER | ECC_W;
+	exc2 = exc_clone_context(exc, &ecc, mask);
+	__run_complex_function_items(
+		&cond_func_rc, type, func, exc2, arguments);
+	exc_destroy_context(exc2);
 	/* This is the right place to ungrab the pointer (see comment above). */
 	func->use_depth--;
 	cf_cleanup(&depth, arguments);
@@ -693,476 +1169,65 @@ Bool functions_is_complex_function(const char *function_name)
 	return False;
 }
 
-/***********************************************************************
- *
- *  Procedure:
- *      execute_function - execute a fvwm built in function
- *
- *  Inputs:
- *      Action  - the action to execute
- *      fw      - the fvwm window structure
- *      eventp  - pointer to the event that caused the function
- *      context - the context in which the button was pressed
- *
- ***********************************************************************/
-void execute_function(exec_func_args_type *efa)
+void execute_function(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags)
 {
-	static unsigned int func_depth = 0;
-	FvwmWindow *s_Fw = Fw;
-	Window w;
-	int j;
-	char *function;
-	char *taction;
-	char *trash;
-	char *trash2;
-	char *expaction = NULL;
-	char *arguments[11];
-	const func_type *bif;
-	Bool set_silent;
-	Bool must_free_string = False;
-	Bool must_free_function = False;
-
-	if (!efa->action)
-	{
-		/* impossibly short command */
-		return;
-	}
-	/* ignore whitespace at the beginning of all config lines */
-	efa->action = SkipSpaces(efa->action, NULL, 0);
-	if (!efa->action || efa->action[0] == 0)
-	{
-		/* impossibly short command */
-		return;
-	}
-	if (efa->action[0] == '#')
-	{
-		/* a comment */
-		return;
-	}
-
-	func_depth++;
-	if (efa->args)
-	{
-		for (j=0;j<11;j++)
-		{
-			arguments[j] = efa->args[j];
-		}
-	}
-	else
-	{
-		for (j=0;j<11;j++)
-		{
-			arguments[j] = NULL;
-		}
-	}
-
-	if (efa->fw == NULL || IS_EWMH_DESKTOP(FW_W(efa->fw)))
-	{
-		if (efa->flags.is_window_unmanaged)
-		{
-			w = efa->win;
-		}
-		else
-		{
-			w = Scr.Root;
-		}
-	}
-	else
-	{
-		if (efa->eventp)
-		{
-			w = GetSubwindowFromEvent(dpy, efa->eventp);
-			if (!w)
-			{
-				w = efa->eventp->xany.window;
-			}
-		}
-		else
-		{
-			w = FW_W(efa->fw);
-		}
-	}
-
-	set_silent = False;
-	if (efa->action[0] == '-')
-	{
-		efa->flags.exec |= FUNC_DONT_EXPAND_COMMAND;
-		efa->action++;
-	}
-
-	taction = efa->action;
-	/* parse prefixes */
-	trash = PeekToken(taction, &trash2);
-	while (trash)
-	{
-		if (StrEquals(trash, PRE_SILENT))
-		{
-			if (Scr.flags.silent_functions == 0)
-			{
-				set_silent = 1;
-				Scr.flags.silent_functions = 1;
-			}
-			taction = trash2;
-			trash = PeekToken(taction, &trash2);
-		}
-		else
-			break;
-	}
-	if (taction == NULL)
-	{
-		if (set_silent)
-			Scr.flags.silent_functions = 0;
-		func_depth--;
-		return;
-	}
-
-	function = PeekToken(taction, NULL);
-	if (function)
-	{
-		function = expand_vars(
-			function, arguments, efa->fw, False, False,
-			efa->cond_rc);
-	}
-	if (function && function[0] != '*')
-	{
-#if 1
-		/* DV: with this piece of code it is impossible to have a
-		 * complex function with embedded whitespace that begins with a
-		 * builtin function name, e.g. a function "echo hello". */
-		/* DV: ... and without it some of the complex functions will
-		 * fail */
-		char *tmp = function;
-
-		while (*tmp && !isspace(*tmp))
-		{
-			tmp++;
-		}
-		*tmp = 0;
-#endif
-		bif = find_builtin_function(function);
-		must_free_function = True;
-	}
-	else
-	{
-		bif = NULL;
-		if (function)
-		{
-			free(function);
-		}
-		function = "";
-	}
-
-#ifdef USEDECOR
-	if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor &&
-	    (!bif || !(bif->flags & FUNC_DECOR)))
-	{
-		fvwm_msg(
-			ERR, "execute_function",
-			"Command can not be added to a decor; executing"
-			" command now: '%s'", efa->action);
-	}
-#endif
-
-	if (!(efa->flags.exec & FUNC_DONT_EXPAND_COMMAND))
-	{
-		expaction = expand_vars(
-			taction, arguments, efa->fw, (bif) ?
-			!!(bif->flags & FUNC_ADD_TO) :
-			False, (taction[0] == '*'), efa->cond_rc);
-		if (func_depth <= 1)
-		{
-			must_free_string = set_repeat_data(
-				expaction, REPEAT_COMMAND, bif);
-		}
-		else
-		{
-			must_free_string = True;
-		}
-	}
-	else
-	{
-		expaction = taction;
-	}
-
-#ifdef FVWM_COMMAND_LOG
-	fvwm_msg(INFO, "LOG", "%s", expaction);
-#endif
-
-	/* Note: the module config command, "*" can not be handled by the
-	 * regular command table because there is no required white space after
-	 * the asterisk. */
-	if (expaction[0] == '*')
-	{
-#ifdef USEDECOR
-		if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
-		{
-			fvwm_msg(
-				WARN, "execute_function",
-				"Command can not be added to a decor;"
-				" executing command now: '%s'", expaction);
-		}
-#endif
-		/* process a module config command */
-		ModuleConfig(expaction);
-	}
-	else
-	{
-		if (bif && bif->func_type != F_FUNCTION)
-		{
-			char *runaction;
-
-			runaction = SkipNTokens(expaction, 1);
-			bif->action(
-				efa->cond_rc, efa->eventp, w, efa->fw,
-				efa->context, runaction, &efa->module);
-		}
-		else
-		{
-			Bool desperate = 1;
-			char *runaction;
-
-			if (bif)
-			{
-				/* strip "function" command */
-				runaction = SkipNTokens(expaction, 1);
-			}
-			else
-			{
-				runaction = expaction;
-			}
-
-			execute_complex_function(
-				efa->cond_rc, efa->eventp, w, efa->fw,
-				efa->context, runaction, &efa->module,
-				&desperate);
-			if (!bif && desperate)
-			{
-				if (executeModuleDesperate(
-					    efa->cond_rc, efa->eventp, w,
-					    efa->fw, efa->context, runaction,
-					    &efa->module) == -1 &&
-				    *function != 0 && !set_silent)
-				{
-					fvwm_msg(
-						ERR, "execute_function",
-						"No such command '%s'",
-						function);
-				}
-			}
-		}
-	}
-
-	if (set_silent)
-	{
-		Scr.flags.silent_functions = 0;
-	}
-
-	if (must_free_string)
-	{
-		free(expaction);
-	}
-	if (must_free_function)
-	{
-		free(function);
-	}
-	if (efa->flags.do_save_tmpwin)
-	{
-		if (check_if_fvwm_window_exists(s_Fw))
-		{
-			Fw = s_Fw;
-		}
-		else
-		{
-			Fw = NULL;
-		}
-	}
-	func_depth--;
+	__execute_function(cond_rc, exc, action, exec_flags, NULL);
 
 	return;
 }
 
-void old_execute_function(
-	fvwm_cond_func_rc *cond_rc, char *action, FvwmWindow *fw,
-	XEvent *eventp, unsigned long context, int Module,
-	FUNC_FLAGS_TYPE exec_flags, char *args[])
+void execute_function_override_wcontext(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags, int wcontext)
 {
-	exec_func_args_type efa;
+	const exec_context_t *exc2;
+	exec_context_changes_t ecc;
 
-	memset(&efa, 0, sizeof(efa));
-	efa.cond_rc = cond_rc;
-	efa.eventp = eventp;
-	efa.fw = fw;
-	efa.action = action;
-	efa.args = args;
-	efa.context = context;
-	efa.module = Module;
-	efa.flags.exec = exec_flags;
-	execute_function(&efa);
+	ecc.w.wcontext = wcontext;
+	exc2 = exc_clone_context(exc, &ecc, ECC_WCONTEXT);
+	execute_function(cond_rc, exc2, action, exec_flags);
+	exc_destroy_context(exc2);
 
 	return;
 }
 
-
-/***********************************************************************
- *
- *  Procedure:
- *      DeferExecution - defer the execution of a function to the
- *          next button press if the context is C_ROOT
- *
- *  Inputs:
- *      eventp  - pointer to XEvent to patch up
- *      w       - pointer to Window to patch up
- *      fw - pointer to FvwmWindow Structure to patch up
- *      context - the context in which the mouse button was pressed
- *      func    - the function to defer
- *      cursor  - the cursor to display while waiting
- *      finishEvent - ButtonRelease or ButtonPress; tells what kind of event to
- *                    terminate on.
- *
- ***********************************************************************/
-int DeferExecution(
-	XEvent *eventp, Window *w, FvwmWindow **fw, unsigned long *context,
-	cursor_t cursor, int FinishEvent)
+void execute_function_override_window(
+	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action,
+	FUNC_FLAGS_TYPE exec_flags, FvwmWindow *fw)
 {
-	int done;
-	int finished = 0;
-	Window dummy;
-	Window original_w;
+	const exec_context_t *exc2;
+	exec_context_changes_t ecc;
 
-	original_w = *w;
+	ecc.w.fw = fw;
+	if (fw != NULL)
+	{
+		ecc.w.w = FW_W(fw);
+		ecc.w.wcontext = C_WINDOW;
+	}
+	else
+	{
+		ecc.w.w = None;
+		ecc.w.wcontext = C_ROOT;
+	}
+	if (exc != NULL)
+	{
+		exc2 = exc_clone_context(
+			exc, &ecc, ECC_FW | ECC_W | ECC_WCONTEXT);
+	}
+	else
+	{
+		ecc.type = EXCT_NULL;
+		exc2 = exc_create_context(
+			&ecc, ECC_TYPE | ECC_FW | ECC_W | ECC_WCONTEXT);
+	}
+	execute_function(cond_rc, exc2, action, exec_flags);
+	exc_destroy_context(exc2);
 
-	if (*context != C_ROOT && *context != C_NO_CONTEXT && *fw != NULL &&
-	    *context != C_EWMH_DESKTOP)
-	{
-		if (FinishEvent == ButtonPress ||
-		    (FinishEvent == ButtonRelease &&
-		     eventp->type != ButtonPress))
-		{
-			return False;
-		}
-		else if (FinishEvent == ButtonRelease)
-		{
-			/* We are only waiting until the user releases the
-			 * button. Do not change the cursor. */
-			cursor = CRS_NONE;
-		}
-	}
-	if (Scr.flags.silent_functions)
-	{
-		return True;
-	}
-	if (!GrabEm(cursor, GRAB_NORMAL))
-	{
-		XBell(dpy, 0);
-		return True;
-	}
-
-	MyXGrabKeyboard(dpy);
-	while (!finished)
-	{
-		done = 0;
-		/* block until there is an event */
-		XMaskEvent(
-			dpy, ButtonPressMask | ButtonReleaseMask |
-			ExposureMask | KeyPressMask | VisibilityChangeMask |
-			ButtonMotionMask | PointerMotionMask
-			/* | EnterWindowMask | LeaveWindowMask*/, eventp);
-		StashEventTime(eventp);
-
-		if (eventp->type == KeyPress)
-		{
-			KeySym keysym = XLookupKeysym(&eventp->xkey,0);
-			if (keysym == XK_Escape)
-			{
-				UngrabEm(GRAB_NORMAL);
-				MyXUngrabKeyboard(dpy);
-				return True;
-			}
-			Keyboard_shortcuts(
-				eventp, NULL, NULL, NULL, FinishEvent);
-		}
-		if (eventp->type == FinishEvent)
-		{
-			finished = 1;
-		}
-		switch (eventp->type)
-		{
-		case KeyPress:
-		case ButtonPress:
-			if (eventp->type != FinishEvent)
-			{
-				original_w = eventp->xany.window;
-			}
-			done = 1;
-			break;
-		case ButtonRelease:
-			done = 1;
-			break;
-		default:
-			break;
-		}
-		if (!done)
-		{
-			DispatchEvent(False);
-		}
-	}
-
-	MyXUngrabKeyboard(dpy);
-	UngrabEm(GRAB_NORMAL);
-	*w = eventp->xany.window;
-	if ((*w == Scr.Root || *w == Scr.NoFocusWin) &&
-	    eventp->xbutton.subwindow != None)
-	{
-		*w = eventp->xbutton.subwindow;
-		eventp->xany.window = *w;
-	}
-	if (*w == Scr.Root || IS_EWMH_DESKTOP(*w))
-	{
-		*context = C_ROOT;
-		XBell(dpy, 0);
-		return True;
-	}
-	if (XFindContext(dpy, *w, FvwmContext, (caddr_t *)fw) == XCNOENT)
-	{
-		*fw = NULL;
-		XBell(dpy, 0);
-		return (True);
-	}
-	if (*w == FW_W_PARENT(*fw))
-	{
-		*w = FW_W(*fw);
-	}
-	if (original_w == FW_W_PARENT(*fw))
-	{
-		original_w = FW_W(*fw);
-	}
-	/* this ugly mess attempts to ensure that the release and press
-	 * are in the same window. */
-	if (*w != original_w && original_w != Scr.Root &&
-	    original_w != None && original_w != Scr.NoFocusWin &&
-	    !IS_EWMH_DESKTOP(original_w))
-	{
-		if (*w != FW_W_FRAME(*fw) || original_w != FW_W(*fw))
-		{
-			*context = C_ROOT;
-			XBell(dpy, 0);
-			return True;
-		}
-	}
-
-	if (IS_EWMH_DESKTOP(FW_W(*fw)))
-	{
-		*context = C_ROOT;
-		XBell(dpy, 0);
-		return True;
-	}
-
-	*context = GetContext(*fw,eventp,&dummy);
-
-	return False;
+	return;
 }
 
-
 void find_func_type(char *action, short *func_type, unsigned char *flags)
 {
 	int j, len = 0;
@@ -1370,7 +1435,7 @@ void CMD_Plus(F_CMD_ARGS)
 		{
 			return;
 		}
-		AddToDecor(tmp, action);
+		AddToDecor(F_PASS_ARGS, tmp);
 	}
 #endif /* USEDECOR */
 
