@@ -229,7 +229,8 @@ typedef struct
 typedef struct
 {
 	/* common flags (former flags in bits 0-12) */
-	unsigned is_sticky : 1;
+	unsigned is_sticky_across_pages : 1;
+	unsigned is_sticky_across_desks : 1;
 	unsigned has_icon_font : 1;
 	unsigned has_no_border : 1;
 	unsigned has_window_font : 1;
@@ -275,7 +276,12 @@ typedef struct
 		unsigned is_bottom_title_rotated : 1;
 		unsigned is_fixed : 1;
 		unsigned is_fixed_ppos : 1;
-		unsigned is_icon_sticky : 1;
+	        unsigned is_uniconifiable : 1;
+	        unsigned is_unmaximizable : 1;
+	        unsigned is_unclosable : 1;
+	        unsigned is_maximize_fixed_size_disallowed : 1;
+		unsigned is_icon_sticky_across_pages : 1;
+		unsigned is_icon_sticky_across_desks : 1;
 		unsigned is_icon_suppressed : 1;
 		unsigned is_left_title_rotated_cw : 1; /* cw = clock wise */
 		unsigned is_lenient : 1;
@@ -291,6 +297,7 @@ typedef struct
 #define WINDOWSHADE_BUSY          2
 #define WINDOWSHADE_LAZY_MASK   0x3
 		unsigned windowshade_laziness : 2;
+		unsigned use_title_decor_rotation : 1;
 		focus_policy_t focus_policy;
 	} s;
 } common_flags_type;
@@ -492,6 +499,9 @@ typedef struct
 	unsigned has_icon_boxes : 1;
 	unsigned has_icon_size_limits : 1;
 	unsigned has_max_window_size : 1;
+	unsigned has_icon_background_padding : 1;
+	unsigned has_icon_background_relief : 1;
+	unsigned has_icon_title_relief : 1;
 	unsigned has_window_shade_steps : 1;
 	unsigned has_mini_icon : 1;
 	unsigned has_mwm_decor : 1;
@@ -513,6 +523,9 @@ typedef struct
 	unsigned use_colorset_hi : 1;
 	unsigned use_border_colorset : 1;
 	unsigned use_border_colorset_hi : 1;
+	unsigned use_icon_title_colorset : 1;
+	unsigned use_icon_title_colorset_hi : 1;
+	unsigned use_icon_background_colorset : 1;
 	unsigned use_layer : 1;
 	unsigned use_no_pposition : 1;
 	unsigned use_no_usposition : 1;
@@ -546,6 +559,15 @@ typedef struct window_style
 	unsigned char max_icon_width;
 	unsigned char min_icon_height;
 	unsigned char max_icon_height;
+#define ICON_RESIZE_TYPE_NONE      0x0
+#define ICON_RESIZE_TYPE_STRETCHED 0x1
+#define ICON_RESIZE_TYPE_ADJUSTED  0x2
+#define ICON_RESIZE_TYPE_SHRUNK    0x3
+#define ICON_RESIZE_TYPE_MASK      0x3
+	unsigned icon_resize_type : 2;
+	unsigned char icon_background_padding;
+	char icon_background_relief;
+	char icon_title_relief;
 	char *icon_font;
 	char *window_font;
 	char *fore_color_name;
@@ -556,6 +578,9 @@ typedef struct window_style
 	int colorset_hi;
 	int border_colorset;
 	int border_colorset_hi;
+	int icon_title_colorset;
+	int icon_title_colorset_hi;
+	int icon_background_colorset;
 	short border_width;
 	/* resize handle width */
 	short handle_width;
@@ -676,6 +701,9 @@ typedef struct FvwmWindow
 	/* pixmap for the icon mask */
 	Pixmap icon_maskPixmap;
 	Pixmap icon_alphaPixmap;
+	int icon_nalloc_pixels;
+	Pixel *icon_alloc_pixels;
+	int icon_no_limit;
 	FlocaleFont *icon_font;
 
 	/* some parts of the window attributes */
@@ -734,6 +762,9 @@ typedef struct FvwmWindow
 	int cs_hi;
 	int border_cs;
 	int border_cs_hi;
+	int icon_title_cs;
+	int icon_title_cs_hi;
+	int icon_background_cs;
 
 	unsigned long buttons;
 	/* zero or more iconboxes */
@@ -743,10 +774,17 @@ typedef struct FvwmWindow
 	/* Note: if the type of this variable is changed, do update the
 	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too! */
 	int layer;
+
 	unsigned char min_icon_width;
 	unsigned char max_icon_width;
 	unsigned char min_icon_height;
 	unsigned char max_icon_height;
+	unsigned short icon_resize_type;
+
+	unsigned char icon_background_padding;
+	char icon_background_relief;
+	char icon_title_relief;
+
 	int max_window_width;
 	int max_window_height;
 	int shade_anim_steps;
