@@ -11,8 +11,6 @@
 /*********************************************************/
 #include "config.h"
 
-#ifdef GNOME
-
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -498,7 +496,7 @@ GNOME_SetHints(FvwmWindow *fwin)
 	atom_set = XInternAtom(dpy, XA_WIN_STATE, False);
 	val = 0;
 
-	if (IS_STICKY_ON_PAGE(fwin) && IS_STICKY_ON_DESK(fwin))
+	if (IS_STICKY_ACROSS_PAGES(fwin) && IS_STICKY_ACROSS_DESKS(fwin))
 	{
 		val |= WIN_STATE_STICKY;
 	}
@@ -567,12 +565,12 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
 	{
 		if (*(int*)retval & WIN_STATE_STICKY)
 		{
-			S_SET_IS_STICKY_ON_PAGE(SCF(*style), 1);
-			S_SET_IS_STICKY_ON_PAGE(SCM(*style), 1);
-			S_SET_IS_STICKY_ON_PAGE(SCC(*style), 1);
-			S_SET_IS_STICKY_ON_DESK(SCF(*style), 1);
-			S_SET_IS_STICKY_ON_DESK(SCM(*style), 1);
-			S_SET_IS_STICKY_ON_DESK(SCC(*style), 1);
+			S_SET_IS_STICKY_ACROSS_PAGES(SCF(*style), 1);
+			S_SET_IS_STICKY_ACROSS_PAGES(SCM(*style), 1);
+			S_SET_IS_STICKY_ACROSS_PAGES(SCC(*style), 1);
+			S_SET_IS_STICKY_ACROSS_DESKS(SCF(*style), 1);
+			S_SET_IS_STICKY_ACROSS_DESKS(SCM(*style), 1);
+			S_SET_IS_STICKY_ACROSS_DESKS(SCC(*style), 1);
 		}
 
 		if (*(int*)retval & WIN_STATE_SHADED)
@@ -774,31 +772,39 @@ void
 GNOME_SetClientList(void)
 {
 	Atom atom_set;
-	Window wl[256];
+	Window *wl=NULL;
+	int displayed=0;
 	int i = 0;
 	FvwmWindow *t;
 
-	atom_set = XInternAtom(dpy, XA_WIN_CLIENT_LIST, False);
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	/* Find out how many window pointers we need to store, allocate
+	 * the space and go through the list again to actually store them.
+	 */
+	for (t = Scr.FvwmRoot.next; t; t = t->next)
 	{
 		if (!DO_SKIP_WINDOW_LIST(t))
 		{
-			wl[i++] = FW_W(t);
+			displayed++;
 		}
 	}
-
-	if (i > 0)
+	if(displayed)
 	{
-		XChangeProperty(
-			dpy, Scr.Root, atom_set, XA_CARDINAL, 32,
-			PropModeReplace, (unsigned char *)wl, i);
-	}
-	else
-	{
-		XChangeProperty(
-			dpy, Scr.Root, atom_set, XA_CARDINAL, 32,
-			PropModeReplace, (unsigned char *)NULL, 0);
+		wl=(Window*)safemalloc(sizeof(Window*)*displayed);
+		for (t = Scr.FvwmRoot.next; t; t = t->next)
+		{
+			if (!DO_SKIP_WINDOW_LIST(t))
+			{
+				wl[i++] = FW_W(t);
+			}
+		}
 	}
+	/* Update X */
+	atom_set = XInternAtom(dpy, XA_WIN_CLIENT_LIST, False);
+	XChangeProperty(
+		dpy, Scr.Root, atom_set, XA_CARDINAL, 32,
+		PropModeReplace, (unsigned char *)wl, i);
+	if(wl)
+		free(wl);
 
 	return;
 }
@@ -1048,13 +1054,13 @@ void GNOME_HandlePropRequest(
 	{
 		if (prop & WIN_STATE_STICKY)
 		{
-			SET_STICKY_ON_PAGE(fwin, 1);
-			SET_STICKY_ON_DESK(fwin, 1);
+			SET_STICKY_ACROSS_PAGES(fwin, 1);
+			SET_STICKY_ACROSS_DESKS(fwin, 1);
 		}
 		else
 		{
-			SET_STICKY_ON_PAGE(fwin, 0);
-			SET_STICKY_ON_DESK(fwin, 0);
+			SET_STICKY_ACROSS_PAGES(fwin, 0);
+			SET_STICKY_ACROSS_DESKS(fwin, 0);
 		}
 		border_draw_decorations(
 			fwin, PART_TITLE, (Scr.Hilite==fwin), True, CLEAR_ALL,
@@ -1115,4 +1121,3 @@ CMD_GnomeButton(F_CMD_ARGS)
 	return;
 }
 
-#endif /* GNOME */
