@@ -72,11 +72,34 @@ static void GetIconBitmap(FvwmWindow *fw);
 /* erase all traces of the last used icon in the window structure */
 void clear_icon(FvwmWindow *fw)
 {
+	int px;
+	int py;
+	int tx;
+	int ty;
+
 	FW_W_ICON_PIXMAP(fw) = None;
 	fw->iconPixmap = None;
 	fw->icon_maskPixmap = None;
 	fw->icon_alphaPixmap = None;
-	memset(&fw->icon_g, 0, sizeof(fw->icon_g));
+	fw->icon_nalloc_pixels = 0;
+	fw->icon_alloc_pixels = NULL;
+	fw->icon_no_limit = 0;
+	if (IS_ICON_MOVED(fw))
+	{
+		px = fw->icon_g.picture_w_g.x;
+		py = fw->icon_g.picture_w_g.y;
+		tx = fw->icon_g.title_w_g.x;
+		ty = fw->icon_g.title_w_g.y;
+		memset(&fw->icon_g, 0, sizeof(fw->icon_g));
+		fw->icon_g.picture_w_g.x = px;
+		fw->icon_g.picture_w_g.y = py;
+		fw->icon_g.title_w_g.x = tx;
+		fw->icon_g.title_w_g.y = ty;
+	}
+	else
+	{
+		memset(&fw->icon_g, 0, sizeof(fw->icon_g));
+	}
 
 	return;
 }
@@ -102,7 +125,7 @@ int get_visible_icon_window_count(FvwmWindow *fw)
 	return count;
 }
 
-static void setup_icon_title_size(FvwmWindow *fw)
+void setup_icon_title_size(FvwmWindow *fw)
 {
 	if (HAS_NO_ICON_TITLE(fw))
 	{
@@ -122,8 +145,11 @@ static void setup_icon_title_size(FvwmWindow *fw)
 			fw->icon_g.title_w_g.width =
 				fw->icon_g.title_text_width +
 				2 * (ICON_TITLE_TEXT_GAP_COLLAPSED +
-				     ICON_RELIEF_WIDTH);
-			if (IS_STICKY(fw) || IS_ICON_STICKY(fw))
+				     abs(fw->icon_title_relief));
+			if (IS_STICKY_ACROSS_PAGES(fw) ||
+			    IS_ICON_STICKY_ACROSS_PAGES(fw) ||
+			    IS_STICKY_ACROSS_DESKS(fw) ||
+			    IS_ICON_STICKY_ACROSS_DESKS(fw))
 			{
 				fw->icon_g.title_w_g.width +=
 					2 * (ICON_TITLE_TO_STICK_EXTRA_GAP +
@@ -140,6 +166,103 @@ static void setup_icon_title_size(FvwmWindow *fw)
 	return;
 }
 
+/****************************************************************************
+ *
+ * Resizes the given icon Pixmap.
+ *
+ ****************************************************************************/
+static void SetIconPixmapSize(
+	Pixmap *icon, unsigned int width, unsigned int height,
+	unsigned int depth, unsigned int newWidth, unsigned int newHeight,
+	Bool force_centering, int resize_type, int *nrx, int *nry,
+	unsigned int freeOldPixmap)
+{
+	Pixmap oldPixmap;
+	Pixmap resizedPixmap = None;
+	int r_w,r_h;
+	GC gc;
+	XGCValues gc_init;
+	
+	*nrx = 0;
+	*nry = 0;
+
+	/* Check for invalid dimensions */
+	if (newWidth == 0 || newHeight == 0)
+	{
+		return;
+	}
+
+	/* Save the existing Pixmap */
+	oldPixmap = *icon;
+
+	gc = XCreateGC(dpy, oldPixmap, 0, &gc_init);
+
+	switch(resize_type)
+	{
+	case ICON_RESIZE_TYPE_ADJUSTED:
+		if (newWidth != width || newHeight != height)
+		{
+			*icon = CreateStretchPixmap(
+				dpy, oldPixmap, width, height, depth, newWidth,
+				newHeight, gc);			
+		}
+		break;
+	case ICON_RESIZE_TYPE_STRETCHED:
+		if (width < newWidth || height < newHeight)
+		{
+			r_w = max(newWidth, width);
+			r_h = max(newHeight, height);
+			resizedPixmap = CreateStretchPixmap(
+				dpy, oldPixmap, width, height, depth, r_w, r_h,
+				gc);
+			width = r_w;
+			height = r_h;
+		}
+		break;
+	case ICON_RESIZE_TYPE_SHRUNK:
+		if (width > newWidth || height > newHeight)
+		{
+			r_w = min(newWidth, width);
+			r_h = min(newHeight, height);
+			resizedPixmap = CreateStretchPixmap(
+				dpy, oldPixmap, width, height, depth, r_w, r_h,
+				gc);
+			width = r_w;
+			height = r_h;
+		}
+		break;
+	default:
+		break;
+	}
+
+	if (resize_type != ICON_RESIZE_TYPE_ADJUSTED)
+	{
+		*icon = XCreatePixmap(
+			dpy, oldPixmap, newWidth, newHeight, depth);
+		XSetForeground(dpy, gc, 0);
+		XFillRectangle(dpy, *icon, gc, 0, 0, newWidth, newHeight);
+
+		/*
+		 * Copy old Pixmap onto new.  Center horizontally.  Center
+		 * vertically if the new height is smaller than the old.
+		 * Otherwise, place the icon on the bottom, along the title bar.
+		 */
+		*nrx = (newWidth - width) / 2;
+		*nry = (newHeight > height && !force_centering) ?
+			newHeight - height : (newHeight - height) / 2;
+		XCopyArea(
+			dpy, (resizedPixmap)? resizedPixmap:oldPixmap, *icon,
+			gc, 0, 0, width, height, *nrx, *nry);
+	}
+
+	XFreeGC(dpy, gc);
+
+	if (freeOldPixmap)
+	{
+		XFreePixmap(dpy, oldPixmap);
+	}
+}
+
 /* Move the icon of a window by dx/dy pixels */
 /****************************************************************************
  *
@@ -303,25 +426,61 @@ ICON_DBG((stderr,"ciw: iph%s used '%s'\n", (fw->icon_g.picture_w_g.height)?"":"
 		}
 		if (resize)
 		{
-ICON_DBG((stderr,"ciw: Changing icon (%s) from %dx%d to %dx%d\n", fw->name, fw->icon_g.picture_w_g.width, fw->icon_g.picture_w_g.height, newWidth, newHeight));
 			/* Resize the icon Pixmap */
+			int force_centering = False;
+			int nrx, nry;
+
+			ICON_DBG((stderr,"ciw: Changing icon (%s) from %dx%d to"
+				  " %dx%d\n", fw->name,
+				  fw->icon_g.picture_w_g.width,
+				  fw->icon_g.picture_w_g.height,
+				  newWidth, newHeight));
+			/* Resize the icon Pixmap */
+			/* force to center if the icon has a bg */
+			if (fw->icon_background_cs >= 0 ||
+			    fw->icon_maskPixmap == None)
+			{
+				force_centering = True;
+			}
 			SetIconPixmapSize(
 				&(fw->iconPixmap),
 				fw->icon_g.picture_w_g.width,
 				fw->icon_g.picture_w_g.height, fw->iconDepth,
-				newWidth, newHeight, (IS_PIXMAP_OURS(fw)));
+				newWidth, newHeight, force_centering,
+				fw->icon_resize_type, &nrx, &nry,
+				IS_PIXMAP_OURS(fw));
+
 			/* Resize the icon mask Pixmap if one was defined */
 			if (fw->icon_maskPixmap)
 			{
 				SetIconPixmapSize(
 					&(fw->icon_maskPixmap),
 					fw->icon_g.picture_w_g.width,
-					fw->icon_g.picture_w_g.height,
-					1, newWidth, newHeight,
-					(IS_PIXMAP_OURS(fw)));
+					fw->icon_g.picture_w_g.height, 1,
+					newWidth, newHeight, force_centering,
+					fw->icon_resize_type, &nrx, &nry,
+					IS_PIXMAP_OURS(fw));
+			}
+			else if ((nrx > 0 || nry > 0) && fw->iconDepth > 1)
+			{
+				fw->icon_maskPixmap = XCreatePixmap(
+					dpy, fw->iconPixmap,
+					newWidth, newHeight, 1);
+				XSetForeground(dpy, Scr.MonoGC, 0);
+				XFillRectangle(
+					dpy, fw->icon_maskPixmap, Scr.MonoGC,
+					0, 0, newWidth, newHeight);
+				XSetForeground(dpy, Scr.MonoGC, 1);
+				XFillRectangle(
+					dpy, fw->icon_maskPixmap, Scr.MonoGC,
+					nrx, nry, fw->icon_g.picture_w_g.width,
+					fw->icon_g.picture_w_g.height);
+				XSetForeground(dpy, Scr.MonoGC, 0);
+				/* set it shaped ? YES */
+				SET_ICON_SHAPED(fw, 1);
 			}
 
-			/* Resize the icon mask Pixmap if one was defined */
+			/* Resize the icon alpha Pixmap if one was defined */
 			if (fw->icon_alphaPixmap)
 			{
 				SetIconPixmapSize(
@@ -329,7 +488,9 @@ ICON_DBG((stderr,"ciw: Changing icon (%s) from %dx%d to %dx%d\n", fw->name, fw->
 					fw->icon_g.picture_w_g.width,
 					fw->icon_g.picture_w_g.height,
 					FRenderGetAlphaDepth(), newWidth,
-					newHeight, (IS_PIXMAP_OURS(fw)));
+					newHeight, force_centering,
+					fw->icon_resize_type, &nrx, &nry,
+					IS_PIXMAP_OURS(fw));
 			}
 
 			/* Set the new dimensions of the icon window */
@@ -343,50 +504,38 @@ ICON_DBG((stderr,"ciw: Changing icon (%s) from %dx%d to %dx%d\n", fw->name, fw->
 
 /****************************************************************************
  *
- * Resizes the given icon Pixmap.
+ * set the icon pixmap window background
  *
  ****************************************************************************/
-void SetIconPixmapSize(
-	Pixmap *icon, unsigned int width, unsigned int height,
-	unsigned int depth, unsigned int newWidth, unsigned int newHeight,
-	unsigned int freeOldPixmap)
+static void set_icon_pixmap_background(FvwmWindow *fw)
 {
-	Pixmap oldPixmap;
-	GC gc;
-	XGCValues gc_init;
-
-	/* Check for invalid dimensions */
-	if (newWidth == 0 || newHeight == 0)
+	if (fw->iconPixmap != None &&
+	    (Pdefault || (fw->iconDepth == 1) || IS_PIXMAP_OURS(fw)))
 	{
-		return;
-	}
-
-	/* Save the existing Pixmap */
-	oldPixmap = *icon;
-
-	/* Create a new Pixmap with the new dimensions */
-	*icon = XCreatePixmap(dpy, oldPixmap, newWidth, newHeight, depth);
-
-	/* Zero out new Pixmap */
-	gc = XCreateGC(dpy, *icon, 0, &gc_init);
-	XSetForeground(dpy, gc, 0);
-	XFillRectangle(dpy, *icon, gc, 0, 0, newWidth, newHeight);
-
-	/*
-	 * Copy old Pixmap onto new.  Center horizontally.  Center
-	 * vertically if the new height is smaller than the old.
-	 * Otherwise, place the icon on the bottom, along the title bar.
-	 */
-	XCopyArea(
-		dpy, oldPixmap, *icon, gc, 0, 0, width, height,
-		(newWidth - width) / 2, newHeight > height ?
-		newHeight - height : (newHeight - height) / 2);
-
-	XFreeGC(dpy, gc);
-
-	if (freeOldPixmap)
-	{
-		XFreePixmap(dpy, oldPixmap);
+		if (fw->icon_background_cs >= 0)
+		{
+			SetWindowBackground(
+				dpy, FW_W_ICON_PIXMAP(fw),
+				fw->icon_g.picture_w_g.width,
+				fw->icon_g.picture_w_g.height,
+				&Colorset[fw->icon_background_cs],
+				Pdepth, Scr.StdGC, False);
+		}
+		else if (FShapesSupported &&
+			 Pdepth == DefaultDepth(dpy, (DefaultScreen(dpy))))
+		{
+			XSetWindowBackgroundPixmap(
+				dpy, FW_W_ICON_PIXMAP(fw), ParentRelative);
+		}
+		else if (Scr.DefaultColorset >= 0)
+		{
+			SetWindowBackground(
+				dpy, FW_W_ICON_PIXMAP(fw),
+				fw->icon_g.picture_w_g.width,
+				fw->icon_g.picture_w_g.height,
+				&Colorset[Scr.DefaultColorset], Pdepth,
+				Scr.StdGC, False);
+		}
 	}
 }
 
@@ -420,7 +569,9 @@ void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y)
 	fw->iconDepth = 0;
 
 	if (IS_ICON_SUPPRESSED(fw))
+	{
 		return;
+	}
 
 	/*
 	 * set up the icon picture
@@ -433,8 +584,12 @@ void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y)
 	if ((IS_ICON_OURS(fw)) && (fw->icon_g.picture_w_g.height > 0)
 	    && (Pdefault || (fw->iconDepth == 1) || IS_PIXMAP_OURS(fw)))
 	{
-		fw->icon_g.picture_w_g.width += 2 * ICON_RELIEF_WIDTH;
-		fw->icon_g.picture_w_g.height += 2 * ICON_RELIEF_WIDTH;
+		fw->icon_g.picture_w_g.width +=
+			2 * abs(fw->icon_background_relief)
+			+ 2 * fw->icon_background_padding;
+		fw->icon_g.picture_w_g.height +=
+			2 * abs(fw->icon_background_relief)
+			+ 2 * fw->icon_background_padding;
 	}
 
 	/*
@@ -525,20 +680,7 @@ void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y)
 					fw->icon_g.picture_w_g.width,
 					fw->icon_g.picture_w_g.height);
 			}
-			if (FShapesSupported &&
-			    Pdepth == DefaultDepth(dpy, (DefaultScreen(dpy))))
-			{
-				XSetWindowBackgroundPixmap(
-					dpy, FW_W_ICON_PIXMAP(fw),
-					ParentRelative);
-			}
-			else if (Scr.DefaultColorset >= 0)
-				SetWindowBackground(
-					dpy, FW_W_ICON_TITLE(fw),
-					fw->icon_g.picture_w_g.width,
-					fw->icon_g.picture_w_g.height,
-					&Colorset[Scr.DefaultColorset], Pdepth,
-					Scr.StdGC, False);
+			set_icon_pixmap_background(fw);
 		}
 		else
 		{
@@ -570,6 +712,13 @@ void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y)
 		valuemask = CWEventMask;
 		XChangeWindowAttributes(
 			dpy, FW_W_ICON_PIXMAP(fw), valuemask,&attributes);
+		if (!IS_ICON_OURS(fw))
+		{
+			XMoveWindow(
+				dpy, FW_W_ICON_PIXMAP(fw),
+				fw->icon_g.picture_w_g.x,
+				fw->icon_g.picture_w_g.y);
+		}
 	}
 	if (old_icon_pixmap_w != None &&
 	    old_icon_pixmap_w != FW_W_ICON_PIXMAP(fw))
@@ -583,15 +732,19 @@ void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y)
 
 	if (FShapesSupported)
 	{
-		if (IS_ICON_SHAPED(fw))
+		if (IS_ICON_SHAPED(fw) && fw->icon_background_cs < 0)
 		{
 			/* when fvwm is using the non-default visual client
 			 * supplied icon pixmaps are drawn in a window with no
 			 * relief */
-			int off;
+			int off = 0;
 
-			off = (Pdefault || (fw->iconDepth == 1) ||
-			       IS_PIXMAP_OURS(fw)) ? ICON_RELIEF_WIDTH : 0;
+			if (Pdefault || (fw->iconDepth == 1) ||
+			    IS_PIXMAP_OURS(fw))
+			{
+				off = abs(fw->icon_background_relief) +
+					fw->icon_background_padding;
+			}
 			FShapeCombineMask(
 				dpy, FW_W_ICON_PIXMAP(fw), FShapeBounding, off,
 				off, fw->icon_maskPixmap, FShapeSet);
@@ -656,24 +809,29 @@ void CreateIconWindow(FvwmWindow *fw, int def_x, int def_y)
 static
 void DrawIconTitleWindow(
 	FvwmWindow *fw, XEvent *pev, Pixel BackColor, GC Shadow, GC Relief,
-	int cs)
+	int cs, int title_cs)
 {
 	int is_expanded = IS_ICON_ENTERED(fw);
 	FlocaleWinString fstr;
 	Region region = None;
 	XRectangle clip, r;
+	int relief = abs(fw->icon_title_relief);
 	int x_title;
 	int x_title_min = 0;
 	int w_title = fw->icon_g.title_text_width;
 	int x_title_w = fw->icon_g.picture_w_g.x;
 	int w_title_w = fw->icon_g.picture_w_g.width;
-	int x_stipple = ICON_RELIEF_WIDTH;
+	int x_stipple = relief;
 	int w_title_text_gap = 0;
 	int w_stipple = 0;
-	int is_sticky = (IS_STICKY(fw) || IS_ICON_STICKY(fw));
+	int is_sticky;
 	int use_unexpanded_size = 1;
 	Bool draw_string = True;
 
+	is_sticky =
+		(IS_STICKY_ACROSS_PAGES(fw) || IS_ICON_STICKY_ACROSS_PAGES(fw));
+	is_sticky |=
+		(IS_STICKY_ACROSS_DESKS(fw) || IS_ICON_STICKY_ACROSS_DESKS(fw));
 	if (is_expanded && FW_W_ICON_PIXMAP(fw) != None)
 	{
 		int sx;
@@ -683,11 +841,12 @@ void DrawIconTitleWindow(
 
 		use_unexpanded_size = 0;
 		w_title_text_gap = ICON_TITLE_TEXT_GAP_EXPANDED;
-		x_title_min = w_title_text_gap + ICON_RELIEF_WIDTH;
+		x_title_min = w_title_text_gap + relief;
 		if (is_sticky)
 		{
 			w_stipple = ICON_TITLE_STICK_MIN_WIDTH;
-			x_title_min += w_stipple + ICON_TITLE_TO_STICK_EXTRA_GAP;
+			x_title_min +=
+				w_stipple + ICON_TITLE_TO_STICK_EXTRA_GAP;
 		}
 		/* resize the icon name window */
 		w_title_w = w_title + 2 * x_title_min;
@@ -730,7 +889,7 @@ void DrawIconTitleWindow(
 	if (use_unexpanded_size)
 	{
 		w_title_text_gap = ICON_TITLE_TEXT_GAP_COLLAPSED;
-		x_title_min = w_title_text_gap + ICON_RELIEF_WIDTH;
+		x_title_min = w_title_text_gap + relief;
 		/* resize the icon name window */
 		if (FW_W_ICON_PIXMAP(fw) != None)
 		{
@@ -762,19 +921,14 @@ void DrawIconTitleWindow(
 			ICON_HEIGHT(fw));
 	}
 
-      /* why not? Should implement:
-       * {Hilight,}IconTitleStyle Colorset int [sunk, flat, raised]
-       */
-#if 0
-	if (cs >= 0)
+	if (title_cs >= 0)
 	{
 		SetWindowBackground(
 			dpy, FW_W_ICON_TITLE(fw), w_title_w,
-			ICON_HEIGHT(fw), &Colorset[cs], Pdepth,
+			ICON_HEIGHT(fw), &Colorset[title_cs], Pdepth,
 			Scr.TitleGC, False);
 	}
 	else
-#endif
 	{
 		XSetWindowBackground(
 			dpy, FW_W_ICON_TITLE(fw), BackColor);
@@ -786,9 +940,9 @@ void DrawIconTitleWindow(
 		x_title = x_title_min;
 	/* text rectangle */
 	r.x = x_title;
-	r.y = ICON_RELIEF_WIDTH;
-	r.width = w_title_w - x_title - ICON_RELIEF_WIDTH;
-	r.height = ICON_HEIGHT(fw) - 2*ICON_RELIEF_WIDTH;
+	r.y = relief;
+	r.width = w_title_w - x_title - relief;
+	r.height = ICON_HEIGHT(fw) - 2*relief;
 	if (is_sticky)
 	{
 		if (w_stipple == 0)
@@ -846,33 +1000,31 @@ void DrawIconTitleWindow(
 	}
 	if (!pev)
 	{
-		clip.x = ICON_RELIEF_WIDTH;
-		clip.y = ICON_RELIEF_WIDTH;
-		clip.width = w_title_w - 2*ICON_RELIEF_WIDTH;
-		clip.height = ICON_HEIGHT(fw) - 2*ICON_RELIEF_WIDTH;
+		clip.x = relief;
+		clip.y = relief;
+		clip.width = w_title_w - 2*relief;
+		clip.height = ICON_HEIGHT(fw) - 2*relief;
 		XClearWindow(dpy, FW_W_ICON_TITLE(fw));
 	}
 	else
 	{
 		/* needed for first drawing */
-		if (x_title-ICON_RELIEF_WIDTH >= 1)
+		if (x_title - relief >= 1)
 		{
 			/* clear before the text */
 			XClearArea(
 				dpy, FW_W_ICON_TITLE(fw),
-				ICON_RELIEF_WIDTH, ICON_RELIEF_WIDTH,
-				x_title-ICON_RELIEF_WIDTH,
-				ICON_HEIGHT(fw) - 2*ICON_RELIEF_WIDTH, False);
+				relief, relief, x_title - relief,
+				ICON_HEIGHT(fw) - 2*relief, False);
 		}
 		if (is_sticky)
 		{
 			/* clear the sticky area after the text */
 			XClearArea(
 				dpy, FW_W_ICON_TITLE(fw),
-				w_title_w - x_stipple - w_stipple -1,
-				ICON_RELIEF_WIDTH,
-				w_stipple + 2,
-				ICON_HEIGHT(fw) - 2*ICON_RELIEF_WIDTH, False);
+				w_title_w - x_stipple - w_stipple -1, relief,
+				w_stipple + 2, ICON_HEIGHT(fw) - 2*relief,
+				False);
 		}
 	}
 
@@ -889,15 +1041,19 @@ void DrawIconTitleWindow(
 		fstr.str = fw->visible_icon_name;
 		fstr.win =  FW_W_ICON_TITLE(fw);
 		fstr.gc = Scr.TitleGC;
-		if (cs >= 0)
+		if (title_cs >= 0)
+		{
+			fstr.colorset = &Colorset[title_cs];
+			fstr.flags.has_colorset = 1;
+		}
+		else if (cs >= 0)
 		{
 			fstr.colorset = &Colorset[cs];
 			fstr.flags.has_colorset = 1;
 		}
 		fstr.x = x_title;
-		fstr.y = fw->icon_g.title_w_g.height -
-			fw->icon_font->height + fw->icon_font->ascent +
-			ICON_TITLE_VERT_TEXT_OFFSET;
+		fstr.y = fw->icon_g.title_w_g.height - relief
+			- fw->icon_font->height + fw->icon_font->ascent;
 		FlocaleDrawString(dpy, fw->icon_font, &fstr, 0);
 		if (pev || is_sticky)
 		{
@@ -910,15 +1066,19 @@ void DrawIconTitleWindow(
 	}
 	RelieveRectangle(
 		dpy, FW_W_ICON_TITLE(fw), 0, 0, w_title_w - 1,
-		ICON_HEIGHT(fw) - 1, Relief, Shadow, ICON_RELIEF_WIDTH);
+		ICON_HEIGHT(fw) - 1,
+		(fw->icon_title_relief > 0)? Relief:Shadow,
+		(fw->icon_title_relief > 0)? Shadow:Relief, relief);
 	if (is_sticky)
 	{
 		/* an odd number of lines every 4 pixels */
-		int num = (ICON_HEIGHT(fw) /
+		int pseudo_height = ICON_HEIGHT(fw)- 2*relief + 2;
+		int num = (pseudo_height /
 			   ICON_TITLE_STICK_VERT_DIST / 2) * 2 - 1;
-		int min = ICON_HEIGHT(fw) / 2 - num * 2 + 1;
-		int max = ICON_HEIGHT(fw) / 2 + num * 2 -
-			ICON_TITLE_STICK_VERT_DIST + 1;
+		int min = ICON_HEIGHT(fw) / 2 -
+			num * 2 + 1;
+		int max = ICON_HEIGHT(fw) / 2 +
+			num * 2 - ICON_TITLE_STICK_VERT_DIST + 1;
 		int i;
 
 		for(i = min; w_stipple > 0 && i <= max;
@@ -935,32 +1095,29 @@ void DrawIconTitleWindow(
 				ICON_TITLE_STICK_HEIGHT);
 		}
 	}
-
+	
 	return;
 }
 
 static
 void DrawIconPixmapWindow(
-	FvwmWindow *fw, XEvent *pev, GC Shadow, GC Relief, int cs)
+	FvwmWindow *fw, Bool reset_bg, XEvent *pev, GC Shadow, GC Relief, int cs)
 {
 	XRectangle r,clip;
+	Bool cleared = False;
 
 	if (!pev)
 	{
 		XMoveWindow(
 			dpy, FW_W_ICON_PIXMAP(fw), fw->icon_g.picture_w_g.x,
 			fw->icon_g.picture_w_g.y);
-	}
-
-	/* only relieve unshaped icons that share fvwm's visual */
-	if ((fw->iconPixmap != None) && !IS_ICON_SHAPED(fw)
-	    && (Pdefault || (fw->iconDepth == 1) || IS_PIXMAP_OURS(fw)))
-	{
-		RelieveRectangle(dpy, FW_W_ICON_PIXMAP(fw), 0, 0,
-				 fw->icon_g.picture_w_g.width - 1,
-				 fw->icon_g.picture_w_g.height - 1,
-				 Relief, Shadow,
-				 ICON_RELIEF_WIDTH);
+		if (reset_bg &&
+		    (fw->iconDepth == 1 || Pdefault || IS_PIXMAP_OURS(fw)))
+		{
+			set_icon_pixmap_background(fw);
+			XClearArea(dpy, FW_W_ICON_PIXMAP(fw), 0, 0, 0, 0, False);
+			cleared = True;
+		}
 	}
 
 	/* need to locate the icon pixmap */
@@ -971,17 +1128,21 @@ void DrawIconPixmapWindow(
 			FvwmRenderAttributes fra;
 			Bool draw_icon = True;
 
+			memset(&fra, 0, sizeof(fra));
 			fra.mask = FRAM_DEST_IS_A_WINDOW;
 			if (cs >= 0)
 			{
 				fra.mask |= FRAM_HAVE_ICON_CSET;
 				fra.colorset = &Colorset[cs];
 			}
-			r.x = r.y = ICON_RELIEF_WIDTH;
+			r.x = r.y = abs(fw->icon_background_relief) +
+				fw->icon_background_padding;
 			r.width = fw->icon_g.picture_w_g.width -
-				2 * ICON_RELIEF_WIDTH;
+				2 * (abs(fw->icon_background_relief) +
+				     fw->icon_background_padding);
 			r.height = fw->icon_g.picture_w_g.height -
-				2 * ICON_RELIEF_WIDTH;
+				2 * (abs(fw->icon_background_relief) +
+				     fw->icon_background_padding);
 			if (pev)
 			{
 				if (!frect_get_intersection(
@@ -999,12 +1160,12 @@ void DrawIconPixmapWindow(
 				clip.width = r.width;
 				clip.height = r.height;
 			}
-
 			if (draw_icon)
 			{
-				if (fw->icon_alphaPixmap ||
-				    (cs >= 0 &&
-				     Colorset[cs].icon_alpha_percent < 100))
+				if (!cleared &&
+				    (fw->icon_alphaPixmap ||
+				     (cs >= 0 &&
+				      Colorset[cs].icon_alpha_percent < 100)))
 				{
 					XClearArea(
 						dpy, FW_W_ICON_PIXMAP(fw),
@@ -1016,7 +1177,7 @@ void DrawIconPixmapWindow(
 					fw->iconPixmap, fw->icon_maskPixmap,
 					fw->icon_alphaPixmap, fw->iconDepth,
 					&fra, FW_W_ICON_PIXMAP(fw),
-					Scr.TitleGC, Scr.MonoGC, None,
+					Scr.TitleGC, Scr.MonoGC, Scr.AlphaGC,
 					clip.x - r.x, clip.y - r.y,
 					clip.width, clip.height,
 					clip.x, clip.y, clip.width, clip.height,
@@ -1041,20 +1202,36 @@ void DrawIconPixmapWindow(
 				UninstallRootColormap();
 		}
 	}
+	/* only relieve unshaped icons or icons with a bg that share fvwm's
+	 * visual */
+	if ((fw->iconPixmap != None) &&
+	    (!IS_ICON_SHAPED(fw) || fw->icon_background_cs >= 0) &&
+	    (Pdefault || (fw->iconDepth == 1) || IS_PIXMAP_OURS(fw)))
+	{
+		RelieveRectangle(
+			dpy, FW_W_ICON_PIXMAP(fw), 0, 0,
+			fw->icon_g.picture_w_g.width - 1,
+			fw->icon_g.picture_w_g.height - 1,
+			(fw->icon_background_relief > 0)? Relief:Shadow,
+			(fw->icon_background_relief > 0)? Shadow:Relief,
+			abs(fw->icon_background_relief));
+	}
+
 }
 
 void DrawIconWindow(
 	FvwmWindow *fw, Bool draw_title, Bool draw_pixmap, Bool focus_change,
-	XEvent *pev)
+	Bool reset_bg, XEvent *pev)
 {
 	GC Shadow;
 	GC Relief;
 	Pixel TextColor;
 	Pixel BackColor;
-	color_quad *draw_colors;
-	color_quad *co_draw_colors;
-	int cs;
-	int co_cs;
+	color_quad draw_colors;
+	color_quad co_draw_colors;
+	int cs, co_cs;
+	int title_cs = -1;
+	int co_title_cs = -1;
 	int is_expanded = IS_ICON_ENTERED(fw);
 
 	if (IS_ICON_SUPPRESSED(fw) || (pev && fw->Desk != Scr.CurrentDesk))
@@ -1064,15 +1241,71 @@ void DrawIconWindow(
 
 	if (Scr.Hilite == fw)
 	{
-		draw_colors = &(fw->hicolors);
-		co_draw_colors = &(fw->colors);
+		if (fw->icon_title_cs_hi >= 0)
+		{
+			title_cs = fw->icon_title_cs_hi;
+			draw_colors.hilight = Colorset[title_cs].hilite;
+			draw_colors.shadow = Colorset[title_cs].shadow;
+			draw_colors.back = Colorset[title_cs].bg;
+			draw_colors.fore = Colorset[title_cs].fg;
+		}
+		else
+		{
+			draw_colors.hilight = fw->hicolors.hilight;
+			draw_colors.shadow = fw->hicolors.shadow;
+			draw_colors.back = fw->hicolors.back;
+			draw_colors.fore = fw->hicolors.fore;
+		}
+		if (fw->icon_title_cs >= 0)
+		{
+			co_title_cs = fw->icon_title_cs;
+			co_draw_colors.hilight = Colorset[co_title_cs].hilite;
+			co_draw_colors.shadow = Colorset[co_title_cs].shadow;
+			co_draw_colors.back = Colorset[co_title_cs].bg;
+			co_draw_colors.fore = Colorset[co_title_cs].fg;
+		}
+		else
+		{
+			co_draw_colors.hilight = fw->colors.hilight;
+			co_draw_colors.shadow = fw->colors.shadow;
+			co_draw_colors.back = fw->colors.back;
+			co_draw_colors.fore = fw->colors.fore;
+		}
 		cs = fw->cs_hi;
 		co_cs = fw->cs;
 	}
 	else
 	{
-		draw_colors = &(fw->colors);
-		co_draw_colors = &(fw->hicolors);
+		if (fw->icon_title_cs >= 0)
+		{
+			title_cs = fw->icon_title_cs;
+			draw_colors.hilight = Colorset[title_cs].hilite;
+			draw_colors.shadow = Colorset[title_cs].shadow;
+			draw_colors.back = Colorset[title_cs].bg;
+			draw_colors.fore = Colorset[title_cs].fg;
+		}
+		else
+		{
+			draw_colors.hilight = fw->colors.hilight;
+			draw_colors.shadow = fw->colors.shadow;
+			draw_colors.back = fw->colors.back;
+			draw_colors.fore = fw->colors.fore;
+		}
+		if (fw->icon_title_cs_hi >= 0)
+		{
+			co_title_cs = fw->icon_title_cs_hi;
+			co_draw_colors.hilight = Colorset[co_title_cs].hilite;
+			co_draw_colors.shadow = Colorset[co_title_cs].shadow;
+			co_draw_colors.back = Colorset[co_title_cs].bg;
+			co_draw_colors.fore = Colorset[co_title_cs].fg;
+		}
+		else
+		{
+			co_draw_colors.hilight = fw->hicolors.hilight;
+			co_draw_colors.shadow = fw->hicolors.shadow;
+			co_draw_colors.back = fw->hicolors.back;
+			co_draw_colors.fore = fw->hicolors.fore;
+		}
 		cs = fw->cs;
 		co_cs = fw->cs_hi;
 	}
@@ -1084,23 +1317,27 @@ void DrawIconWindow(
 	else
 	{
 		if (Pdepth < 2 && Scr.Hilite == fw)
+		{
 			Relief = Scr.ScratchGC2;
+		}
 		else
 		{
-			Globalgcv.foreground = draw_colors->hilight;
+			Globalgcv.foreground = draw_colors.hilight;
 			Globalgcm = GCForeground;
 			XChangeGC(dpy,Scr.ScratchGC1,Globalgcm,&Globalgcv);
 			Relief = Scr.ScratchGC1;
 		}
-		Globalgcv.foreground = draw_colors->shadow;
+		Globalgcv.foreground = draw_colors.shadow;
 		XChangeGC(dpy,Scr.ScratchGC2, Globalgcm, &Globalgcv);
 		Shadow = Scr.ScratchGC2;
 	}
-	TextColor = draw_colors->fore;
-	BackColor = draw_colors->back;
+	TextColor = draw_colors.fore;
+	BackColor = draw_colors.back;
 	/* set up TitleGC for drawing the icon label */
 	if (fw->icon_font != NULL)
+	{
 		NewFontAndColor(fw->icon_font, TextColor, BackColor);
+	}
 
 	if (draw_title && FW_W_ICON_TITLE(fw) != None)
 	{
@@ -1113,7 +1350,8 @@ void DrawIconWindow(
 				flush_accumulate_expose(
 					FW_W_ICON_TITLE(fw), &e);
 				DrawIconTitleWindow(
-					fw, &e, BackColor, Shadow, Relief, cs);
+					fw, &e, BackColor, Shadow, Relief, cs,
+					title_cs);
 			}
 		}
 		else
@@ -1123,7 +1361,36 @@ void DrawIconWindow(
 				flush_expose(FW_W_ICON_TITLE(fw));
 			}
 			DrawIconTitleWindow(
-				fw, pev, BackColor, Shadow, Relief, cs);
+				fw, pev, BackColor, Shadow, Relief, cs,
+				title_cs);
+		}
+	}
+
+	if (draw_pixmap)
+	{
+		int bg_cs = fw->icon_background_cs;
+
+		if (bg_cs >= 0 &&
+		    (fw->iconDepth != 1 ||
+		     fw->icon_background_padding > 0 ||
+		     fw->icon_maskPixmap != None ||
+		     fw->icon_alphaPixmap != None))
+		{
+			if (Pdepth < 2 && Scr.Hilite == fw)
+			{
+				Relief = Scr.ScratchGC2;
+			}
+			else
+			{
+				Globalgcv.foreground = Colorset[bg_cs].hilite;
+				Globalgcm = GCForeground;
+				XChangeGC(
+					dpy,Scr.ScratchGC1,Globalgcm,&Globalgcv);
+				Relief = Scr.ScratchGC1;
+			}
+			Globalgcv.foreground = Colorset[bg_cs].shadow;
+			XChangeGC(dpy,Scr.ScratchGC2, Globalgcm, &Globalgcv);
+			Shadow = Scr.ScratchGC2;
 		}
 	}
 
@@ -1144,20 +1411,20 @@ void DrawIconWindow(
 		else if (fw->iconDepth == 1)
 		{
 			color_change =
-				(draw_colors->fore !=
-				 co_draw_colors->back) ||
-				(draw_colors->fore !=
-				 co_draw_colors->back);
+				(draw_colors.fore !=
+				 co_draw_colors.back) ||
+				(draw_colors.fore !=
+				 co_draw_colors.back);
 		}
 		if (!relief_change &&
 		    (fw->iconPixmap != None) && !IS_ICON_SHAPED(fw)
 		    && (Pdefault || (fw->iconDepth == 1) || IS_PIXMAP_OURS(fw)))
 		{
 			relief_change =
-				(draw_colors->hilight !=
-				 co_draw_colors->hilight) ||
-				(draw_colors->shadow !=
-				 co_draw_colors->shadow);
+				(draw_colors.hilight !=
+				 co_draw_colors.hilight) ||
+				(draw_colors.shadow !=
+				 co_draw_colors.shadow);
 		}
 		if (cs >= 0 && co_cs >= 0)
 		{
@@ -1200,7 +1467,7 @@ void DrawIconWindow(
 				flush_accumulate_expose(
 					FW_W_ICON_PIXMAP(fw), &e);
 				DrawIconPixmapWindow(
-					fw, &e, Shadow, Relief, cs);
+					fw, reset_bg, &e, Shadow, Relief, cs);
 			}
 		}
 		else
@@ -1210,7 +1477,7 @@ void DrawIconWindow(
 				flush_expose(FW_W_ICON_PIXMAP(fw));
 			}
 			DrawIconPixmapWindow(
-				fw, pev, Shadow, Relief, cs);
+				fw, reset_bg, pev, Shadow, Relief, cs);
 		}
 	}
 
@@ -1295,7 +1562,7 @@ void ChangeIconPixmap(FvwmWindow *fw)
 			}
 		}
 		SET_ICONIFIED(fw, 1);
-		DrawIconWindow(fw, False, True, False, NULL);
+		DrawIconWindow(fw, False, True, False, False, NULL);
 	}
 
 	return;
@@ -1321,7 +1588,7 @@ void RedoIconName(FvwmWindow *fw)
 	/* clear the icon window, and trigger a re-draw via an expose event */
 	if (IS_ICONIFIED(fw))
 	{
-		DrawIconWindow(fw, True, False, False, NULL);
+		DrawIconWindow(fw, True, False, False, False, NULL);
 		XClearArea(dpy, FW_W_ICON_TITLE(fw), 0, 0, 0, 0, True);
 	}
 
@@ -1355,7 +1622,11 @@ void AutoPlaceIcon(
   /* New! Put icon in same page as the center of the window */
   /* Not a good idea for StickyIcons. Neither for icons of windows that are
    * visible on the current page. */
-  if (IS_ICON_STICKY(t) || IS_STICKY(t))
+  if (IS_ICON_STICKY_ACROSS_DESKS(t) || IS_STICKY_ACROSS_DESKS(t))
+  {
+    t->Desk = Scr.CurrentDesk;
+  }
+  if (IS_ICON_STICKY_ACROSS_PAGES(t) || IS_STICKY_ACROSS_PAGES(t))
   {
     base_x = 0;
     base_y = 0;
@@ -1368,7 +1639,6 @@ void AutoPlaceIcon(
       new_y += Scr.MyDisplayHeight;
     frame_setup_window(
 	    t, new_x, new_y, t->frame_g.width, t->frame_g.height, False);
-    t->Desk = Scr.CurrentDesk;
   }
   else if (IsRectangleOnThisPage(&(t->frame_g), t->Desk))
   {
@@ -1807,7 +2077,7 @@ static void GetIconFromFile(FvwmWindow *fw)
 	char *path = NULL;
 	FvwmPictureAttributes fpa;
 
-	fpa.mask = FPAM_NO_ALLOC_PIXELS; /* olicha why ? */
+	fpa.mask = 0;
 	if (fw->cs >= 0 && Colorset[fw->cs].do_dither_icon)
 	{
 		fpa.mask |= FPAM_DITHER;
@@ -1820,9 +2090,11 @@ static void GetIconFromFile(FvwmWindow *fw)
 		return;
 	}
 	if (!PImageLoadPixmapFromFile(
-		dpy, Scr.Root, path, &fw->iconPixmap, &fw->icon_maskPixmap,
+		dpy, Scr.NoFocusWin, path, &fw->iconPixmap, &fw->icon_maskPixmap,
 		&fw->icon_alphaPixmap, &fw->icon_g.picture_w_g.width,
-		&fw->icon_g.picture_w_g.height, &fw->iconDepth, 0, NULL, fpa))
+		&fw->icon_g.picture_w_g.height, &fw->iconDepth,
+		&fw->icon_nalloc_pixels, &fw->icon_alloc_pixels,
+		&fw->icon_no_limit, fpa))
 	{
 		fvwm_msg(ERR, "GetIconFromFile", "Failed to load %s", path);
 		free(path);
@@ -2277,7 +2549,11 @@ void Iconify(FvwmWindow *fw, initial_window_options_type *win_opts)
 	 * if the window has an icon, but neither a pixmap nor a title. */
 	if (HAS_NO_ICON_TITLE(fw) && FW_W_ICON_PIXMAP(fw) == None)
 	{
-		memset(&fw->icon_g, 0, sizeof(fw->icon_g));
+		fw->icon_g.picture_w_g.width = 0;
+		fw->icon_g.picture_w_g.height = 0;
+		fw->icon_g.title_w_g.width = 0;
+		fw->icon_g.title_w_g.height = 0;
+		fw->icon_g.title_text_width = 0;
 	}
 	SET_ICONIFIED(fw, 1);
 	SET_ICON_UNMAPPED(fw, 0);
@@ -2297,7 +2573,7 @@ void Iconify(FvwmWindow *fw, initial_window_options_type *win_opts)
 	{
 		LowerWindow(fw);
 	}
-	if (IS_ICON_STICKY(fw) || IS_STICKY(fw))
+	if (IS_ICON_STICKY_ACROSS_DESKS(fw) || IS_STICKY_ACROSS_DESKS(fw))
 	{
 		fw->Desk = Scr.CurrentDesk;
 	}
