@@ -6,12 +6,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /* ---------------------------- included header files ----------------------- */
@@ -19,31 +19,20 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <string.h>
 #include <assert.h>
 #include <X11/keysym.h>
 
-#include <libs/fvwmlib.h>
+#include "libs/fvwmlib.h"
+#include "libs/Picture.h"
+#include "libs/PictureUtils.h"
 #include "fvwm.h"
 #include "externs.h"
-#include "events.h"
-#include "menus.h"
-#include "cursor.h"
+#include "execcontext.h"
 #include "functions.h"
-#include "repeat.h"
 #include "misc.h"
-#include "move_resize.h"
 #include "screen.h"
-#include "borders.h"
 #include "colors.h"
-#include "colormaps.h"
-#include "decorations.h"
 #include "colorset.h"
-#include "defaults.h"
-#include "libs/FScreen.h"
-#include "libs/Flocale.h"
-#include "libs/Picture.h"
-
 #include "menustyle.h"
 
 /* ---------------------------- local definitions --------------------------- */
@@ -71,15 +60,24 @@ static void menustyle_free_face(MenuFace *mf)
 	switch (mf->type)
 	{
 	case GradientMenu:
-		/* - should we check visual is not TrueColor before doing this?
-		 */
-#if 0
-		XFreeColors(
-			dpy, PictureCMap, ms->u.grad.pixels,
-			ms->u.grad.npixels, AllPlanes);
-#endif
-		free(mf->u.grad.pixels);
-		mf->u.grad.pixels = NULL;
+		if (Pdepth <= 8 && mf->u.grad.npixels > 0 &&
+		    !mf->u.grad.do_dither)
+		{
+			Pixel *p;
+			int i;
+
+			p = (Pixel *)safemalloc(
+				mf->u.grad.npixels * sizeof(Pixel));
+			for(i=0; i < mf->u.grad.npixels; i++)
+			{
+				p[i] = mf->u.grad.xcs[i].pixel;
+			}
+			PictureFreeColors(
+				dpy, Pcmap, p, mf->u.grad.npixels, 0, False);
+			free(p);
+		}
+		free(mf->u.grad.xcs);
+		mf->u.grad.xcs = NULL;
 		break;
 	case PixmapMenu:
 	case TiledPixmapMenu:
@@ -90,7 +88,7 @@ static void menustyle_free_face(MenuFace *mf)
 		mf->u.p = NULL;
 		break;
 	case SolidMenu:
-		FreeColors(&mf->u.back, 1);
+		FreeColors(&mf->u.back, 1, True);
 	default:
 		break;
 	}
@@ -109,6 +107,7 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 	char *style;
 	char *token;
 	char *action = s;
+	FvwmPictureAttributes fpa;
 
 	s = GetNextToken(s, &style);
 	if (style && strncasecmp(style, "--", 2) == 0)
@@ -151,7 +150,7 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 	{
 		char **s_colors;
 		int npixels, nsegs, *perc;
-		Pixel *pixels;
+		XColor *xcs;
 
 		if (!IsGradientTypeSupported(style[0]))
 		{
@@ -164,14 +163,21 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 		{
 			return False;
 		}
+		/* dither ? */
+		mf->u.grad.do_dither = False;
+		if (Pdepth <= 8)
+		{
+			mf->u.grad.do_dither = True;
+		}
 		/* grab the colors */
-		pixels = AllocAllGradientColors(s_colors, perc, nsegs, npixels);
-		if (pixels == None)
+		xcs = AllocAllGradientColors(
+			s_colors, perc, nsegs, npixels, mf->u.grad.do_dither);
+		if (xcs == None)
 		{
 			return False;
 		}
 
-		mf->u.grad.pixels = pixels;
+		mf->u.grad.xcs = xcs;
 		mf->u.grad.npixels = npixels;
 		mf->type = GradientMenu;
 		mf->gradient_type = toupper(style[0]);
@@ -180,12 +186,11 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 	else if (StrEquals(style,"Pixmap") || StrEquals(style,"TiledPixmap"))
 	{
 		s = GetNextToken(s, &token);
+		fpa.mask = (Pdepth <= 8)?  FPAM_DITHER:0;
 		if (token)
 		{
-			mf->u.p = PCacheFvwmPicture(dpy, Scr.NoFocusWin,
-						    NULL,
-						    token,
-						    Scr.ColorLimit);
+			mf->u.p = PCacheFvwmPicture(
+				dpy, Scr.NoFocusWin, NULL, token, fpa);
 			if (mf->u.p == NULL)
 			{
 				if (verbose)
@@ -347,6 +352,20 @@ static int menustyle_get_styleopt_index(char *option)
 	return GetTokenIndex(option, optlist, 0, NULL);
 }
 
+static void change_or_make_gc(GC *gc, unsigned long gcm, XGCValues *gcv)
+{
+	if (*gc != None)
+	{
+		XChangeGC(dpy, *gc, gcm, gcv);
+	}
+	else
+	{
+		*gc = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, gcv);
+	}
+
+	return;
+}
+
 /* ---------------------------- interface functions ------------------------- */
 
 MenuStyle *menustyle_get_default_style(void)
@@ -363,37 +382,37 @@ void menustyle_free(MenuStyle *ms)
 		return;
 	}
 	menustyle_free_face(&ST_FACE(ms));
-	if (ST_MENU_GC(ms))
+	if (FORE_GC(ST_MENU_INACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_GC(ms));
+		XFreeGC(dpy, FORE_GC(ST_MENU_INACTIVE_GCS(ms)));
 	}
-	if (ST_MENU_ACTIVE_GC(ms))
+	if (FORE_GC(ST_MENU_ACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_ACTIVE_GC(ms));
+		XFreeGC(dpy, FORE_GC(ST_MENU_ACTIVE_GCS(ms)));
 	}
-	if (ST_MENU_ACTIVE_BACK_GC(ms))
+	if (BACK_GC(ST_MENU_ACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_ACTIVE_BACK_GC(ms));
+		XFreeGC(dpy, BACK_GC(ST_MENU_ACTIVE_GCS(ms)));
 	}
-	if (ST_MENU_ACTIVE_RELIEF_GC(ms))
+	if (HILIGHT_GC(ST_MENU_ACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_ACTIVE_RELIEF_GC(ms));
+		XFreeGC(dpy, HILIGHT_GC(ST_MENU_ACTIVE_GCS(ms)));
 	}
-	if (ST_MENU_ACTIVE_SHADOW_GC(ms))
+	if (SHADOW_GC(ST_MENU_ACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_ACTIVE_SHADOW_GC(ms));
+		XFreeGC(dpy, SHADOW_GC(ST_MENU_ACTIVE_GCS(ms)));
 	}
-	if (ST_MENU_RELIEF_GC(ms))
+	if (HILIGHT_GC(ST_MENU_INACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_RELIEF_GC(ms));
+		XFreeGC(dpy, HILIGHT_GC(ST_MENU_INACTIVE_GCS(ms)));
 	}
 	if (ST_MENU_STIPPLE_GC(ms))
 	{
 		XFreeGC(dpy, ST_MENU_STIPPLE_GC(ms));
 	}
-	if (ST_MENU_SHADOW_GC(ms))
+	if (SHADOW_GC(ST_MENU_INACTIVE_GCS(ms)))
 	{
-		XFreeGC(dpy, ST_MENU_SHADOW_GC(ms));
+		XFreeGC(dpy, SHADOW_GC(ST_MENU_INACTIVE_GCS(ms)));
 	}
 	if (ST_SIDEPIC(ms))
 	{
@@ -401,7 +420,7 @@ void menustyle_free(MenuStyle *ms)
 	}
 	if (ST_HAS_SIDE_COLOR(ms) == 1)
 	{
-		FreeColors(&ST_SIDE_COLOR(ms), 1);
+		FreeColors(&ST_SIDE_COLOR(ms), 1, True);
 	}
 	if (ST_PSTDFONT(ms) && !ST_USING_DEFAULT_FONT(ms))
 	{
@@ -445,17 +464,12 @@ void menustyle_update(MenuStyle *ms)
 {
 	XGCValues gcv;
 	unsigned long gcm;
-	Pixel menu_fore;
-	Pixel menu_back;
-	Pixel relief_fore;
-	Pixel relief_back;
-	Pixel active_fore;
-	Pixel active_back;
-	Pixel active_relief_fore;
-	Pixel active_relief_back;
-	colorset_struct *menu_cs = &Colorset[ST_CSET_MENU(ms)];
-	colorset_struct *active_cs = &Colorset[ST_CSET_ACTIVE(ms)];
-	colorset_struct *greyed_cs = &Colorset[ST_CSET_GREYED(ms)];
+	color_quad c_inactive;
+	color_quad c_active;
+	color_quad c_stipple;
+	colorset_t *menu_cs = &Colorset[ST_CSET_MENU(ms)];
+	colorset_t *active_cs = &Colorset[ST_CSET_ACTIVE(ms)];
+	colorset_t *greyed_cs = &Colorset[ST_CSET_GREYED(ms)];
 
 	if (ST_USAGE_COUNT(ms) != 0)
 	{
@@ -464,7 +478,10 @@ void menustyle_update(MenuStyle *ms)
 		return;
 	}
 	ST_IS_UPDATED(ms) = 1;
-
+	if (ST_USING_DEFAULT_FONT(ms))
+	{
+		ST_PSTDFONT(ms) = Scr.DefaultFont;
+	}
 	/* calculate colors based on foreground */
 	if (!ST_HAS_ACTIVE_FORE(ms))
 	{
@@ -479,62 +496,75 @@ void menustyle_update(MenuStyle *ms)
 	{
 		ST_MENU_STIPPLE_COLORS(ms).fore = ST_MENU_COLORS(ms).back;
 	}
-	if (Pdepth > 2)
-	{
-		/* if not black and white */
-		ST_MENU_RELIEF_COLORS(ms).back =
-			GetShadow(ST_MENU_COLORS(ms).back);
-		ST_MENU_RELIEF_COLORS(ms).fore =
-			GetHilite(ST_MENU_COLORS(ms).back);
-	}
-	else
-	{
-		/* black and white */
-		ST_MENU_RELIEF_COLORS(ms).back =
-			GetColor(DEFAULT_SHADOW_COLOR);
-		ST_MENU_RELIEF_COLORS(ms).fore =
-			GetColor(DEFAULT_HILIGHT_COLOR);
-	}
 	ST_MENU_STIPPLE_COLORS(ms).back = ST_MENU_COLORS(ms).back;
-
-	/* calculate some pixel values for convenience reasons */
+	/* prepare colours for changing the gcs */
 	if (ST_HAS_MENU_CSET(ms))
 	{
-		menu_fore = menu_cs->fg;
-		menu_back = menu_cs->bg;
-		relief_fore = menu_cs->hilite;
-		relief_back = menu_cs->shadow;
-		active_relief_fore = menu_cs->hilite;
-		active_relief_back = menu_cs->shadow;
+		c_inactive.fore = menu_cs->fg;
+		c_inactive.back = menu_cs->bg;
+		c_inactive.hilight = menu_cs->hilite;
+		c_inactive.shadow = menu_cs->shadow;
 	}
 	else
 	{
-		menu_fore = ST_MENU_COLORS(ms).fore;
-		menu_back = ST_MENU_COLORS(ms).back;
-		relief_fore = ST_MENU_RELIEF_COLORS(ms).fore;
-		relief_back = ST_MENU_RELIEF_COLORS(ms).back;
-		active_relief_fore = ST_MENU_RELIEF_COLORS(ms).fore;
-		active_relief_back = ST_MENU_RELIEF_COLORS(ms).back;
+		c_inactive.fore = ST_MENU_COLORS(ms).fore;
+		c_inactive.back = ST_MENU_COLORS(ms).back;
+		if (Pdepth > 2)
+		{
+			c_inactive.hilight = GetHilite(ST_MENU_COLORS(ms).back);
+			c_inactive.shadow = GetShadow(ST_MENU_COLORS(ms).back);
+		}
+		else
+		{
+			c_inactive.hilight = GetColor(DEFAULT_HILIGHT_COLOR);
+			c_inactive.shadow = GetColor(DEFAULT_SHADOW_COLOR);
+		}
 	}
 	if (ST_HAS_ACTIVE_CSET(ms))
 	{
-		active_fore = active_cs->fg;
-		active_back = active_cs->bg;
-		active_relief_fore = active_cs->hilite;
-		active_relief_back = active_cs->shadow;
+		c_active.fore = active_cs->fg;
+		c_active.back = active_cs->bg;
+		c_active.hilight = active_cs->hilite;
+		c_active.shadow = active_cs->shadow;
 	}
 	else
 	{
-		active_fore = (ST_HAS_ACTIVE_FORE(ms)) ?
-			ST_MENU_ACTIVE_COLORS(ms).fore : menu_fore;
-		active_back = (ST_HAS_ACTIVE_BACK(ms)) ?
-			ST_MENU_ACTIVE_COLORS(ms).back : menu_back;
+		c_active.fore = ST_MENU_ACTIVE_COLORS(ms).fore;
+		c_active.back = ST_MENU_ACTIVE_COLORS(ms).back;
+		if (Pdepth > 2)
+		{
+			c_active.hilight =
+				GetHilite(ST_MENU_ACTIVE_COLORS(ms).back);
+			c_active.shadow =
+				GetShadow(ST_MENU_ACTIVE_COLORS(ms).back);
+		}
+		else
+		{
+			c_active.hilight = GetColor(DEFAULT_HILIGHT_COLOR);
+			c_active.shadow = GetColor(DEFAULT_SHADOW_COLOR);
+		}
 	}
-	if (ST_USING_DEFAULT_FONT(ms))
+	if (ST_HAS_GREYED_CSET(ms))
 	{
-		ST_PSTDFONT(ms) = Scr.DefaultFont;
+		c_stipple.fore = greyed_cs->fg;
+		c_stipple.back = greyed_cs->fg;
+	}
+	else
+	{
+		c_stipple.fore = ST_MENU_STIPPLE_COLORS(ms).fore;
+		c_stipple.back = ST_MENU_STIPPLE_COLORS(ms).back;
+	}
+	/* override hilighting colours if necessary */
+	if (!ST_DO_HILIGHT_FORE(ms))
+	{
+		c_active.fore = c_inactive.fore;
+	}
+	if (!ST_DO_HILIGHT_BACK(ms))
+	{
+		c_active.back = c_inactive.back;
+		c_active.hilight = c_inactive.hilight;
+		c_active.shadow = c_inactive.shadow;
 	}
-
 	/* make GC's */
 	gcm = GCFunction|GCLineWidth|GCForeground|GCBackground;
 	if (ST_PSTDFONT(ms)->font != NULL)
@@ -544,125 +574,51 @@ void menustyle_update(MenuStyle *ms)
 	}
 	gcv.function = GXcopy;
 	gcv.line_width = 0;
-
-	/* update relief gc */
-	gcv.foreground = relief_fore;
-	gcv.background = relief_back;
-	if (ST_MENU_RELIEF_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_RELIEF_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_RELIEF_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update shadow gc */
-	gcv.foreground = relief_back;
-	gcv.background = relief_fore;
-	if (ST_MENU_SHADOW_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_SHADOW_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_SHADOW_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update active gc */
-	gcv.foreground = active_fore;
-	gcv.background = active_back;
-	if (ST_MENU_ACTIVE_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_ACTIVE_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_ACTIVE_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update active relief gc */
-	gcv.foreground = active_relief_fore;
-	gcv.background = active_relief_back;
-	if (ST_MENU_ACTIVE_RELIEF_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_ACTIVE_RELIEF_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_ACTIVE_RELIEF_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update active shadow gc */
-	gcv.foreground = active_relief_back;
-	gcv.background = active_relief_fore;
-	if (ST_MENU_ACTIVE_SHADOW_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_ACTIVE_SHADOW_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_ACTIVE_SHADOW_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update active back gc */
-	gcv.foreground = active_back;
-	gcv.background = active_fore;
-	if (ST_MENU_ACTIVE_BACK_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_ACTIVE_BACK_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_ACTIVE_BACK_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update menu gc */
-	gcv.foreground = menu_fore;
-	gcv.background = menu_back;
-	if (ST_MENU_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	}
-
-	/* update stipple gc */
+	/* update inactive menu gcs */
+	gcv.foreground = c_inactive.fore;
+	gcv.background = c_inactive.back;
+	change_or_make_gc(&FORE_GC(ST_MENU_INACTIVE_GCS(ms)), gcm, &gcv);
+	BACK_GC(ST_MENU_INACTIVE_GCS(ms)) = FORE_GC(ST_MENU_INACTIVE_GCS(ms));
+	gcv.foreground = c_inactive.hilight;
+	gcv.background = c_inactive.shadow;
+	change_or_make_gc(&HILIGHT_GC(ST_MENU_INACTIVE_GCS(ms)), gcm, &gcv);
+	gcv.foreground = c_inactive.shadow;
+	gcv.background = c_inactive.hilight;
+	change_or_make_gc(&SHADOW_GC(ST_MENU_INACTIVE_GCS(ms)), gcm, &gcv);
+	/* update active menu gcs */
+	gcv.foreground = c_active.fore;
+	gcv.background = c_active.back;
+	change_or_make_gc(&FORE_GC(ST_MENU_ACTIVE_GCS(ms)), gcm, &gcv);
+	gcv.foreground = c_active.back;
+	gcv.background = c_active.fore;
+	change_or_make_gc(&BACK_GC(ST_MENU_ACTIVE_GCS(ms)), gcm, &gcv);
+	gcv.foreground = c_active.hilight;
+	gcv.background = c_active.shadow;
+	change_or_make_gc(&HILIGHT_GC(ST_MENU_ACTIVE_GCS(ms)), gcm, &gcv);
+	gcv.foreground = c_active.shadow;
+	gcv.background = c_active.hilight;
+	change_or_make_gc(&SHADOW_GC(ST_MENU_ACTIVE_GCS(ms)), gcm, &gcv);
+	/* update stipple menu gcs */
+	SHADOW_GC(ST_MENU_STIPPLE_GCS(ms)) =
+		SHADOW_GC(ST_MENU_INACTIVE_GCS(ms));
 	if (Pdepth < 2)
 	{
 		gcv.fill_style = FillStippled;
 		gcv.stipple = Scr.gray_bitmap;
 		/* no need to reset fg/bg, FillStipple wins */
 		gcm |= GCStipple | GCFillStyle;
-	}
-	else if (ST_HAS_GREYED_CSET(ms))
-	{
-		gcv.foreground = greyed_cs->fg;
-		gcv.background = greyed_cs->bg;
+		HILIGHT_GC(ST_MENU_STIPPLE_GCS(ms)) =
+			SHADOW_GC(ST_MENU_INACTIVE_GCS(ms));
 	}
 	else
 	{
-		gcv.foreground = ST_MENU_STIPPLE_COLORS(ms).fore;
-		gcv.background = ST_MENU_STIPPLE_COLORS(ms).back;
-	}
-	if (ST_MENU_STIPPLE_GC(ms))
-	{
-		XChangeGC(dpy, ST_MENU_STIPPLE_GC(ms), gcm, &gcv);
-	}
-	else
-	{
-		ST_MENU_STIPPLE_GC(ms) =
-			fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
+		gcv.foreground = c_stipple.fore;
+		gcv.background = c_stipple.back;
+		HILIGHT_GC(ST_MENU_STIPPLE_GCS(ms)) =
+			HILIGHT_GC(ST_MENU_INACTIVE_GCS(ms));
 	}
+	change_or_make_gc(&FORE_GC(ST_MENU_STIPPLE_GCS(ms)), gcm, &gcv);
+	BACK_GC(ST_MENU_STIPPLE_GCS(ms)) = BACK_GC(ST_MENU_INACTIVE_GCS(ms));
 
 	return;
 }
@@ -685,6 +641,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 	FlocaleFont *new_font = NULL;
 	int i;
 	KeyCode keycode;
+	FvwmPictureAttributes fpa;
 
 	action = GetNextToken(action, &name);
 	if (!name)
@@ -778,7 +735,8 @@ void menustyle_parse_style(F_CMD_ARGS)
 				ST_TITLE_UNDERLINES(tmpms) = 1;
 				ST_HAS_LONG_SEPARATORS(tmpms) = 0;
 				ST_HAS_TRIANGLE_RELIEF(tmpms) = 1;
-				ST_DO_HILIGHT(tmpms) = 0;
+				ST_DO_HILIGHT_BACK(tmpms) = 0;
+				ST_DO_HILIGHT_FORE(tmpms) = 0;
 			}
 			else if (i == 1)
 			{
@@ -792,7 +750,8 @@ void menustyle_parse_style(F_CMD_ARGS)
 				ST_TITLE_UNDERLINES(tmpms) = 2;
 				ST_HAS_LONG_SEPARATORS(tmpms) = 1;
 				ST_HAS_TRIANGLE_RELIEF(tmpms) = 1;
-				ST_DO_HILIGHT(tmpms) = 0;
+				ST_DO_HILIGHT_BACK(tmpms) = 0;
+				ST_DO_HILIGHT_FORE(tmpms) = 0;
 			}
 			else /* i == 2 */
 			{
@@ -806,10 +765,14 @@ void menustyle_parse_style(F_CMD_ARGS)
 				ST_TITLE_UNDERLINES(tmpms) = 1;
 				ST_HAS_LONG_SEPARATORS(tmpms) = 0;
 				ST_HAS_TRIANGLE_RELIEF(tmpms) = 0;
-				ST_DO_HILIGHT(tmpms) = 1;
+				ST_DO_HILIGHT_BACK(tmpms) = 1;
+				ST_DO_HILIGHT_FORE(tmpms) = 1;
 			}
 
 			/* common settings */
+			ST_CSET_MENU(tmpms) = -1;
+			ST_CSET_ACTIVE(tmpms) = -1;
+			ST_CSET_GREYED(tmpms) = -1;
 			ST_BORDER_WIDTH(tmpms) = DEFAULT_MENU_BORDER_WIDTH;
 			ST_ACTIVE_AREA_PERCENT(tmpms) =
 				DEFAULT_MENU_POPUP_NOW_RATIO;
@@ -835,7 +798,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 			has_gc_changed = True;
 			if (ST_HAS_SIDE_COLOR(tmpms) == 1)
 			{
-				FreeColors(&ST_SIDE_COLOR(tmpms), 1);
+				FreeColors(&ST_SIDE_COLOR(tmpms), 1, True);
 				ST_HAS_SIDE_COLOR(tmpms) = 0;
 			}
 			ST_HAS_SIDE_COLOR(tmpms) = 0;
@@ -854,7 +817,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 			break;
 
 		case 3: /* Foreground */
-			FreeColors(&ST_MENU_COLORS(tmpms).fore, 1);
+			FreeColors(&ST_MENU_COLORS(tmpms).fore, 1, True);
 			if (arg1)
 			{
 				ST_MENU_COLORS(tmpms).fore = GetColor(arg1);
@@ -868,7 +831,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 			break;
 
 		case 4: /* Background */
-			FreeColors(&ST_MENU_COLORS(tmpms).back, 1);
+			FreeColors(&ST_MENU_COLORS(tmpms).back, 1, True);
 			if (arg1)
 			{
 				ST_MENU_COLORS(tmpms).back = GetColor(arg1);
@@ -885,7 +848,8 @@ void menustyle_parse_style(F_CMD_ARGS)
 			if (ST_HAS_STIPPLE_FORE(tmpms))
 			{
 				FreeColors(
-					&ST_MENU_STIPPLE_COLORS(tmpms).fore, 1);
+					&ST_MENU_STIPPLE_COLORS(tmpms).fore, 1,
+					True);
 			}
 			if (arg1 == NULL)
 			{
@@ -904,7 +868,8 @@ void menustyle_parse_style(F_CMD_ARGS)
 			if (ST_HAS_ACTIVE_BACK(tmpms))
 			{
 				FreeColors(
-					&ST_MENU_ACTIVE_COLORS(tmpms).back, 1);
+					&ST_MENU_ACTIVE_COLORS(tmpms).back, 1,
+					True);
 			}
 			if (arg1 == NULL)
 			{
@@ -916,12 +881,12 @@ void menustyle_parse_style(F_CMD_ARGS)
 					GetColor(arg1);
 				ST_HAS_ACTIVE_BACK(tmpms) = 1;
 			}
-			ST_DO_HILIGHT(tmpms) = 1;
+			ST_DO_HILIGHT_BACK(tmpms) = 1;
 			has_gc_changed = True;
 			break;
 
 		case 7: /* HilightBackOff */
-			ST_DO_HILIGHT(tmpms) = 0;
+			ST_DO_HILIGHT_BACK(tmpms) = 0;
 			has_gc_changed = True;
 			break;
 
@@ -929,7 +894,8 @@ void menustyle_parse_style(F_CMD_ARGS)
 			if (ST_HAS_ACTIVE_FORE(tmpms))
 			{
 				FreeColors(
-					&ST_MENU_ACTIVE_COLORS(tmpms).fore, 1);
+					&ST_MENU_ACTIVE_COLORS(tmpms).fore, 1,
+					True);
 			}
 			if (arg1 == NULL)
 			{
@@ -941,11 +907,12 @@ void menustyle_parse_style(F_CMD_ARGS)
 					GetColor(arg1);
 				ST_HAS_ACTIVE_FORE(tmpms) = 1;
 			}
+			ST_DO_HILIGHT_FORE(tmpms) = 1;
 			has_gc_changed = True;
 			break;
 
 		case 9: /* ActiveForeOff */
-			ST_HAS_ACTIVE_FORE(tmpms) = 0;
+			ST_DO_HILIGHT_FORE(tmpms) = 0;
 			has_gc_changed = True;
 			break;
 
@@ -1102,9 +1069,9 @@ void menustyle_parse_style(F_CMD_ARGS)
 			}
 			if (arg1)
 			{
+				fpa.mask = (Pdepth <= 8)?  FPAM_DITHER:0;
 				ST_SIDEPIC(tmpms) = PCacheFvwmPicture(
-					dpy, Scr.NoFocusWin, NULL, arg1,
-					Scr.ColorLimit);
+					dpy, Scr.NoFocusWin, NULL, arg1, fpa);
 				if (!ST_SIDEPIC(tmpms))
 				{
 					fvwm_msg(WARN, "NewMenuStyle",
@@ -1117,7 +1084,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 		case 32: /* SideColor */
 			if (ST_HAS_SIDE_COLOR(tmpms) == 1)
 			{
-				FreeColors(&ST_SIDE_COLOR(tmpms), 1);
+				FreeColors(&ST_SIDE_COLOR(tmpms), 1, True);
 				ST_HAS_SIDE_COLOR(tmpms) = 0;
 			}
 			if (arg1)
@@ -1222,6 +1189,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 			    *val < 0)
 			{
 				ST_HAS_MENU_CSET(tmpms) = 0;
+				ST_CSET_MENU(tmpms) = -1;
 			}
 			else
 			{
@@ -1236,6 +1204,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 			    *val < 0)
 			{
 				ST_HAS_ACTIVE_CSET(tmpms) = 0;
+				ST_CSET_ACTIVE(tmpms) = -1;
 			}
 			else
 			{
@@ -1251,6 +1220,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 			    *val < 0)
 			{
 				ST_HAS_GREYED_CSET(tmpms) = 0;
+				ST_CSET_GREYED(tmpms) = -1;
 			}
 			else
 			{
@@ -1333,7 +1303,7 @@ void menustyle_parse_style(F_CMD_ARGS)
 	if (has_gc_changed)
 	{
 		menustyle_update(tmpms);
-	} /* if (has_gc_changed) */
+	}
 
 	if (default_menu_style == NULL)
 	{
@@ -1370,6 +1340,7 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 	char *buffer;
 	MenuStyle *origms;
 	MenuStyle *destms;
+	FvwmPictureAttributes fpa;
 
 	origname = PeekToken(action, &action);
 	if (origname == NULL)
@@ -1444,15 +1415,15 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 	   strcture. Use  the same order as in menustyle_parse_style */
 
 	/* menu colors */
-	FreeColors(&ST_MENU_COLORS(destms).fore, 1);
-	FreeColors(&ST_MENU_COLORS(destms).back, 1);
+	FreeColors(&ST_MENU_COLORS(destms).fore, 1, True);
+	FreeColors(&ST_MENU_COLORS(destms).back, 1, True);
 	memcpy(&ST_MENU_COLORS(destms), &ST_MENU_COLORS(origms),
 	       sizeof(ColorPair));
 
 	/* Greyed */
 	if (ST_HAS_STIPPLE_FORE(destms))
 	{
-		FreeColors(&ST_MENU_STIPPLE_COLORS(destms).fore, 1);
+		FreeColors(&ST_MENU_STIPPLE_COLORS(destms).fore, 1, True);
 	}
 	ST_HAS_STIPPLE_FORE(destms) = ST_HAS_STIPPLE_FORE(origms);
 	if (ST_HAS_STIPPLE_FORE(origms))
@@ -1464,7 +1435,7 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 	/* HilightBack */
 	if (ST_HAS_ACTIVE_BACK(destms))
 	{
-		FreeColors(&ST_MENU_ACTIVE_COLORS(destms).back, 1);
+		FreeColors(&ST_MENU_ACTIVE_COLORS(destms).back, 1, True);
 	}
 	ST_HAS_ACTIVE_BACK(destms) = ST_HAS_ACTIVE_BACK(origms);
 	if (ST_HAS_ACTIVE_BACK(origms))
@@ -1472,13 +1443,12 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 		memcpy(&ST_MENU_ACTIVE_COLORS(destms).back,
 		       &ST_MENU_ACTIVE_COLORS(origms).back, sizeof(Pixel));
 	}
-
-	ST_DO_HILIGHT(destms) = ST_DO_HILIGHT(origms);
+	ST_DO_HILIGHT_BACK(destms) = ST_DO_HILIGHT_BACK(origms);
 
 	/* ActiveFore */
 	if (ST_HAS_ACTIVE_FORE(destms))
 	{
-		FreeColors(&ST_MENU_ACTIVE_COLORS(destms).fore, 1);
+		FreeColors(&ST_MENU_ACTIVE_COLORS(destms).fore, 1, True);
 	}
 	ST_HAS_ACTIVE_FORE(destms) = ST_HAS_ACTIVE_FORE(origms);
 	if (ST_HAS_ACTIVE_FORE(origms))
@@ -1486,6 +1456,7 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 		memcpy(&ST_MENU_ACTIVE_COLORS(destms).fore,
 		       &ST_MENU_ACTIVE_COLORS(origms).fore, sizeof(Pixel));
 	}
+	ST_DO_HILIGHT_FORE(destms) = ST_DO_HILIGHT_FORE(origms);
 
 	/* Hilight3D */
 	ST_RELIEF_THICKNESS(destms) = ST_RELIEF_THICKNESS(origms);
@@ -1529,21 +1500,24 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 		ST_FACE(destms).type = SolidMenu;
 		break;
 	case GradientMenu:
-		ST_FACE(destms).u.grad.pixels =
-			(Pixel *)safemalloc(sizeof(Pixel) *
+		ST_FACE(destms).u.grad.xcs =
+			(XColor *)safemalloc(sizeof(XColor) *
 					    ST_FACE(origms).u.grad.npixels);
-		memcpy(ST_FACE(destms).u.grad.pixels,
-		       ST_FACE(origms).u.grad.pixels,
+		memcpy(ST_FACE(destms).u.grad.xcs,
+		       ST_FACE(origms).u.grad.xcs,
 		       sizeof(Pixel) * ST_FACE(origms).u.grad.npixels);
 		ST_FACE(destms).u.grad.npixels = ST_FACE(origms).u.grad.npixels;
+		ST_FACE(destms).u.grad.do_dither =
+			ST_FACE(origms).u.grad.do_dither;
 		ST_FACE(destms).type = GradientMenu;
 		ST_FACE(destms).gradient_type = ST_FACE(origms).gradient_type;
 		break;
 	case PixmapMenu:
 	case TiledPixmapMenu:
+		fpa.mask = (Pdepth <= 8)?  FPAM_DITHER:0;
 		ST_FACE(destms).u.p = PCacheFvwmPicture(
 			dpy, Scr.NoFocusWin, NULL, ST_FACE(origms).u.p->name,
-			Scr.ColorLimit);
+			fpa);
 		memcpy(&ST_FACE(destms).u.back, &ST_FACE(origms).u.back,
 		       sizeof(Pixel));
 		ST_FACE(destms).type = ST_FACE(origms).type;
@@ -1578,15 +1552,16 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 	}
 	if (ST_SIDEPIC(origms))
 	{
+		fpa.mask = (Pdepth <= 8)?  FPAM_DITHER:0;
 		ST_SIDEPIC(destms) = PCacheFvwmPicture(
 			dpy, Scr.NoFocusWin, NULL, ST_SIDEPIC(origms)->name,
-			Scr.ColorLimit);
+			fpa);
 	}
 
 	/* side color */
 	if (ST_HAS_SIDE_COLOR(destms) == 1)
 	{
-		FreeColors(&ST_SIDE_COLOR(destms), 1);
+		FreeColors(&ST_SIDE_COLOR(destms), 1, True);
 	}
 	ST_HAS_SIDE_COLOR(destms) = ST_HAS_SIDE_COLOR(origms);
 	if (ST_HAS_SIDE_COLOR(origms) == 1)
