@@ -149,7 +149,7 @@ Bool GrabEm(int cursor, int grab_context)
 				    dpy, (PressedW != None) ?
 				    PressedW : Scr.NoFocusWin, True,
 				    GRAB_EVMASK, GrabModeAsync, GrabModeAsync,
-				    None, Scr.FvwmCursors[CRS_DEFAULT],
+				    Scr.Root, Scr.FvwmCursors[CRS_DEFAULT],
 				    CurrentTime) != GrabSuccess)
 			{
 				return False;
@@ -199,7 +199,7 @@ Bool GrabEm(int cursor, int grab_context)
 	while (i < rep &&
 	       (val = XGrabPointer(
 		      dpy, grab_win, True, GRAB_EVMASK, GrabModeAsync,
-		      GrabModeAsync, None,
+		      GrabModeAsync, Scr.Root,
 		      (grab_context == GRAB_FREEZE_CURSOR) ?
 		      None : Scr.FvwmCursors[cursor], CurrentTime) !=
 		GrabSuccess))
