@@ -140,29 +140,29 @@ static int get_outline_rects(
 
 	for (i = 0; i < n; i++)
 	{
-		rects[i+i].x = x + i;
-		rects[i+i].y = y + i;
-		rects[i+i].width = width - (i << 1);
-		rects[i+i].height = height - (i << 1);
+		rects[i].x = x + i;
+		rects[i].y = y + i;
+		rects[i].width = width - (i << 1);
+		rects[i].height = height - (i << 1);
 	}
 	if (width - (n << 1) >= 5 && height - (n << 1) >= 5)
 	{
 		if (width - (n << 1) >= 10)
 		{
 			int off = (width - (n << 1)) / 3 + n;
-			rects[i+i].x = x + off;
-			rects[i+i].y = y + n;
-			rects[i+i].width = width - (off << 1);
-			rects[i+i].height = height - (n << 1);
+			rects[i].x = x + off;
+			rects[i].y = y + n;
+			rects[i].width = width - (off << 1);
+			rects[i].height = height - (n << 1);
 			i++;
 		}
 		if (height - (n << 1) >= 10)
 		{
 			int off = (height - (n << 1)) / 3 + n;
-			rects[i+i].x = x + n;
-			rects[i+i].y = y + off;
-			rects[i+i].width = width - (n << 1);
-			rects[i+i].height = height - (off << 1);
+			rects[i].x = x + n;
+			rects[i].y = y + off;
+			rects[i].width = width - (n << 1);
+			rects[i].height = height - (off << 1);
 			i++;
 		}
 	}
@@ -207,7 +207,7 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height)
 		move_resize_grid.flags.is_enabled = 0;
 		nrects +=
 			get_outline_rects(
-				&(rects[1]), move_resize_grid.geom.x,
+				&(rects[0]), move_resize_grid.geom.x,
 				move_resize_grid.geom.y,
 				move_resize_grid.geom.width,
 				move_resize_grid.geom.height);
@@ -219,11 +219,12 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height)
 		move_resize_grid.geom.y = y;
 		move_resize_grid.geom.width = width;
 		move_resize_grid.geom.height = height;
-		nrects += get_outline_rects(&(rects[0]), x, y, width, height);
+		nrects += get_outline_rects(
+			&(rects[nrects]), x, y, width, height);
 	}
 	if (nrects > 0)
 	{
-		XDrawRectangles(dpy, Scr.Root, Scr.XorGC, rects, 10);
+		XDrawRectangles(dpy, Scr.Root, Scr.XorGC, rects, nrects);
 		XFlush(dpy);
 	}
 
@@ -1337,7 +1338,7 @@ static void __move_window(F_CMD_ARGS, Bool do_animate, int mode)
 	}
 
 
-	if (mode == MOVE_PAGE && IS_STICKY_ON_PAGE(fw))
+	if (mode == MOVE_PAGE && IS_STICKY_ACROSS_PAGES(fw))
 	{
 		return;
 	}
@@ -2302,7 +2303,6 @@ Bool __move_loop(
 				}
 				paged++;
 			}  /* end while (paged) */
-
 			break;
 
 		case Expose:
@@ -2829,6 +2829,7 @@ static Bool __resize_window(F_CMD_ARGS)
 	long evmask;
 	FvwmWindow *fw = exc->w.fw;
 
+/* 	fprintf(stderr,"dje: fvwm entering resize window\n"); */
 	bad_window = False;
 	ResizeWindow = FW_W_FRAME(fw);
 	if (FQueryPointer(
@@ -2881,6 +2882,7 @@ static Bool __resize_window(F_CMD_ARGS)
 	{
 		rectangle new_g;
 
+/* 		fprintf(stderr,"dje: resize args equal 2\n"); */
 		/* size will be less or equal to requested */
 		new_g = fw->frame_g;
 		constrain_size(
@@ -3185,6 +3187,7 @@ static Bool __resize_window(F_CMD_ARGS)
 				/* Fake an event to force window reposition */
 				ev.type = MotionNotify;
 				ev.xmotion.time = fev_get_evtime();
+/* 				fprintf(stderr,"dje: force redraw\n"); */
 				fForceRedraw = True;
 				break;
 			}
@@ -3308,6 +3311,7 @@ static Bool __resize_window(F_CMD_ARGS)
 			}
 			fForceRedraw = False;
 			is_done = True;
+			break;
 
 		case PropertyNotify:
 		{
@@ -3318,6 +3322,7 @@ static Bool __resize_window(F_CMD_ARGS)
 			ea.exc = exc_clone_context(exc, &ecc, ECC_ETRIGGER);
 			HandlePropertyNotify(&ea);
 			exc_destroy_context(ea.exc);
+			is_done = True;
 			break;
 		}
 
@@ -3327,9 +3332,11 @@ static Bool __resize_window(F_CMD_ARGS)
 		if (!is_done)
 		{
 			if (!do_resize_opaque)
+			{
 				/* must undraw the rubber band in case the
 				 * event causes some drawing */
 				switch_move_resize_grid(False);
+			}
 			dispatch_event(&ev);
 			if (!do_resize_opaque)
 			{
@@ -3454,10 +3461,10 @@ void CMD_Resize(F_CMD_ARGS)
 
 /* ----------------------------- maximizing code --------------------------- */
 
-Bool is_window_sticky_on_page(FvwmWindow *fw)
+Bool is_window_sticky_across_pages(FvwmWindow *fw)
 {
-	if (IS_STICKY_ON_PAGE(fw) ||
-	    (IS_ICONIFIED(fw) && IS_ICON_STICKY_ON_PAGE(fw)))
+	if (IS_STICKY_ACROSS_PAGES(fw) ||
+	    (IS_ICONIFIED(fw) && IS_ICON_STICKY_ACROSS_PAGES(fw)))
 	{
 		return True;
 	}
@@ -3467,10 +3474,10 @@ Bool is_window_sticky_on_page(FvwmWindow *fw)
 	}
 }
 
-Bool is_window_sticky_on_desk(FvwmWindow *fw)
+Bool is_window_sticky_across_desks(FvwmWindow *fw)
 {
-	if (IS_STICKY_ON_DESK(fw) ||
-	    (IS_ICONIFIED(fw) && IS_ICON_STICKY_ON_DESK(fw)))
+	if (IS_STICKY_ACROSS_DESKS(fw) ||
+	    (IS_ICONIFIED(fw) && IS_ICON_STICKY_ACROSS_DESKS(fw)))
 	{
 		return True;
 	}
@@ -3547,7 +3554,7 @@ static void MaximizeHeight(
 	{
 		if (cwin == win ||
 		    (cwin->Desk != win->Desk &&
-		     !is_window_sticky_on_desk(cwin)) ||
+		     !is_window_sticky_across_desks(cwin)) ||
 		    (layer_grow && cwin->layer <= win->layer))
 		{
 			continue;
@@ -3561,7 +3568,7 @@ static void MaximizeHeight(
 		y21 = g.y;
 		x22 = x21 + g.width;
 		y22 = y21 + g.height;
-		if (is_window_sticky_on_page(cwin))
+		if (is_window_sticky_across_pages(cwin))
 		{
 			move_sticky_window_to_same_page(
 				&x21, &x22, &new_y1, &new_y2, x11, x12, y11,
@@ -3618,7 +3625,7 @@ static void MaximizeWidth(
 	{
 		if (cwin == win ||
 		    (cwin->Desk != win->Desk &&
-		     !is_window_sticky_on_desk(cwin)) ||
+		     !is_window_sticky_across_desks(cwin)) ||
 		    (layer_grow && cwin->layer <= win->layer))
 		{
 			continue;
@@ -3632,7 +3639,7 @@ static void MaximizeWidth(
 		y21 = g.y;
 		x22 = x21 + g.width;
 		y22 = y21 + g.height;
-		if (is_window_sticky_on_page(cwin))
+		if (is_window_sticky_across_pages(cwin))
 		{
 			move_sticky_window_to_same_page(
 				&new_x1, &new_x2, &y21, &y22, x11, x12, y11,
@@ -4059,18 +4066,18 @@ void CMD_ResizeMoveMaximize(F_CMD_ARGS)
 
 /* ----------------------------- stick code -------------------------------- */
 
-int stick_page(F_CMD_ARGS, int toggle)
+int stick_across_pages(F_CMD_ARGS, int toggle)
 {
 	FvwmWindow *fw = exc->w.fw;
 
-	if ((toggle == 1 && IS_STICKY_ON_PAGE(fw)) ||
-	    (toggle == 0 && !IS_STICKY_ON_PAGE(fw)))
+	if ((toggle == 1 && IS_STICKY_ACROSS_PAGES(fw)) ||
+	    (toggle == 0 && !IS_STICKY_ACROSS_PAGES(fw)))
 	{
 		return 0;
 	}
-	if (IS_STICKY_ON_PAGE(fw))
+	if (IS_STICKY_ACROSS_PAGES(fw))
 	{
-		SET_STICKY_ON_PAGE(fw, 0);
+		SET_STICKY_ACROSS_PAGES(fw, 0);
 	}
 	else
 	{
@@ -4079,25 +4086,25 @@ int stick_page(F_CMD_ARGS, int toggle)
 			action = "";
 			__move_window(F_PASS_ARGS, FALSE, MOVE_PAGE);
 		}
-		SET_STICKY_ON_PAGE(fw, 1);
+		SET_STICKY_ACROSS_PAGES(fw, 1);
 	}
 
 	return 1;
 }
 
-int stick_desk(F_CMD_ARGS, int toggle)
+int stick_across_desks(F_CMD_ARGS, int toggle)
 {
 	FvwmWindow *fw = exc->w.fw;
 
-	if ((toggle == 1 && IS_STICKY_ON_DESK(fw)) ||
-	    (toggle == 0 && !IS_STICKY_ON_DESK(fw)))
+	if ((toggle == 1 && IS_STICKY_ACROSS_DESKS(fw)) ||
+	    (toggle == 0 && !IS_STICKY_ACROSS_DESKS(fw)))
 	{
 		return 0;
 	}
 
-	if (IS_STICKY_ON_DESK(fw))
+	if (IS_STICKY_ACROSS_DESKS(fw))
 	{
-		SET_STICKY_ON_DESK(fw, 0);
+		SET_STICKY_ACROSS_DESKS(fw, 0);
 		fw->Desk = Scr.CurrentDesk;
 		GNOME_SetDeskCount();
 		GNOME_SetDesk(fw);
@@ -4108,7 +4115,7 @@ int stick_desk(F_CMD_ARGS, int toggle)
 		{
 			do_move_window_to_desk(fw, Scr.CurrentDesk);
 		}
-		SET_STICKY_ON_DESK(fw, 1);
+		SET_STICKY_ACROSS_DESKS(fw, 1);
 	}
 
 	return 1;
@@ -4134,13 +4141,13 @@ static void __handle_stick_exit(
 	return;
 }
 
-void handle_stick_page(
+void handle_stick_across_pages(
 	F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
 {
 	FvwmWindow *fw = exc->w.fw;
 	int did_change;
 
-	did_change = stick_page(F_PASS_ARGS, toggle);
+	did_change = stick_across_pages(F_PASS_ARGS, toggle);
 	if (did_change)
 	{
 		__handle_stick_exit(fw, do_not_draw, do_silently);
@@ -4149,13 +4156,13 @@ void handle_stick_page(
 	return;
 }
 
-void handle_stick_desk(
+void handle_stick_across_desks(
 	F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
 {
 	FvwmWindow *fw = exc->w.fw;
 	int did_change;
 
-	did_change = stick_desk(F_PASS_ARGS, toggle);
+	did_change = stick_across_desks(F_PASS_ARGS, toggle);
 	if (did_change)
 	{
 		__handle_stick_exit(fw, do_not_draw, do_silently);
@@ -4172,8 +4179,8 @@ void handle_stick(
 	int did_change;
 
 	did_change = 0;
-	did_change |= stick_desk(F_PASS_ARGS, toggle_desk);
-	did_change |= stick_page(F_PASS_ARGS, toggle_page);
+	did_change |= stick_across_desks(F_PASS_ARGS, toggle_desk);
+	did_change |= stick_across_pages(F_PASS_ARGS, toggle_page);
 	if (did_change)
 	{
 		__handle_stick_exit(fw, do_not_draw, do_silently);
@@ -4187,8 +4194,8 @@ void CMD_Stick(F_CMD_ARGS)
 	int toggle;
 
 	toggle = ParseToggleArgument(action, &action, -1, 0);
-	if (toggle == -1 &&
-	    IS_STICKY_ON_DESK(exc->w.fw) != IS_STICKY_ON_PAGE(exc->w.fw))
+	if (toggle == -1 && IS_STICKY_ACROSS_DESKS(exc->w.fw) !=
+	    IS_STICKY_ACROSS_PAGES(exc->w.fw))
 	{
 		/* don't switch between only stickypage and only stickydesk.
 		 * rather switch it off completely */
@@ -4199,22 +4206,22 @@ void CMD_Stick(F_CMD_ARGS)
 	return;
 }
 
-void CMD_StickPage(F_CMD_ARGS)
+void CMD_StickAcrossPages(F_CMD_ARGS)
 {
 	int toggle;
 
 	toggle = ParseToggleArgument(action, &action, -1, 0);
-	handle_stick_page(F_PASS_ARGS, toggle, 0, 0);
+	handle_stick_across_pages(F_PASS_ARGS, toggle, 0, 0);
 
 	return;
 }
 
-void CMD_StickDesk(F_CMD_ARGS)
+void CMD_StickAcrossDesks(F_CMD_ARGS)
 {
 	int toggle;
 
 	toggle = ParseToggleArgument(action, &action, -1, 0);
-	handle_stick_desk(F_PASS_ARGS, toggle, 0, 0);
+	handle_stick_across_desks(F_PASS_ARGS, toggle, 0, 0);
 
 	return;
 }
