@@ -5,12 +5,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "config.h"
@@ -31,14 +31,14 @@
 
 typedef struct
 {
-  int id;
-  Time time_to_execute;
-  Window window;
-  char *command;
-  struct
-  {
-    unsigned is_scheduled_for_destruction : 1;
-  } flags;
+	int id;
+	Time time_to_execute;
+	Window window;
+	char *command;
+	struct
+	{
+		unsigned is_scheduled_for_destruction : 1;
+	} flags;
 } sq_object_type;
 
 static int last_schedule_id = 0;
@@ -111,8 +111,6 @@ static void deschedule(int *pid)
 	{
 		id = last_schedule_id;
 	}
-
-
 	/* deschedule matching jobs */
 	fqueue_remove_or_operate_all(&sq, deschedule_obj_func, (void *)&id);
 
@@ -166,31 +164,25 @@ static int execute_obj_func(void *object, void *args)
 		if (obj->command != NULL)
 		{
 			/* execute the command */
-			exec_func_args_type efa;
-			XEvent ev;
-			FvwmWindow *fw;
+			fvwm_cond_func_rc cond_rc;
+			const exec_context_t *exc;
+			exec_context_changes_t ecc;
+			exec_context_change_mask_t mask =
+				ECC_TYPE | ECC_WCONTEXT;
 
-			obj->flags.is_scheduled_for_destruction = 1;
-			memset(&efa, 0, sizeof(efa));
-			memset(&ev, 0, sizeof(ev));
-			efa.eventp = &ev;
-			efa.fw = NULL;
-			efa.action = obj->command;
-			efa.args = NULL;
-			if (XFindContext(dpy, obj->window, FvwmContext,
-					 (caddr_t *)&fw) == XCNOENT)
+			ecc.type = EXCT_SCHEDULE;
+			ecc.w.wcontext = C_ROOT;
+			if (XFindContext(
+				    dpy, obj->window, FvwmContext,
+				    (caddr_t *)&ecc.w.fw) != XCNOENT)
 			{
-				fw = NULL;
-				efa.context = C_ROOT;
+				ecc.w.wcontext = C_WINDOW;
+				mask |= ECC_FW;
 			}
-			else
-			{
-				efa.context = C_WINDOW;
-			}
-			efa.module = -1;
-			efa.flags.exec = 0;
-			execute_function(&efa);
+			exc = exc_create_context(&ecc, mask);
+			execute_function(&cond_rc, exc, obj->command, 0);
 			free(obj->command);
+			exc_destroy_context(exc);
 		}
 		free(obj);
 		XFlush(dpy);
@@ -226,7 +218,7 @@ int squeue_get_next_ms(void)
 	{
 		return -1;
 	}
-	if (cmp_times(lastTimestamp, obj->time_to_execute) >= 0)
+	if (cmp_times(fev_get_evtime(), obj->time_to_execute) >= 0)
 	{
 		/* jobs pending to be executed immediately */
 		ms = 0;
@@ -234,7 +226,7 @@ int squeue_get_next_ms(void)
 	else
 	{
 		/* execute jobs later */
-		ms =  obj->time_to_execute - lastTimestamp;
+		ms =  obj->time_to_execute - fev_get_evtime();
 	}
 
 	return ms;
@@ -245,7 +237,6 @@ int squeue_get_next_id(void)
 	return next_schedule_id;
 }
 
-
 int squeue_get_last_id(void)
 {
 	return last_schedule_id;
@@ -261,6 +252,7 @@ void CMD_Schedule(F_CMD_ARGS)
 	int id;
 	int *pid;
 	int n;
+	FvwmWindow * const fw = exc->w.fw;
 
 	/* get the time to execute */
 	n = GetIntegerArguments(action, &action, &ms, 1);
@@ -278,9 +270,9 @@ void CMD_Schedule(F_CMD_ARGS)
 	/* eats up way too much cpu if schedule is used excessively */
 	current_time = get_server_time();
 #else
-	/* with this version, scheduled commands may be executed later than
+	/* with this version, scheduled commands may be executed earlier than
 	 * intended. */
-	current_time = lastTimestamp;
+	current_time = fev_get_evtime();
 #endif
 	time = current_time + (Time)ms;
 	/* get the job group id to schedule */
