@@ -38,6 +38,12 @@
 	((sf)->has_icon_boxes)
 #define SHAS_ICON_SIZE_LIMITS(sf) \
 	((sf)->has_icon_size_limits)
+#define SHAS_ICON_BACKGROUND_PADDING(sf) \
+	((sf)->has_icon_background_padding)
+#define SHAS_ICON_BACKGROUND_RELIEF(sf) \
+	((sf)->has_icon_background_relief)
+#define SHAS_ICON_TITLE_RELIEF(sf) \
+	((sf)->has_icon_title_relief)
 #define SHAS_MAX_WINDOW_SIZE(sf) \
 	((sf)->has_max_window_size)
 #define SHAS_WINDOW_SHADE_STEPS(sf) \
@@ -72,6 +78,12 @@
 	((sf)->use_border_colorset)
 #define SUSE_BORDER_COLORSET_HI(sf) \
 	((sf)->use_border_colorset_hi)
+#define SUSE_ICON_TITLE_COLORSET(sf) \
+	((sf)->use_icon_title_colorset)
+#define SUSE_ICON_TITLE_COLORSET_HI(sf) \
+	((sf)->use_icon_title_colorset_hi)
+#define SUSE_ICON_BACKGROUND_COLORSET(sf) \
+	((sf)->use_icon_background_colorset)
 #define SUSE_LAYER(sf) \
 	((sf)->use_layer)
 #define SUSE_NO_PPOSITION(sf) \
@@ -126,14 +138,14 @@
 	((c).title_dir)
 #define S_SET_TITLE_DIR(c,x) \
 	((c).title_dir = x)
-#define S_IS_STICKY_ON_PAGE(c) \
-	((c).is_sticky_on_page)
-#define S_SET_IS_STICKY_ON_PAGE(c,x) \
-	((c).is_sticky_on_page = !!(x))
-#define S_IS_STICKY_ON_DESK(c) \
-	((c).is_sticky_on_desk)
-#define S_SET_IS_STICKY_ON_DESK(c,x) \
-	((c).is_sticky_on_desk = !!(x))
+#define S_IS_STICKY_ACROSS_PAGES(c) \
+	((c).is_sticky_across_pages)
+#define S_SET_IS_STICKY_ACROSS_PAGES(c,x) \
+	((c).is_sticky_across_pages = !!(x))
+#define S_IS_STICKY_ACROSS_DESKS(c) \
+	((c).is_sticky_across_desks)
+#define S_SET_IS_STICKY_ACROSS_DESKS(c,x) \
+	((c).is_sticky_across_desks = !!(x))
 #define S_DO_CIRCULATE_SKIP(c) \
 	((c).s.do_circulate_skip)
 #define S_SET_DO_CIRCULATE_SKIP(c,x) \
@@ -250,14 +262,22 @@
 	((c).s.is_fixed_ppos)
 #define S_SET_IS_FIXED_PPOS(c,x) \
 	((c).s.is_fixed_ppos = !!(x))
-#define S_IS_ICON_STICKY_ON_PAGE(c) \
-	((c).s.is_icon_sticky_on_page)
-#define S_SET_IS_ICON_STICKY_ON_PAGE(c,x) \
-	((c).s.is_icon_sticky_on_page = !!(x))
-#define S_IS_ICON_STICKY_ON_DESK(c) \
-	((c).s.is_icon_sticky_on_desk)
-#define S_SET_IS_ICON_STICKY_ON_DESK(c,x) \
-	((c).s.is_icon_sticky_on_desk = !!(x))
+#define S_SET_IS_UNICONIFIABLE(c,x) \
+        ((c).s.is_uniconifiable = !!(x))
+#define S_SET_IS_UNMAXIMIZABLE(c,x) \
+        ((c).s.is_unmaximizable = !!(x))
+#define S_SET_IS_UNCLOSABLE(c,x) \
+        ((c).s.is_unclosable = !!(x))
+#define S_SET_MAXIMIZE_FIXED_SIZE_DISALLOWED(c,x) \
+        ((c).s.is_maximize_fixed_size_disallowed = !!(x))
+#define S_IS_ICON_STICKY_ACROSS_PAGES(c) \
+	((c).s.is_icon_sticky_across_pages)
+#define S_SET_IS_ICON_STICKY_ACROSS_PAGES(c,x) \
+	((c).s.is_icon_sticky_across_pages = !!(x))
+#define S_IS_ICON_STICKY_ACROSS_DESKS(c) \
+	((c).s.is_icon_sticky_across_desks)
+#define S_SET_IS_ICON_STICKY_ACROSS_DESKS(c,x) \
+	((c).s.is_icon_sticky_across_desks = !!(x))
 #define S_IS_ICON_SUPPRESSED(c) \
 	((c).s.is_icon_suppressed)
 #define S_SET_IS_ICON_SUPPRESSED(c,x) \
@@ -298,6 +318,10 @@
 	((c).s.windowshade_laziness)
 #define S_SET_WINDOWSHADE_LAZINESS(c,x) \
 	((c).s.windowshade_laziness = (x))
+#define S_USE_TITLE_DECOR_ROTATION(c) \
+	((c).s.use_title_decor_rotation)
+#define S_SET_USE_TITLE_DECOR_ROTATION(c,x) \
+	((c).s.use_title_decor_rotation = !!(x))
 #define S_DO_EWMH_MINI_ICON_OVERRIDE(c) \
 	((c).s.do_ewmh_mini_icon_override)
 #define S_SET_DO_EWMH_MINI_ICON_OVERRIDE(c,x) \
@@ -394,6 +418,18 @@
 	((s).border_colorset_hi)
 #define SSET_BORDER_COLORSET_HI(s,x) \
 	((s).border_colorset_hi = (x))
+#define SSET_ICON_TITLE_COLORSET(s,x) \
+	((s).icon_title_colorset = (x))
+#define SGET_ICON_TITLE_COLORSET(s) \
+	((s).icon_title_colorset)
+#define SSET_ICON_TITLE_COLORSET_HI(s,x) \
+	((s).icon_title_colorset_hi = (x))
+#define SGET_ICON_TITLE_COLORSET_HI(s) \
+	((s).icon_title_colorset_hi)
+#define SSET_ICON_BACKGROUND_COLORSET(s,x) \
+	((s).icon_background_colorset = (x))
+#define SGET_ICON_BACKGROUND_COLORSET(s) \
+	((s).icon_background_colorset)
 #define SGET_FLAGS_POINTER(s) \
 	(&((s).flags))
 #define SGET_BORDER_WIDTH(s) \
@@ -440,6 +476,22 @@
 	((s).max_icon_height)
 #define SSET_MAX_ICON_HEIGHT(s,x) \
 	((s).max_icon_height = (x))
+#define SGET_ICON_RESIZE_TYPE(s) \
+	((s).icon_resize_type)
+#define SSET_ICON_RESIZE_TYPE(s,x) \
+	((s).icon_resize_type = (x))
+#define SGET_ICON_BACKGROUND_RELIEF(s) \
+	((s).icon_background_relief)
+#define SSET_ICON_BACKGROUND_RELIEF(s,x) \
+	((s).icon_background_relief = (x))
+#define SGET_ICON_BACKGROUND_PADDING(s) \
+	((s).icon_background_padding)
+#define SSET_ICON_BACKGROUND_PADDING(s,x) \
+	((s).icon_background_padding = (x))
+#define SGET_ICON_TITLE_RELIEF(s) \
+	((s).icon_title_relief)
+#define SSET_ICON_TITLE_RELIEF(s,x) \
+	((s).icon_title_relief = (x))
 #define SGET_MAX_WINDOW_WIDTH(s) \
 	((s).max_window_width)
 #define SSET_MAX_WINDOW_WIDTH(s,x) \
@@ -506,6 +558,9 @@ void reset_style_changes(void);
 void update_style_colorset(int colorset);
 void update_window_color_style(FvwmWindow *fw, window_style *style);
 void update_window_color_hi_style(FvwmWindow *fw, window_style *style);
+void update_icon_title_cs_style(FvwmWindow *fw, window_style *pstyle);
+void update_icon_title_cs_hi_style(FvwmWindow *fw, window_style *pstyle);
+void update_icon_background_cs_style(FvwmWindow *fw, window_style *pstyle);
 void free_icon_boxes(icon_boxes *ib);
 
 #endif /* _STYLE_ */
