@@ -20,31 +20,28 @@
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "cursor.h"
-#include "functions.h"
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
-#include "defaults.h"
 #include "update.h"
 #include "style.h"
 #include "builtins.h"
-#include "colorset.h"
 #include "borders.h"
 #include "frame.h"
 #include "gnome.h"
 #include "ewmh.h"
 #include "icons.h"
-#include "focus.h"
 #include "geometry.h"
 #include "move_resize.h"
 #include "add_window.h"
 #include "module_interface.h"
 #include "focus.h"
 #include "stack.h"
+#include "icons.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -84,6 +81,8 @@ static void apply_window_updates(
 	short buttons;
 	Bool is_style_initialised = False;
 	rectangle frame_g;
+	const exec_context_t *exc;
+	exec_context_changes_t ecc;
 
 	frame_g.x = t->frame_g.x;
 	frame_g.y = t->frame_g.y;
@@ -126,18 +125,25 @@ static void apply_window_updates(
 	 *
 	 * These are a bit complicated because they can move windows to a
 	 * different page or desk. */
-	if (flags->do_update_stick_icon && IS_ICONIFIED(t) && !IS_STICKY(t))
+	ecc.type = EXCT_NULL;
+	ecc.w.fw = t;
+	ecc.w.w = FW_W_FRAME(t);
+	ecc.w.wcontext = C_FRAME;
+	exc = exc_create_context(
+		&ecc, ECC_TYPE | ECC_FW | ECC_W | ECC_WCONTEXT);
+	if (flags->do_update_stick_icon && IS_ICONIFIED(t) &&
+	    !(IS_STICKY_ACROSS_PAGES(t) || IS_STICKY_ACROSS_DESKS(t)))
 	{
-		if (IS_ICON_STICKY(pstyle))
+		if (IS_ICON_STICKY_ACROSS_PAGES(pstyle) ||
+		    IS_ICON_STICKY_ACROSS_DESKS(pstyle))
 		{
 			/* stick and unstick the window to force the icon on
 			 * the current page */
 			handle_stick(
-				NULL, &Event, FW_W_FRAME(t), t, C_FRAME, "", 0,
-				1);
-			handle_stick(
-				NULL, &Event, FW_W_FRAME(t), t, C_FRAME, "", 0,
-				0);
+				NULL, exc, "",
+				S_IS_STICKY_ACROSS_PAGES(SCF(*pstyle)),
+				S_IS_STICKY_ACROSS_DESKS(SCF(*pstyle)), 1, 1);
+			handle_stick(NULL, exc, "", 0, 0, 1, 0);
 		}
 		else
 		{
@@ -147,9 +153,10 @@ static void apply_window_updates(
 	else if (flags->do_update_stick)
 	{
 		handle_stick(
-			NULL, &Event, FW_W_FRAME(t), t, C_FRAME, "", 0,
-			S_IS_STICKY(SCF(*pstyle)));
+			NULL, exc, "", S_IS_STICKY_ACROSS_PAGES(SCF(*pstyle)),
+			S_IS_STICKY_ACROSS_DESKS(SCF(*pstyle)), 0, 0);
 	}
+	exc_destroy_context(exc);
 	if (FMiniIconsSupported && flags->do_update_mini_icon)
 	{
 		if (!HAS_EWMH_MINI_ICON(t) || DO_EWMH_MINI_ICON_OVERRIDE(t))
@@ -197,6 +204,10 @@ static void apply_window_updates(
 		setup_window_font(t, pstyle, flags->do_update_window_font);
 		flags->do_redecorate = True;
 	}
+	if (flags->do_update_title_text_dir)
+	{
+		flags->do_redecorate = True;
+	}
 	if (flags->do_redecorate || flags->do_update_title_dir)
 	{
 		size_borders b_old;
@@ -307,13 +318,13 @@ static void apply_window_updates(
 		flags->do_setup_frame = True;
 		flags->do_redraw_decoration = True;
 	}
-	if (flags->do_update_title_text_dir)
+	if (flags->do_update_rotated_title)
 	{
-		if (!flags->do_update_title_dir)
+		if (t->title_text_rotation != ROTATION_0)
 		{
-			setup_title_geometry(t, pstyle);
-		}
-		flags->do_redraw_decoration = True;
+			flags->do_setup_frame = True;
+			flags->do_redraw_decoration = True;
+		} 
 	}
 	if (0 && flags->do_update_title_dir)
 	{
@@ -387,14 +398,17 @@ static void apply_window_updates(
 	}
 	if (flags->do_setup_frame)
 	{
+		FvwmWindow *tmp;
+
 		setup_title_geometry(t, pstyle);
 		/* frame_force_setup_window needs to know if the window is
 		 * hilighted */
+		tmp = get_focus_window();
 		set_focus_window(focus_w);
 		frame_force_setup_window(
 			t, frame_g.x, frame_g.y, frame_g.width, frame_g.height,
 			True);
-		set_focus_window(NULL);
+		set_focus_window(tmp);
 		GNOME_SetWinArea(t);
 		EWMH_SetFrameStrut(t);
 	}
@@ -423,13 +437,41 @@ static void apply_window_updates(
 			flags->do_broadcast_focus = True;
 		}
 	}
-	if (flags->do_redraw_decoration)
+	if (flags->do_update_icon_title_cs_hi)
 	{
-		/* frame_redraw_decorations needs to know if the window is
-		 * hilighted */
-		set_focus_window(focus_w);
-		border_redraw_decorations(t);
-		set_focus_window(NULL);
+		if (t == focus_w && IS_ICONIFIED(t))
+		{
+			flags->do_redraw_icon = True;
+		}
+		update_icon_title_cs_hi_style(t, pstyle);
+	}
+	if (flags->do_update_icon_title_cs)
+	{
+		if (t != focus_w && IS_ICONIFIED(t))
+		{
+			flags->do_redraw_icon = True;
+		}
+		update_icon_title_cs_style(t, pstyle);
+	}
+	if (flags->do_update_icon_background_cs)
+	{
+		int old_cs = t->icon_background_cs;
+		
+		update_icon_background_cs_style(t, pstyle);
+		if ((old_cs < 0 && t->icon_background_cs >= 0) ||
+		    (old_cs >= 0 && t->icon_background_cs < 0))
+		{
+			flags->do_update_icon = True;
+		}
+		else
+		{
+			flags->do_redraw_icon = True;
+		}
+	}
+	if (flags->do_update_icon_size_limits)
+	{
+		setup_icon_size_limits(t, pstyle);
+		flags->do_update_icon = True;
 	}
 	if (flags->do_update_icon_font)
 	{
@@ -447,18 +489,24 @@ static void apply_window_updates(
 	}
 	if (flags->do_update_icon)
 	{
+		setup_icon_background_parameters(t, pstyle);
+		setup_icon_title_parameters(t, pstyle);
 		change_icon(t, pstyle);
 		flags->do_update_icon_placement = True;
 		flags->do_update_icon_title = False;
+		flags->do_redraw_icon = False;
 		flags->do_update_ewmh_icon = True;
 	}
+	if (flags->do_redraw_icon)
+	{
+		/* should not test if the window is iconified */
+		DrawIconWindow(t, True, True, False, True, NULL);;
+		flags->do_redraw_decoration = False;
+		flags->do_update_icon_title = False;
+	}
 	if (flags->do_update_icon_title)
 	{
 		RedoIconName(t);
-		if (IS_ICONIFIED(t))
-		{
-			DrawIconWindow(t);
-		}
 	}
 	if (flags->do_update_icon_placement)
 	{
@@ -469,8 +517,27 @@ static void apply_window_updates(
 			memset(&win_opts, 0, sizeof(win_opts));
 			SET_ICONIFIED(t, 0);
 			Iconify(t, &win_opts);
+			flags->do_redraw_decoration = False;
 		}
 	}
+	if (flags->do_redraw_decoration)
+	{
+		FvwmWindow *tmp;
+
+		/* frame_redraw_decorations needs to know if the window is
+		 * hilighted */
+		tmp = get_focus_window();
+		set_focus_window(focus_w);
+		if (IS_ICONIFIED(t))
+		{
+			DrawIconWindow(t, True, True, False, False, NULL);
+		}
+		else
+		{
+			border_redraw_decorations(t);
+		}
+		set_focus_window(tmp);
+	}
 	if (flags->do_update_frame_attributes)
 	{
 		setup_frame_attributes(t, pstyle);
@@ -551,8 +618,9 @@ void destroy_scheduled_windows(void)
 	FvwmWindow *next;
 	Bool do_need_ungrab = False;
 
-	if (Scr.flags.is_executing_complex_function != 0 ||
-	    Scr.flags.is_window_scheduled_for_destroy == 0)
+	if (Scr.flags.is_executing_complex_function ||
+	    Scr.flags.is_executing_menu_function ||
+	    !Scr.flags.is_window_scheduled_for_destroy)
 	{
 		return;
 	}
@@ -595,7 +663,6 @@ void apply_decor_change(FvwmWindow *fw)
 	flags.do_redecorate = True;
 	flags.do_update_window_font_height = True;
 	apply_window_updates(fw, &flags, &style, get_focus_window());
-	Scr.flags.do_need_window_update = 1;
 
 	return;
 }
