@@ -65,6 +65,7 @@
 #include "fvwmlib.h"
 #include "safemalloc.h"
 #include "FScreen.h"
+#include "PictureBase.h"
 
 #ifdef HAVE_XINERAMA
 #define FScreenHaveXinerama 1
@@ -79,6 +80,9 @@ typedef struct
 	short width;
 	short height;
 } XineramaScreenInfo;
+#define XineramaQueryScreens(a, b) 0
+#define XineramaQueryExtension(da, b, c) 0
+#define XineramaIsActive(a) 0
 #endif
 
 #ifdef USE_XINERAMA_EMULATION
@@ -168,7 +172,7 @@ static void GetMouseXY(XEvent *eventp, int *x, int *y)
 			eventp = &e;
 			e.type = 0;
 		}
-		GetLocationFromEventOrQuery(
+		fev_get_evpos_or_query(
 			disp, DefaultRootWindow(disp), eventp, x, y);
 	}
 
@@ -246,6 +250,7 @@ void FScreenInit(Display *dpy)
 	{
 		return;
 	}
+	dummy_rc = 0;
 	is_initialised = True;
 	disp = dpy;
 	if (FScreenXineramaEmulation)
@@ -299,7 +304,7 @@ void FScreenInit(Display *dpy)
 		screens_xi[2].width         = w - ws;
 		screens_xi[2].height        = 7 * h / 8;
 		/* add delimiter */
-		attributes.background_pixel = WhitePixel(disp, scr);
+		attributes.background_pixel = PictureWhitePixel();
 		attributes.override_redirect = True;
 		blank_w = XCreateWindow(
 			disp, root, 0, screens_xi[1].y_org - 1,
