@@ -29,6 +29,8 @@
 #include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
 #include "libs/PictureGraphics.h"
+#include "libs/gravity.h"
+#include "libs/FImage.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -58,12 +60,15 @@
  * Top and bottom lines come out full length, the sides come out 1 pixel less
  * This is so FvwmBorder windows have a correct bottom edge and the sticky lines
  * look like just lines
+ * rotation rotate the relief and shadow part 
  */
-void do_relieve_rectangle(
+void do_relieve_rectangle_with_rotation(
 	Display *dpy, Drawable d, int x, int y, int w, int h,
-	GC ReliefGC, GC ShadowGC, int line_width, Bool use_alternate_shading)
+	GC ReliefGC, GC ShadowGC, int line_width, Bool use_alternate_shading,
+	int rotation)
 {
 	XSegment* seg;
+	GC shadow_gc, relief_gc;
 	int i,i2;
 	int a = (use_alternate_shading) ? 1 : 0;
 
@@ -71,35 +76,87 @@ void do_relieve_rectangle(
 	{
 		return;
 	}
+
+	if (rotation == ROTATION_270)
+	{
+		rotation = ROTATION_90;
+		shadow_gc = ReliefGC;
+		relief_gc = ShadowGC;
+	}
+	else if (rotation == ROTATION_180)
+	{
+		rotation = ROTATION_0;
+		shadow_gc = ReliefGC;
+		relief_gc = ShadowGC;
+	}
+	else
+	{
+		shadow_gc = ShadowGC;
+		relief_gc = ReliefGC;
+	}
+
 	seg = (XSegment*)alloca((sizeof(XSegment) * line_width) * 2);
-	/* left side, from 0 to the lesser of line_width & just over half w */
-	for (i = 0; (i < line_width) && (i <= w / 2); i++) {
-		seg[i].x1 = x+i; seg[i].y1 = y+i+a;
-		seg[i].x2 = x+i; seg[i].y2 = y+h-i-1+a;
+	/* from 0 to the lesser of line_width & just over half w */
+	for (i = 0; (i < line_width) && (i <= w / 2); i++)
+	{
+		if (rotation == ROTATION_0)
+		{
+			/* left */
+			seg[i].x1 = x+i; seg[i].y1 = y+i+a;
+			seg[i].x2 = x+i; seg[i].y2 = y+h-i-1+a;
+		}
+		else /* ROTATION_90 */
+		{
+			/* right */
+			seg[i].x1 = x+w-i; seg[i].y1 = y+h-i-a;
+			seg[i].x2 = x+w-i; seg[i].y2 = y+i+1-a;
+		}
 	}
 	i2 = i;
 	/* draw top segments */
-	for (i = 0; (i < line_width) && (i <= h / 2); i++,i2++) {
+	for (i = 0; (i < line_width) && (i <= h / 2); i++,i2++)
+	{
 		seg[i2].x1 = x+w-i-a; seg[i2].y1 = y+i;
 		seg[i2].x2 = x+i+1-a; seg[i2].y2 = y+i;
 	}
-	XDrawSegments(dpy, d, ReliefGC, seg, i2);
+	XDrawSegments(dpy, d, relief_gc, seg, i2);
 	/* bottom */
-	for (i = 0; (i < line_width) && (i <= h / 2); i++) {
+	for (i = 0; (i < line_width) && (i <= h / 2); i++)
+	{
 		seg[i].x1 = x+i+a;     seg[i].y1 = y+h-i;
 		seg[i].x2 = x+w-i-1+a; seg[i].y2 = y+h-i;
 	}
-	/* right */
 	i2 = i;
-	for (i = 0; (i < line_width) && (i <= w / 2); i++,i2++) {
-		seg[i2].x1 = x+w-i; seg[i2].y1 = y+h-i-a;
-		seg[i2].x2 = x+w-i; seg[i2].y2 = y+i+1-a;
+	for (i = 0; (i < line_width) && (i <= w / 2); i++,i2++)
+	{
+		if (rotation == ROTATION_0)
+		{
+			/* right */
+			seg[i2].x1 = x+w-i; seg[i2].y1 = y+h-i-a;
+			seg[i2].x2 = x+w-i; seg[i2].y2 = y+i+1-a;
+		}
+		else
+		{
+			/* left */
+			seg[i2].x1 = x+i; seg[i2].y1 = y+i+a;
+			seg[i2].x2 = x+i; seg[i2].y2 = y+h-i-1+a;
+		}
 	}
-	XDrawSegments(dpy, d, ShadowGC, seg, i2);
+	XDrawSegments(dpy, d, shadow_gc, seg, i2);
 
 	return;
 }
 
+void do_relieve_rectangle(
+	Display *dpy, Drawable d, int x, int y, int w, int h,
+	GC ReliefGC, GC ShadowGC, int line_width, Bool use_alternate_shading)
+{
+	do_relieve_rectangle_with_rotation(
+		dpy, d, x, y, w, h, ReliefGC, ShadowGC, line_width,
+		use_alternate_shading, ROTATION_0);
+	return;
+}
+
 /* Creates a pixmap that is a horizontally stretched version of the input
  * pixmap
  */
@@ -208,7 +265,7 @@ Pixmap CreateStretchPixmap(
 	}
 	pixmap = CreateStretchYPixmap(
 		dpy, temp_pixmap, dest_width, src_height, src_depth,
-		dest_height, gc);
+		dest_height, (gc == None)? my_gc:gc);
 	XFreePixmap(dpy, temp_pixmap);
 	if (my_gc)
 	{
@@ -238,7 +295,11 @@ Pixmap CreateTiledPixmap(
 	}
 	xgcv.fill_style = FillTiled;
 	xgcv.tile = src;
-	XChangeGC(dpy, gc, GCFillStyle | GCTile, &xgcv);
+	xgcv.ts_x_origin = 0;
+	xgcv.ts_y_origin = 0;
+	XChangeGC(
+		dpy, gc, GCFillStyle | GCTile | GCTileStipXOrigin |
+		GCTileStipYOrigin, &xgcv);
 	XFillRectangle(dpy, pixmap, gc, 0, 0, dest_width, dest_height);
 	xgcv.fill_style = FillSolid;
 	XChangeGC(dpy, gc, GCFillStyle, &xgcv);
@@ -246,6 +307,117 @@ Pixmap CreateTiledPixmap(
 	return pixmap;
 }
 
+Pixmap CreateRotatedPixmap(
+	Display *dpy, Pixmap src, int src_width, int src_height, int depth,
+	GC gc, int rotation)
+{
+	GC my_gc = None;
+	Pixmap pixmap = None;
+	int dest_width, dest_height, i, j;
+	Bool error = False;
+	FImage *fim = NULL;
+	FImage *src_fim = NULL;
+
+	if (src_width <= 0 || src_height <= 0)
+	{
+		return None;
+	}
+	switch(rotation)
+	{
+	case ROTATION_90:
+	case ROTATION_270:
+		dest_width = src_height;
+		dest_height = src_width;
+		break;
+	case ROTATION_0:
+	case ROTATION_180:
+		dest_width = src_width;
+		dest_height = src_height;
+		break;
+	default:
+		return None;
+		break;
+	}
+	pixmap = XCreatePixmap(dpy, src, dest_width, dest_height, depth);
+	if (pixmap == None)
+	{
+		return None;
+	}
+	if (gc == None)
+	{
+		my_gc = fvwmlib_XCreateGC(dpy, src, 0, 0);
+	}
+	if (rotation == ROTATION_0)
+	{
+		XCopyArea(
+			dpy, src, pixmap, (gc == None)? my_gc:gc,
+			0, 0, src_width, src_height, 0, 0);
+		goto bail;
+	}
+
+	if (!(src_fim = FGetFImage(
+		dpy, src, Pvisual, depth, 0, 0, src_width, src_height,
+		AllPlanes, ZPixmap)))
+	{
+		error = True;
+		goto bail;
+	}
+	if (!(fim = FCreateFImage(
+		dpy, Pvisual, depth, ZPixmap, dest_width, dest_height)))
+	{
+		error = True;
+		goto bail;
+	}
+
+	for (j = 0; j < src_height; j++)
+	{
+		for (i = 0; i < src_width; i++)
+		{
+			switch(rotation)
+			{
+			case ROTATION_270:
+				XPutPixel(
+					fim->im, j, src_width - i - 1,
+					XGetPixel(src_fim->im, i, j));
+				break;
+			case ROTATION_90:
+				XPutPixel(
+					fim->im, src_height - j - 1, i,
+					XGetPixel(src_fim->im, i, j));
+				break;
+			case ROTATION_180:
+				XPutPixel(
+					fim->im,
+					src_width - i - 1, src_height - j - 1,
+					XGetPixel(src_fim->im, i, j));
+				break;
+			default:
+				break;
+			}
+		}
+	}
+	FPutFImage(dpy, pixmap, gc, fim, 0, 0, 0, 0, dest_width, dest_height);
+ bail:
+	if (error && pixmap)
+	{
+		XFreePixmap(dpy,pixmap);
+		pixmap = None;
+	}
+	if (fim)
+	{
+		FDestroyFImage(dpy, fim);
+	}
+	if (src_fim)
+	{
+		FDestroyFImage(dpy, src_fim);
+	}
+	if (my_gc)
+	{
+		XFreeGC(dpy, my_gc);
+	}
+	return pixmap;
+}
+
 /****************************************************************************
  *
  * Returns True if the given type of gradient is supported.
@@ -727,7 +899,7 @@ Drawable CreateGradientPixmap(
 	Pixmap pixmap = None;
 	PictureImageColorAllocator *pica = NULL;
 	XColor c;
-	XImage *image;
+	FImage *fim;
 	register int i, j;
 	XGCValues xgcv;
 	Drawable target;
@@ -775,19 +947,15 @@ Drawable CreateGradientPixmap(
 		t_height = d_height;
 	}
 
-	/* create an XImage structure */
-	image = XCreateImage(
-		dpy, Pvisual, Pdepth, ZPixmap, 0, 0, t_width, t_height,
-		Pdepth > 16 ? 32 : (Pdepth > 8 ? 16 : 8), 0);
-	if (!image)
+	fim = FCreateFImage(
+		dpy, Pvisual, Pdepth, ZPixmap, t_width, t_height);
+	if (!fim)
 	{
 		fprintf(stderr, "%cGradient couldn't get image\n", type);
 		if (pixmap != None)
 			XFreePixmap(dpy, pixmap);
 		return None;
 	}
-	/* create space for drawing the image locally */
-	image->data = safemalloc(image->bytes_per_line * t_height);
 	if (dither)
 	{
 		pica = PictureOpenImageColorAllocator(
@@ -812,7 +980,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 	}
@@ -831,7 +999,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 		break;
@@ -849,7 +1017,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 		break;
@@ -868,7 +1036,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 		break;
@@ -889,7 +1057,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 	}
@@ -915,7 +1083,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 		break;
@@ -951,7 +1119,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 	}
@@ -1000,7 +1168,7 @@ Drawable CreateGradientPixmap(
 					PictureAllocColorImage(
 						dpy, pica, &c, i, j);
 				}
-				XPutPixel(image, i, j, c.pixel);
+				XPutPixel(fim->im, i, j, c.pixel);
 			}
 		}
 	}
@@ -1008,8 +1176,8 @@ Drawable CreateGradientPixmap(
 	default:
 		/* placeholder function, just fills the pixmap with the first
 		 * color */
-		memset(image->data, 0, image->bytes_per_line * t_height);
-		XAddPixel(image, xcs[0].pixel);
+		memset(fim->im->data, 0, fim->im->bytes_per_line * t_height);
+		XAddPixel(fim->im, xcs[0].pixel);
 		break;
 	}
 
@@ -1038,12 +1206,12 @@ Drawable CreateGradientPixmap(
 		XSetClipRectangles(dpy, gc, 0, 0, rclip, 1, Unsorted);
 	}
 	/* copy the image to the server */
-	XPutImage(dpy, target, gc, image, 0, 0, t_x, t_y, t_width, t_height);
+	FPutFImage(dpy, target, gc, fim, 0, 0, t_x, t_y, t_width, t_height);
 	if (rclip)
 	{
 		XSetClipMask(dpy, gc, None);
 	}
-	XDestroyImage(image);
+	FDestroyFImage(dpy, fim);
 	return target;
 }
 
