@@ -34,6 +34,11 @@
 /* Fvwm needs at least 3 buttons. X currently supports up to 5 buttons, so you
  * can use 3, 4 or 5 here. */
 #define NUMBER_OF_MOUSE_BUTTONS		   5
+/* The "extended" buttons do not provide the full functionality because X has
+ * no bit mask value for them.  Things like dragging windows don't work with
+ * them.  By default, they are ignoreg, but you can use mouse bindingslike
+ * "mouse 7 a a raise" to activate them. */
+#define NUMBER_OF_EXTENDED_MOUSE_BUTTONS   9
 #define DEFAULT_ALL_BUTTONS_MASK	   \
 	((Button1Mask * (NUMBER_OF_MOUSE_BUTTONS >= 1)) | \
 	 (Button2Mask * (NUMBER_OF_MOUSE_BUTTONS >= 2)) | \
@@ -182,6 +187,12 @@
 #define WINDOW_TITLE_TO_STICK_GAP     4 /* pixels */
 /* gap between border of title window and text */
 #define WINDOW_TITLE_TEXT_OFFSET     10 /* pixels */
+/* minimal gap between border of title window and text */
+#define MIN_WINDOW_TITLE_TEXT_OFFSET  2 /* pixels */
+/* left and right padding for the text into under text area (MultiPixmap)*/
+#define TBMP_TITLE_PADDING            5
+/* minimal left and right area vs left/right of text/end (MultiPixmap) */
+#define TBMP_MIN_RL_TITLE_LENGTH      7
 /* maximum number of segemnts in a vector button */
 #define MAX_TITLE_BUTTON_VECTOR_LINES 10000
 
@@ -229,6 +240,8 @@
 #define ICON_TITLE_TO_STICK_EXTRA_GAP	   1 /* pixels */
 /* minimum distance of icons in icon box */
 #define MIN_ICON_BOX_DIST		   3 /* pixels */
+/* padding for the icon into its background */
+#define ICON_BACKGROUND_PADDING            0 /* pixels */
 
 /*** general keyboard shortcuts used in move, resize, ... ***/
 #define DEFAULT_KDB_SHORTCUT_MOVE_DISTANCE 5 /* pixels */
@@ -277,8 +290,9 @@
 #define DEFAULT_EMULATE_MWM	       False
 #define DEFAULT_EMULATE_WIN	       False
 
-#define DEFAULT_USE_ACTIVE_DOWN_BUTTONS True
-#define DEFAULT_USE_INACTIVE_BUTTONS	True
+#define DEFAULT_USE_ACTIVE_DOWN_BUTTONS   True
+#define DEFAULT_USE_INACTIVE_BUTTONS      True
+#define DEFAULT_USE_INACTIVE_DOWN_BUTTONS True
 
 /*** Gradients ***/
 #define MAX_GRADIENT_SEGMENTS		1000
@@ -321,7 +335,7 @@
 #define DEF_FP_MOUSE_BUTTONS ( \
 	((1 << 0) | (1 << 1) | (1 << 2)) & \
 	((1 << NUMBER_OF_MOUSE_BUTTONS) - 1))
-#define DEF_FP_MODIFIERS			0
+#define DEF_FP_MODIFIERS		        0
 #define DEF_FP_PASS_FOCUS_CLICK			1
 #define DEF_FP_PASS_RAISE_CLICK			1
 #define DEF_FP_IGNORE_FOCUS_CLICK_MOTION	0
