@@ -5,7 +5,7 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
@@ -30,7 +30,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/wait.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -62,7 +62,7 @@ typedef struct _PImageIcon {
 	int width;
 	int height;
 	int depth;
-}	 PImageIcon;
+}        PImageIcon;
 
 /**************************************************************************
  * A few function prototypes
@@ -84,7 +84,7 @@ static int MyNameLen;
 
 int timeout = 3000000; /* default time of 3 seconds */
 
-Display *dpy;			/* which display are we talking to */
+Display *dpy;                   /* which display are we talking to */
 Window Root;
 int screen;
 int x_fd;
@@ -116,8 +116,8 @@ int main(int argc, char **argv)
   string = strrchr (argv[0], '/');
   if (string != (char *) 0) string++;
 
-  MyNameLen=strlen(string)+1;		/* account for '*' */
-  MyName = safemalloc(MyNameLen+1);	/* account for \0 */
+  MyNameLen=strlen(string)+1;           /* account for '*' */
+  MyName = safemalloc(MyNameLen+1);     /* account for \0 */
   *MyName='*';
   strcpy (MyName+1, string);
 
@@ -182,8 +182,8 @@ int main(int argc, char **argv)
 	  GC gc;
 	  Pixmap temp;
 
-	  gcv.background= WhitePixel(dpy, screen);
-	  gcv.foreground= BlackPixel(dpy, screen);
+	  gcv.background= PictureWhitePixel();
+	  gcv.foreground= PictureBlackPixel();
 	  gc = fvwmlib_XCreateGC(dpy, win,
 				 GCForeground | GCBackground, &gcv);
 	  temp = XCreatePixmap(dpy, win, view.width, view.height, Pdepth);
@@ -241,7 +241,7 @@ int main(int argc, char **argv)
     FD_ZERO(&in_fdset);
     FD_SET(x_fd,&in_fdset);
 
-    if(!XPending(dpy))
+    if(!FPending(dpy))
 
       retval=select(fd_width,SELECT_FD_SET_CAST &in_fdset, 0, 0, &value);
 
@@ -255,7 +255,7 @@ int main(int argc, char **argv)
     if (FD_ISSET(x_fd, &in_fdset))
     {
       /* read a packet */
-      XNextEvent(dpy,&Event);
+      FNextEvent(dpy,&Event);
       switch(Event.type)
       {
       case ButtonRelease:
@@ -313,26 +313,30 @@ void GetXPMData(char **data)
 
 void GetImageFile(char *file, char *path)
 {
-  char *full_file = NULL;
-  FvwmPictureFlags fpf;
+	char *full_file = NULL;
+	FvwmPictureAttributes fpa;
 
-  fpf.alloc_pixels = 0;
-  fpf.alpha = 0; /* no alpha */
-  if (file)
-    full_file = PictureFindImageFile(file,path,R_OK);
+	fpa.mask = FPAM_NO_ALLOC_PIXELS | FPAM_DITHER | FPAM_NO_ALPHA;
+	if (file)
+		full_file = PictureFindImageFile(file,path,R_OK);
 
-  if (full_file)
-  {
-    if(PImageLoadPixmapFromFile(dpy, Root, full_file, 0,
-				&view.pixmap, &view.mask, None,
-				&view.width, &view.height,
-				&view.depth, 0, NULL,fpf))
-      return;
-    fprintf(stderr,"FvwmBanner: ERROR loading image file\n");
-  }
-  else
-    fprintf(stderr,"FvwmBanner: ERROR finding image file in ImagePath\n");
-  GetXPMData(fvwm_logo_xpm);
+	if (full_file)
+	{
+		if(PImageLoadPixmapFromFile(
+			dpy, Root, full_file, &view.pixmap, &view.mask, None,
+			&view.width, &view.height, &view.depth, 0, NULL, 0,
+			fpa))
+		{
+			return;
+		}
+		fprintf(stderr,"FvwmBanner: ERROR loading image file\n");
+	}
+	else
+	{
+		fprintf(stderr,
+			"FvwmBanner: ERROR finding image file in ImagePath\n");
+		GetXPMData(fvwm_logo_xpm);
+	}
 }
 
 void nocolor(char *a, char *b)
@@ -422,7 +426,7 @@ void change_window_name(char *str)
 /***********************************************************************
  *
  *  Procedure:
- *	SIGPIPE handler - SIGPIPE means fvwm is dying
+ *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
  ***********************************************************************/
 
