@@ -801,6 +801,7 @@ static void ParseButton(button_info **uberb,char *s)
       "center",
       "colorset",
       "action",
+      "id",
       NULL
     };
     s = trimleft(s);
@@ -1106,6 +1107,15 @@ static void ParseButton(button_info **uberb,char *s)
 	  fprintf(stderr,"%s: Unmatched END in config file\n",MyName);
 	  exit(1);
 	}
+	if (ub->parent->c->flags&b_Colorset ||
+	    ub->parent->c->flags&b_ColorsetParent)
+	{
+		ub->c->flags |= b_ColorsetParent;
+	}
+	if (ub->parent->c->flags&b_IconBack || ub->parent->c->flags&b_IconParent)
+	{
+		ub->c->flags |= b_IconParent;
+	}
 	return;
 
       case 13: /* NoSize */
@@ -1204,6 +1214,38 @@ static void ParseButton(button_info **uberb,char *s)
 	  fprintf(stderr,"%s: Missing action argument\n",MyName);
 	break;
 
+      case 20: /* Id */
+	s = trimleft(s);
+	s = DoGetNextToken(s, &t, NULL, ",)", &terminator);
+
+	/* it should include the delimiter */
+	if (s && terminator == ')')
+	{
+	  s--;
+	}
+
+	if (t)
+	{
+	  if (isalpha(t[0]))
+	  {
+	    /* should check for duplicate ids first... */
+	    b->flags |= b_Id;
+	    if (b->id)
+	      free(b->id);
+	    CopyString(&b->id, t);
+	  }
+          else
+	  {
+	    fprintf(stderr, "%s: Incorrect id '%s' ignored\n", MyName, t);
+	  }
+	  free(t);
+	}
+	else
+	{
+	  fprintf(stderr, "%s: Missing id argument\n", MyName);
+	}
+	break;
+
       default:
 	t=seekright(&s);
 	fprintf(stderr,"%s: Illegal button option \"%s\"\n",MyName,
