@@ -63,8 +63,10 @@
 #include "libs/Colorset.h"
 #include "fvwm/fvwm.h"
 #include "libs/PictureGraphics.h"
-#include "FvwmIconBox.h"
+#include "libs/PictureUtils.h"
+#include "libs/Rectangles.h"
 
+#include "FvwmIconBox.h"
 
 char *MyName;
 
@@ -150,6 +152,15 @@ int icon_relief = 4;
 int margin1 = 8;
 int margin2 = 6;
 
+/* true if iconified windows doesn't have () around the title */
+Bool iconified_has_normal_title = False;
+
+int normal_title_relief = 2;
+int iconified_title_relief = 2;
+
+Bool normal_title_inverted_relief = False;
+Bool iconified_title_inverted_relief = False;
+
 Pixmap IconwinPixmap = None;
 char *IconwinPixmapFile = NULL;
 
@@ -338,40 +349,58 @@ void Loop(void)
   int tw, th;
   int diffx, diffy;
   int oldw, oldh;
+  int ex, ey, ex2, ey2;
 
   while( !isTerminated )
     {
       if(My_XNextEvent(dpy,&Event))
+      {
+	switch(Event.type)
 	{
-	  switch(Event.type)
-	    {
-	    case Expose:
-	      if(Event.xexpose.count == 0){
+	case Expose:
+		ex = Event.xexpose.x;
+		ey = Event.xexpose.y;
+		ex2 = Event.xexpose.x + Event.xexpose.width;
+		ey2 = Event.xexpose.y + Event.xexpose.height;
+		while (FCheckTypedWindowEvent(
+			dpy, Event.xany.window, Expose, &Event))
+		{
+			ex = min(ex, Event.xexpose.x);
+			ey = min(ey, Event.xexpose.y);
+			ex2 = max(ex2, Event.xexpose.x + Event.xexpose.width);
+			ey2 = max(ey2 , Event.xexpose.y + Event.xexpose.height);
+		}
+		Event.xexpose.x = ex;
+		Event.xexpose.y = ey;
+		Event.xexpose.width = ex2 - ex;
+		Event.xexpose.height = ey2 - ey;
 		if (Event.xany.window == main_win)
 		{
-			RedrawWindow();
+			RedrawWindow(&Event);
 		}
 		else
 		{
-		  tmp = Head;
-		  while(tmp != NULL){
-		    if (Event.xany.window == tmp->icon_pixmap_w){
-		      RedrawIcon(tmp, 1);
-		      break;
-		    }else if(Event.xany.window == tmp->IconWin){
-		      RedrawIcon(tmp, 2);
-		      break;
-		    }
-		    tmp = tmp->next;
-		  }
+			tmp = Head;
+			while(tmp != NULL)
+			{
+				if (Event.xany.window == tmp->icon_pixmap_w)
+				{
+					RedrawIcon(tmp, 1, &Event);
+					break;
+				}
+				else if (Event.xany.window == tmp->IconWin)
+				{
+					RedrawIcon(tmp, 2, &Event);
+					break;
+				}
+				tmp = tmp->next;
+			}
 		}
-	      }
-	      break;
+		break;
 
-	    case ConfigureNotify:
+	case ConfigureNotify:
 	      if (Event.xconfigure.window == icon_win &&
-		  (colorset < 0 ||
-		   Colorset[colorset].pixmap != ParentRelative))
+		  !CSET_IS_TRANSPARENT(colorset))
 	      {
 		/* needs this event for transparency */
 		break;
@@ -422,19 +451,28 @@ void Loop(void)
 		  change_colorset(colorset);
 		else {
 		  XClearWindow(dpy,main_win);
-		  RedrawWindow();
+		  RedrawWindow(NULL);
 		}
 	      }
-	      else if (ready &&
+	      else if (ready && CSET_IS_TRANSPARENT(colorset) &&
 		       (Event.xconfigure.send_event ||
-			Event.xconfigure.window == icon_win) && colorset >= 0 &&
-		       Colorset[colorset].pixmap == ParentRelative)
+			Event.xconfigure.window == icon_win))
 	      {
-		/* moved */
-		XClearArea(dpy, icon_win, 0,0,0,0, False);
-		/* XClearArea(dpy, icon_win, 0,0,0,0, True); is not enough
-		 * when you unshade */
-		RedrawWindow();
+		      /* moved */
+		      if (!CSET_IS_TRANSPARENT_PR_PURE(colorset))
+		      {
+			      SetWindowBackground(
+				      dpy, icon_win, icon_win_width,
+				      icon_win_height,
+				      &Colorset[(colorset)],
+				      Pdepth, NormalGC, False);
+			      XClearArea(dpy, icon_win, 0,0,0,0, False);
+		      }
+		      else
+		      {
+			      XClearArea(dpy, icon_win, 0,0,0,0, False);
+		      }
+		      /* RedrawWindow(NULL);*/
 	      }
 	      break;
 	    case KeyPress:
@@ -519,8 +557,8 @@ void Loop(void)
 		if ((exhilite = Hilite) != tmp){
 		  Hilite = tmp;
 		  if (exhilite != NULL)
-		    RedrawIcon(exhilite, redraw_flag);
-		  RedrawIcon(tmp, redraw_flag);
+		    RedrawIcon(exhilite, redraw_flag, NULL);
+		  RedrawIcon(tmp, redraw_flag, NULL);
 		}
 	      break;
 
@@ -528,7 +566,7 @@ void Loop(void)
 	      if ((tmp = Search(Event.xcrossing.window)) != NULL &&
 		  tmp == Hilite){
 		Hilite = NULL;
-		RedrawIcon(tmp, redraw_flag);
+		RedrawIcon(tmp, redraw_flag, NULL);
 	      }
 	      if (!(local_flags & HIDE_H) && Event.xbutton.window ==
 		  l_button && Pressed == l_button){
@@ -603,7 +641,7 @@ void Loop(void)
 		  }
 		  AdjustIconWindow(tmp, i);
 		  if (max_icon_height != 0)
-		  RedrawIcon(tmp, 1);
+		  RedrawIcon(tmp, 1, NULL);
 		}
 		break;
 	      }
@@ -631,6 +669,8 @@ void HScroll(int x)
     if (!(local_flags & HIDE_H))
       RedrawHScrollbar();
   }
+  if (CSET_IS_TRANSPARENT_PR_PURE(colorset))
+	  XClearWindow(dpy, icon_win);
 }
 
 void VScroll(int y)
@@ -647,6 +687,8 @@ void VScroll(int y)
     if (!(local_flags & HIDE_V))
       RedrawVScrollbar();
   }
+  if (CSET_IS_TRANSPARENT_PR_PURE(colorset))
+	  XClearWindow(dpy, icon_win);
 }
 
 struct icon_info *Search(Window w)
@@ -667,12 +709,14 @@ struct icon_info *Search(Window w)
  * Draw the window
  *
  ***********************************************************************/
-void RedrawWindow(void)
+void RedrawWindow(XEvent *evp)
 {
   XEvent dummy;
 
-  while (FCheckTypedWindowEvent(dpy, main_win, Expose, &dummy));
-
+  if (!evp)
+  {
+	  while (FCheckTypedWindowEvent(dpy, main_win, Expose, &dummy));
+  }
   RelieveRectangle(dpy, main_win, margin1, margin1, Width + 3,
 		   Height + 3, ShadowGC, ReliefGC, 2);
   if (!(local_flags & HIDE_H))
@@ -702,27 +746,33 @@ void RedrawWindow(void)
   }
 
   /* icons */
-  RedrawIcons();
+  if (!evp)
+  {
+	  RedrawIcons(NULL);
+  }
 }
 
-void RedrawIcons(void)
+void RedrawIcons(XEvent *evp)
 {
   struct icon_info *tmp;
 
   tmp = Head;
   while(tmp != NULL){
     if (window_cond(tmp))
-      RedrawIcon(tmp, redraw_flag);
+      RedrawIcon(tmp, redraw_flag, evp);
     tmp = tmp->next;
   }
 }
 
-void RedrawIcon(struct icon_info *item, int f)
+void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 {
 	int hr, len;
 	int diff, lm ,w, h, tw;
+	int title_relief;
 	char label[256];
 	int cs;
+	XRectangle inter;
+	Region region = None;
 
 	hr = icon_relief/2;
 
@@ -760,8 +810,29 @@ void RedrawIcon(struct icon_info *item, int f)
 	{
 		if (item->iconPixmap != None && item->icon_pixmap_w != None)
 		{
-			if (item->icon_depth == 1 || IS_PIXMAP_OURS(item) ||
-			    Pdefault)
+			Bool do_draw = True;
+
+			if (evp)
+			{
+				if (!frect_get_intersection(
+					evp->xexpose.x, evp->xexpose.y,
+					evp->xexpose.width, evp->xexpose.height,
+					hr, hr, item->icon_w, item->icon_h,
+					&inter))
+				{
+					do_draw = False;
+				}
+			}
+			else
+			{
+				inter.x = hr;
+				inter.y = hr;
+				inter.width = item->icon_w;
+				inter.height = item->icon_h;
+			}
+			if (do_draw &&
+			    (item->icon_depth == 1 || IS_PIXMAP_OURS(item) ||
+			     Pdefault))
 			{
 				FvwmRenderAttributes fra;
 
@@ -775,7 +846,22 @@ void RedrawIcon(struct icon_info *item, int f)
 				    (cs >= 0 &&
 				     Colorset[cs].icon_alpha_percent < 100))
 				{
-					XClearWindow(dpy, item->icon_pixmap_w);
+					if (evp)
+					{
+						XClearArea(
+							dpy, item->icon_pixmap_w,
+							evp->xexpose.x,
+							evp->xexpose.y,
+							evp->xexpose.width,
+							evp->xexpose.height,
+							False);
+					}
+					else
+					{
+						XClearWindow(
+							dpy,
+							item->icon_pixmap_w);
+					}
 				}
 				PGraphicsRenderPixmaps(
 					dpy, item->icon_pixmap_w,
@@ -786,16 +872,19 @@ void RedrawIcon(struct icon_info *item, int f)
 					&fra,
 					item->icon_pixmap_w,
 					NormalGC, None, None,
-					0, 0, item->icon_w, item->icon_h,
-					hr, hr, 0, 0, False);
+					inter.x - hr, inter.y - hr,
+					inter.width, inter.height,
+					inter.x, inter.y,
+					inter.width, inter.height, False);
 			}
-			else
+			else if (do_draw)
 			{
 				XCopyArea(dpy, item->iconPixmap,
 					  item->icon_pixmap_w,
 					  DefaultGC(dpy, screen),
-					  0, 0,
-					  item->icon_w, item->icon_h, 0, 0);
+					  inter.x - hr, inter.y - hr,
+					  inter.width, inter.height,
+					  inter.x, inter.y);
 			}
 		}
 
@@ -805,17 +894,16 @@ void RedrawIcon(struct icon_info *item, int f)
 		{
 			if (item->icon_w > 0 && item->icon_h > 0)
 				RelieveRectangle(dpy, item->icon_pixmap_w,
-						 0, 0, item->icon_w
-						 +icon_relief - 1,
+						 0, 0, 
+						 item->icon_w + icon_relief - 1,
 						 item->icon_h + icon_relief - 1,
 						 IconReliefGC,
 						 IconShadowGC, 2);
 			else
 				RelieveRectangle(dpy, item->icon_pixmap_w,
-						 0, 0, max_icon_width
-						 + icon_relief - 1,
-						 max_icon_height +
-						 icon_relief - 1,
+						 0, 0,
+						 max_icon_width + icon_relief - 1,
+						 max_icon_height + icon_relief - 1,
 						 IconReliefGC,
 						 IconShadowGC, 2);
 		}
@@ -829,24 +917,51 @@ void RedrawIcon(struct icon_info *item, int f)
 
 		if (IS_ICONIFIED(item))
 		{
-			sprintf(label, "(%s)", item->name);
+		        if( ! iconified_has_normal_title )
+			        sprintf(label, "(%s)", item->name);
+			else
+			        strcpy(label, item->name);
+		    
 		}
 		else
 		{
 			strcpy(label, item->name);
 		}
 
+		/* calculate the actual relief thickness */
+		title_relief = IS_ICONIFIED(item) ? 
+			                iconified_title_relief :
+			                normal_title_relief;
+		
+
+		if (evp)
+		{
+			inter.x = evp->xexpose.x;
+			inter.y = evp->xexpose.y;
+			inter.width = evp->xexpose.width;
+			inter.height = evp->xexpose.height;
+			region = XCreateRegion();
+			XUnionRectWithRegion (&inter, region, region);
+			FwinString->flags.has_clip_region = True;
+			FwinString->clip_region = region;
+		}
+		else
+		{
+			FwinString->flags.has_clip_region = False;
+		}
 		len = strlen(label);
 		tw = FlocaleTextWidth(Ffont, label, len);
 		diff = max_icon_width + icon_relief - tw;
 		lm = diff/2;
-		lm = lm > 4 ? lm : 4;
+		lm = lm > 2 + title_relief ? lm : 2 + title_relief;
 
 		FwinString->str = label;
 		FwinString->win = item->IconWin;
 		FwinString->gc =  NormalGC;
 		FwinString->x = lm;
-		FwinString->y = 3 + Ffont->ascent;
+		FwinString->y = 1 + max(normal_title_relief,
+					iconified_title_relief) + 
+		                Ffont->ascent;
 		if (cs >= 0)
 		{
 			FwinString->colorset = &Colorset[cs];
@@ -856,32 +971,124 @@ void RedrawIcon(struct icon_info *item, int f)
 		{
 			FwinString->flags.has_colorset = False;
 		}
+		
 
+		
 		if (Hilite == item)
 		{
 			XRaiseWindow(dpy, item->IconWin);
 			XMoveResizeWindow(dpy, item->IconWin,
 					  item->x + min(0, (diff - 8))/2,
 					  item->y + h,
-					  max(tw + 8, w), 6 + Ffont->height);
-			XClearWindow(dpy, item->IconWin);
+					  max(tw + 4 + 2 * title_relief, w), 
+					  2 + 
+					  2 * max(iconified_title_relief,
+						  normal_title_relief) + 
+					  Ffont->height);
+			if (evp)
+			{
+				XClearArea(
+					dpy, item->IconWin,
+					evp->xexpose.x, evp->xexpose.y,
+					evp->xexpose.width, evp->xexpose.height,
+					False);
+			}
+			else
+			{
+				XClearWindow(dpy, item->IconWin);
+			}
 			FlocaleDrawString(dpy, Ffont, FwinString, 0);
-			RelieveRectangle(dpy, item->IconWin, 0, 0,
-					 max(tw + 8, w) - 1,
-					 6 + Ffont->height - 1,
-					 IconReliefGC, IconShadowGC, 2);
+
+			if((IS_ICONIFIED(item) && 
+			    !iconified_title_inverted_relief) ||
+			   (!IS_ICONIFIED(item) &&
+			    !normal_title_inverted_relief))
+			{
+			        RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 max(tw + 4 + 2 * title_relief,
+						     w) 
+						 - 1,
+						 2 + 
+						 2 * 
+						 max(iconified_title_relief,
+						     normal_title_relief) + 
+						 Ffont->height - 1,
+						 IconReliefGC, IconShadowGC, 
+						 title_relief);
+			}
+			else
+			{
+			         RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 max(tw + 4 + 2 * title_relief,
+						     w) 
+						 - 1,
+						 2 + 
+						 2 * 
+						 max(iconified_title_relief,
+						     normal_title_relief) + 
+						 Ffont->height - 1,
+						 IconShadowGC, IconReliefGC, 
+						 title_relief);
+			}
 		}
+
 		else
 		{
 			XMoveResizeWindow(dpy, item->IconWin,
 					  item->x, item->y + h,
-					  w, 6 + Ffont->height);
-			XClearWindow(dpy, item->IconWin);
+					  w, 
+					  2 + 
+					  2 * max(iconified_title_relief,
+						  normal_title_relief) + 
+					  Ffont->height);
+			if (evp)
+			{
+				XClearArea(
+					dpy, item->IconWin,
+					evp->xexpose.x, evp->xexpose.y,
+					evp->xexpose.width, evp->xexpose.height,
+					False);
+			}
+			else
+			{
+				XClearWindow(dpy, item->IconWin);
+			}
 			FlocaleDrawString(dpy, Ffont, FwinString, 0);
-			RelieveRectangle(dpy, item->IconWin, 0, 0,
-					 w - 1, 5 + Ffont->height,
-					 IconReliefGC, IconShadowGC, 2);
+			
+			if((IS_ICONIFIED(item) && 
+			    !iconified_title_inverted_relief) ||
+			   (!IS_ICONIFIED(item) &&
+			    !normal_title_inverted_relief))
+			{
+			        RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 w - 1,
+						 2 + 
+						 2 * 
+						 max(iconified_title_relief,
+						     normal_title_relief) + 
+						 Ffont->height - 1,
+						 IconReliefGC, IconShadowGC,
+						 title_relief );
+			}
+			else
+			{
+			        RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 w - 1,
+						 2 + 
+						 2 * 
+						 max(iconified_title_relief,
+						     normal_title_relief) + 
+						 Ffont->height - 1,
+						 IconShadowGC, IconReliefGC,
+						 title_relief );
+			}
+			
 		}
+		if (region)
+		{
+			XDestroyRegion(region);
+		}
+		FwinString->flags.has_clip_region = False;
 	}
 
 	if (Hilite == item)
@@ -1110,8 +1317,10 @@ void CreateWindow(void)
   if ((local_flags & HIDE_V))
     h_margin -= bar_width + margin2 + 4;
 
+  /* calculate the (maximal) width and height of an icon */
   UWidth = max_icon_width + icon_relief + interval;
-  UHeight = Ffont->height + max_icon_height + icon_relief + 6 + interval;
+  UHeight = Ffont->height + 2 + 2 * max(normal_title_relief,iconified_title_relief) + 
+            max_icon_height + icon_relief + interval;
   Width = UWidth * num_columns + interval -1;
   Height = UHeight * num_rows + interval -1;
 
@@ -1433,11 +1642,11 @@ static void change_colorset(int color)
 		  {
 			  if (tmp == Hilite && do_update_icon_hi)
 			  {
-				  RedrawIcon(tmp,redraw_flag);
+				  RedrawIcon(tmp,redraw_flag,NULL);
 			  }
 			  if (tmp != Hilite && do_update_icon)
 			  {
-				  RedrawIcon(tmp,redraw_flag);
+				  RedrawIcon(tmp,redraw_flag,NULL);
 			  }
 		  }
 		  tmp = tmp->next;
@@ -1548,9 +1757,9 @@ void Next(void)
     return;
 
   if (old != NULL)
-    RedrawIcon(old, redraw_flag);
+    RedrawIcon(old, redraw_flag,NULL);
   if (new != NULL)
-    RedrawIcon(new, redraw_flag);
+    RedrawIcon(new, redraw_flag,NULL);
 
   i=0;
   if (new->x < icon_win_x){
@@ -1594,9 +1803,9 @@ void Prev(void)
     return;
 
   if (old != NULL)
-    RedrawIcon(old, redraw_flag);
+    RedrawIcon(old, redraw_flag,NULL);
   if (new != NULL)
-    RedrawIcon(new, redraw_flag);
+    RedrawIcon(new, redraw_flag,NULL);
 
   i=0;
   if (new->x < icon_win_x){
@@ -1791,6 +2000,41 @@ void ParseOptions(void)
 	  max_icon_width += 4;
 	}
       }
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "NoIconifiedParentheses"),
+			   Clength+23) == 0)
+      {
+	iconified_has_normal_title = True;
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "NormalTitleRelief"),
+			   Clength+18) == 0)
+      {
+	normal_title_relief = atoi(&tline[Clength+18]);
+      }	
+      
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "IconifiedTitleRelief"),
+			   Clength+21) == 0)
+      {
+	iconified_title_relief = atoi(&tline[Clength+21]);
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "NormalTitleInvertedRelief"),
+			   Clength+26) == 0)
+      {
+	normal_title_inverted_relief = True;
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "IconifiedTitleInvertedRelief"),
+			   Clength+29) == 0)
+      {
+	iconified_title_inverted_relief = True;
+      }
+
       else if (strncasecmp(tline, "Colorset", 8) == 0)
       {
 	LoadColorset(&tline[8]);
@@ -2310,7 +2554,7 @@ void process_message(unsigned long type, unsigned long *body)
 	    SET_DO_SKIP_WINDOW_LIST(tmp,DO_SKIP_WINDOW_LIST(cfgpacket));
 	  }
 	  if ((local_flags & CURRENT_ONLY) &&
-	      tmp->desk != cfgpacket->desk && !IS_STICKY(tmp))
+	      tmp->desk != cfgpacket->desk && !IS_STICKY_ACROSS_DESKS(tmp))
 	  {
 	    olddesk = tmp->desk;
 	    tmp->desk = cfgpacket->desk;
@@ -2323,10 +2567,12 @@ void process_message(unsigned long type, unsigned long *body)
 	      else
 		remove = 1;
 	    }
-	  }else if ((IS_STICKY(cfgpacket)) && !(IS_STICKY(tmp))) /* stick */
-	    SET_STICKY(tmp, True);
-	  else if (!(IS_STICKY(cfgpacket)) && (IS_STICKY(tmp))){ /* unstick */
-	    SET_STICKY(tmp, False);
+	  }else if ((IS_STICKY_ACROSS_DESKS(cfgpacket)) &&
+		    !(IS_STICKY_ACROSS_DESKS(tmp))) /* stick */
+	    SET_STICKY_ACROSS_DESKS(tmp, True);
+	  else if (!(IS_STICKY_ACROSS_DESKS(cfgpacket)) &&
+		   (IS_STICKY_ACROSS_DESKS(tmp))){ /* unstick */
+	    SET_STICKY_ACROSS_DESKS(tmp, False);
 	    tmp->desk = cfgpacket->desk;
 	  }
 	  if ((add && window_cond(tmp)) || remove) {
@@ -2452,7 +2698,7 @@ void process_message(unsigned long type, unsigned long *body)
 	&& window_cond(tmp) && SortItem(tmp) == True)
       AdjustIconWindows();
     if (tmp->IconWin != None && window_cond(tmp))
-      RedrawIcon(tmp, 2);
+      RedrawIcon(tmp, 2, NULL);
     break;
   case M_DEFAULTICON:
     str = (char *)safemalloc(strlen((char *)&body[3])+1);
@@ -2464,7 +2710,7 @@ void process_message(unsigned long type, unsigned long *body)
   case M_ICONIFY:
   case M_DEICONIFY:
     if (ready && (tmp = SetFlag(body[0], type)) != NULL)
-      RedrawIcon(tmp, 2);
+      RedrawIcon(tmp, 2, NULL);
     if (tmp && window_cond(tmp)) animate(tmp,body);
     SendUnlockNotification(fd);
     break;
@@ -2479,9 +2725,9 @@ void process_message(unsigned long type, unsigned long *body)
     old = Hilite;
     Hilite = tmp;
     if (old != NULL)
-      RedrawIcon(old, redraw_flag);
+      RedrawIcon(old, redraw_flag, NULL);
     if (tmp != NULL)
-      RedrawIcon(tmp, redraw_flag);
+      RedrawIcon(tmp, redraw_flag, NULL);
     break;
   case M_NEW_DESK:
     if (CurrentDesk != body[0])
@@ -2558,7 +2804,7 @@ void process_message(unsigned long type, unsigned long *body)
   break;
   case MX_PROPERTY_CHANGE:
     if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND &&
-	(colorset > -1 && Colorset[colorset].pixmap == ParentRelative) &&
+	(CSET_IS_TRANSPARENT_PR(colorset)) &&
 	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == icon_win)))
     {
       XClearArea(dpy, icon_win, 0,0,0,0, False);
@@ -2623,7 +2869,7 @@ int AdjustIconWindows(void)
 int window_cond(struct icon_info *item)
 {
   if ((!(local_flags & CURRENT_ONLY) ||
-      (IS_STICKY(item)) || (item->desk == CurrentDesk)) &&
+      (IS_STICKY_ACROSS_DESKS(item)) || (item->desk == CurrentDesk)) &&
       IS_ICON_SUPPRESSED(item) &&
       (!DO_SKIP_WINDOW_LIST(item) || !UseSkipList))
     return 1;
@@ -2962,35 +3208,50 @@ void ShowKAction(void)
 
 void freeitem(struct icon_info *item, int d)
 {
-  if (item == NULL)
-    return;
+	if (item == NULL)
+		return;
 
-  if (!(IS_ICON_OURS(item))){
-    if (max_icon_height != 0)
-    XUnmapWindow(dpy, item->icon_pixmap_w);
-    if (d == 0)
-      XReparentWindow(dpy, item->icon_pixmap_w, Root, 0, 0);
-  }
+	if (!(IS_ICON_OURS(item))){
+		if (max_icon_height != 0)
+			XUnmapWindow(dpy, item->icon_pixmap_w);
+		if (d == 0)
+			XReparentWindow(dpy, item->icon_pixmap_w, Root, 0, 0);
+	}
 
-  if (item->name != NULL)
-    free(item->name);
-  if (item->window_name != NULL)
-    free(item->window_name);
-  if (item->res_name != NULL)
-    free(item->res_name);
-  if (item->res_class != NULL)
-    free(item->res_class);
-  if (item->icon_file != NULL)
-    free(item->icon_file);
-  if (item->iconPixmap != None)
-    XFreePixmap(dpy, item->iconPixmap);
-  if (item->icon_maskPixmap != None &&
-      (item->wmhints == NULL ||
-       !(item->wmhints->flags & (IconPixmapHint|IconWindowHint))))
-    XFreePixmap(dpy, item->icon_maskPixmap);
-  if (item->wmhints != NULL)
-    XFree(item->wmhints);
-  free(item);
+	if (item->name != NULL)
+		free(item->name);
+	if (item->window_name != NULL)
+		free(item->window_name);
+	if (item->res_name != NULL)
+		free(item->res_name);
+	if (item->res_class != NULL)
+		free(item->res_class);
+	if (item->icon_file != NULL)
+		free(item->icon_file);
+	if (item->iconPixmap != None)
+		XFreePixmap(dpy, item->iconPixmap);
+	if (item->icon_maskPixmap != None &&
+	    (item->wmhints == NULL ||
+	     !(item->wmhints->flags & (IconPixmapHint|IconWindowHint))))
+	{
+		XFreePixmap(dpy, item->icon_maskPixmap);
+	}
+	if (item->iconPixmap != None)
+		XFreePixmap(dpy, item->icon_alphaPixmap);
+	if (item->icon_alloc_pixels != NULL)
+	{
+		if (item->icon_nalloc_pixels != 0)
+		{
+			PictureFreeColors(
+				dpy, Pcmap, item->icon_alloc_pixels,
+				item->icon_nalloc_pixels, 0,
+				item->icon_no_limit);
+		}
+		free(item->icon_alloc_pixels);
+	}
+	if (item->wmhints != NULL)
+		XFree(item->wmhints);
+	free(item);
 }
 
 
