@@ -61,6 +61,9 @@ extern Bool was_bad_access;
 extern char *MyName;
 extern int Iconcolorset;
 
+extern int normal_title_relief;
+extern int iconified_title_relief;
+
 #define ICON_EVENTS (ExposureMask |\
 ButtonReleaseMask | ButtonPressMask | EnterWindowMask | LeaveWindowMask)
 
@@ -127,7 +130,7 @@ void CreateIconWindow(struct icon_info *item)
 	 * visual window to be drawn into */
 	if (IS_ICON_OURS(item))
 	{
-		if (Pdefault | (item->icon_depth == 1) | IS_PIXMAP_OURS(item))
+		if (Pdefault || (item->icon_depth == 1) || IS_PIXMAP_OURS(item))
 		{
 			item->icon_pixmap_w = XCreateWindow(dpy, icon_win, 0, 0,
 					max(max_icon_width, item->icon_w),
@@ -160,11 +163,10 @@ void CreateIconWindow(struct icon_info *item)
 				  hr, hr, item->icon_maskPixmap, FShapeSet);
 	}
 
-	if (item->icon_alphaPixmap != None)
+	if (FShapesSupported)
 	{
-		/* pixmap icon with some alpha */
-		XSetWindowBackgroundPixmap(dpy,
-					   item->icon_pixmap_w, ParentRelative);
+		XSetWindowBackgroundPixmap(
+			dpy, item->icon_pixmap_w, ParentRelative);
 	}
 }
 
@@ -180,8 +182,9 @@ void AdjustIconWindow(struct icon_info *item, int n)
   int x=0,y=0,w,h,h2,h3,w3;
 
   w3 = w = max_icon_width + icon_relief;
+  /* the height of the picture window */
   h3 = h2 = max_icon_height + icon_relief;
-  h = h2 + 6 + Ffont->height;
+  h = h2 + 2 + 2 * max(normal_title_relief,iconified_title_relief) + Ffont->height;
 
   switch (primary){
   case LEFT:
@@ -236,18 +239,28 @@ void AdjustIconWindow(struct icon_info *item, int n)
  ****************************************************************************/
 void GetIconFromFile(struct icon_info *item)
 {
-	char *path = NULL;
+	char *path = PictureFindImageFile(item->icon_file, imagePath, R_OK);
 	FvwmPictureAttributes fpa;
 
-	fpa.mask = FPAM_NO_ALLOC_PIXELS; /* olicha why ? */
+	if (NULL == path)
+	{
+		fprintf(stderr, "[FvwmIconBox] cannot find %s on ImagePath %s\n",
+			item->icon_file, imagePath);
+		return;
+	}
+	fpa.mask = 0;
 	if (Iconcolorset >= 0 && Colorset[Iconcolorset].do_dither_icon)
 	{
 		fpa.mask |= FPAM_DITHER;
 	}
+	item->icon_w = 0;
+	item->icon_h = 0;
 	if (!PImageLoadPixmapFromFile(
-		dpy, Root, path,&item->iconPixmap, &item->icon_maskPixmap,
+		dpy, main_win, path, &item->iconPixmap,
+		&item->icon_maskPixmap,
 		&item->icon_alphaPixmap, &item->icon_w, &item->icon_h,
-		&item->icon_depth, 0, NULL, fpa))
+		&item->icon_depth, &item->icon_nalloc_pixels,
+		&item->icon_alloc_pixels, &item->icon_no_limit, fpa))
 	{
 		fprintf(stderr, "[FvwmIconBox] cannot load pixmap from "
 			"file '%s'\n",path);
@@ -425,7 +438,7 @@ Bool GetBackPixmap(void)
 	}
 	if (!PImageLoadPixmapFromFile(
 		dpy, main_win, path, &tmp_pixmap, &maskPixmap, NULL, &w, &h,
-		&icon_depth, 0, NULL, fpa))
+		&icon_depth, 0, NULL, 0, fpa))
 	{
 		w = 0;
 		h = 0;
