@@ -87,7 +87,8 @@ static int count_nonsticky_in_hashtab (void *arg)
   WinData *win = (WinData *)arg;
   WinManager *man = the_manager;
 
-  if (!(IS_STICKY(win)) && win->complete && win->manager == man)
+  if (!(IS_STICKY_ACROSS_DESKS(win) || IS_STICKY_ACROSS_PAGES(win)) &&
+      win->complete && win->manager == man)
     return 1;
   return 0;
 }
@@ -145,9 +146,21 @@ int win_in_viewport (WinData *win)
   WinManager *manager = win->manager;
   int flag = 0;
   int reverse = 0;
+  rectangle g;
 
   assert (manager);
 
+  if (IS_ICONIFIED(win) && !IS_ICON_SUPPRESSED(win))
+  {
+	  g = win->icon_g;
+  }
+  else
+  {
+	  g.x = win->x;
+	  g.y = win->y;
+	  g.width = win->width;
+	  g.height = win->height;
+  }
   switch (manager->res) {
   case SHOW_GLOBAL:
     flag = 1;
@@ -157,7 +170,7 @@ int win_in_viewport (WinData *win)
     reverse = 1;
     /* fall through to next case */
   case SHOW_DESKTOP:
-    if (IS_STICKY(win) || win->desknum == globals.desknum)
+    if (IS_STICKY_ACROSS_PAGES(win) || win->desknum == globals.desknum)
       flag = 1;
     break;
 
@@ -165,12 +178,12 @@ int win_in_viewport (WinData *win)
     reverse = 1;
     /* fall through to next case */
   case SHOW_PAGE:
-    if (IS_STICKY(win)) {
+    if (IS_STICKY_ACROSS_PAGES(win)) {
       flag = 1;
     } else if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
       flag = RECTANGLES_INTERSECT(
-	win->x, win->y, win->width, win->height,
+	g.x, g.y, g.width, g.height,
 	manager->managed_g.x, manager->managed_g.y,
 	manager->managed_g.width, manager->managed_g.height);
     }
@@ -183,7 +196,7 @@ int win_in_viewport (WinData *win)
     if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
       flag = RECTANGLES_INTERSECT(
-	win->x, win->y, win->width, win->height,
+	g.x, g.y, g.width, g.height,
 	manager->managed_g.x, manager->managed_g.y,
 	manager->managed_g.width, manager->managed_g.height);
     }
@@ -490,17 +503,62 @@ static void mini_icon (FvwmPacketBody *body)
 		win->pic.width, win->pic.height, win->pic.depth);
 }
 
-static void iconify (FvwmPacketBody *body, int dir)
+static void icon_location(FvwmPacketBody *body)
 {
-  Ulong app_id = body->minimal_data.app_id;
-  WinData *win;
+	Ulong app_id = body->minimal_data.app_id;
+	WinData *win;
 
-  win = id_to_win (app_id);
+	win = id_to_win(app_id);
+	if (win == NULL)
+	{
+		return;
+	}
+	win->icon_g.x = body->icon_data.xpos;
+	win->icon_g.y = body->icon_data.ypos;
+	win->icon_g.width = body->icon_data.icon_width;
+	win->icon_g.height = body->icon_data.icon_height;
+	check_in_window (win);
+}
 
-  set_win_iconified (win, dir);
+static void iconify(FvwmPacketBody *body, int dir)
+{
+	Ulong app_id = body->minimal_data.app_id;
+	WinData *win;
 
-  check_win_complete (win);
-  check_in_window (win);
+	win = id_to_win(app_id);
+	set_win_iconified(win, dir);
+	icon_location(body);
+	check_win_complete(win);
+	check_in_window (win);
+}
+
+static void window_shade(FvwmPacketBody *body, int do_shade)
+{
+	Window w;
+	WinManager *man;
+
+	w = (Window)(body->minimal_data.app_id);
+	man = find_windows_manager(w);
+	if (man == NULL)
+	{
+		return;
+	}
+	if (do_shade)
+	{
+		man->flags.needs_resize_after_unshade = 0;
+		man->flags.is_shaded = 1;
+	}
+	else
+	{
+		man->flags.is_shaded = 0;
+		if (man->flags.needs_resize_after_unshade)
+		{
+			draw_manager(man);
+			man->flags.needs_resize_after_unshade = 0;
+		}
+	}
+
+	return;
 }
 
 /* only used by remanage_winlist */
@@ -657,6 +715,10 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     SendUnlockNotification(Fvwm_fd);
     break;
 
+  case M_ICON_LOCATION:
+    icon_location(body);
+    break;
+
   case M_END_WINDOWLIST:
     ConsoleDebug (FVWM, "DEBUG::M_END_WINDOWLIST\n");
     ConsoleDebug (FVWM,
@@ -707,6 +769,14 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     property_change(body);
     break;
 
+  case M_WINDOWSHADE:
+    window_shade(body, 1);
+    break;
+
+  case M_DEWINDOWSHADE:
+    window_shade(body, 0);
+    break;
+
   default:
     break;
   }
