@@ -1459,11 +1459,11 @@ void read_in_resources (char *file)
        ConsoleDebug (CONFIG, "Setting showtransient to: %d\n", i);
        SET_MANAGER (manager, showtransient, i);
       }
-      else if (!strcasecmp (option1, "showonlyiconic")) {
+      else if (!strcasecmp (option1, "showonlyicons")) {
        p = read_next_cmd (READ_ARG);
        if (!p) {
 	 ConsoleMessage ("Bad line: %s\n", current_line);
-	 ConsoleMessage ("Need argument to showonlyiconic\n");
+	 ConsoleMessage ("Need argument to showonlyicons\n");
 	 continue;
        }
        if (!strcasecmp (p, "true")) {
@@ -1477,7 +1477,7 @@ void read_in_resources (char *file)
 	 ConsoleMessage ("What is this: %s?\n", p);
 	 continue;
        }
-       ConsoleDebug (CONFIG, "Setting showonlyiconic to: %d\n", i);
+       ConsoleDebug (CONFIG, "Setting showonlyiconx to: %d\n", i);
        SET_MANAGER (manager, showonlyiconic, i);
       }
       else if (!strcasecmp (option1, "font")) {
