@@ -1133,8 +1133,8 @@ void MakeList(void)
 	AddToList("Y (current page):",   ystr);
 	AddToList("Boundary Width:", borderw);
 
-	AddToList("StickyPage:",    (IS_STICKY_ON_PAGE(targ)    ? yes : no));
-	AddToList("StickyDesk:",    (IS_STICKY_ON_DESK(targ)    ? yes : no));
+	AddToList("StickyPage:",    (IS_STICKY_ACROSS_PAGES(targ) ? yes : no));
+	AddToList("StickyDesk:",    (IS_STICKY_ACROSS_DESKS(targ) ? yes : no));
 	AddToList("NoTitle:",       (HAS_TITLE(targ)    ? no : yes));
 	AddToList("Iconified:",     (IS_ICONIFIED(targ) ? yes : no));
 	AddToList("Transient:",     (IS_TRANSIENT(targ) ? yes : no));
