@@ -12,12 +12,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Combined FvwmTile and FvwmCascade to FvwmRearrange module.
  * 9-Nov-1998 Dominik Vogt
@@ -27,8 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include <string.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 #include <sys/types.h>
 #include <unistd.h>
 #include <signal.h>
@@ -78,7 +77,8 @@ int desk = 0;
 int reversed = 0, raise_window = 1;
 int resize = 0;
 int nostretch = 0;
-int sticky = 0;
+int sticky_page = 0;
+int sticky_desk = 0;
 int flatx = 0, flaty = 0;
 int incx = 0, incy = 0;
 int horizontal = 0;
@@ -131,7 +131,10 @@ int is_suitable_window(unsigned long *body)
   if ((IS_MAXIMIZED(cfgpacket)) && !maximized)
     return 0;
 
-  if ((IS_STICKY(cfgpacket)) && !sticky)
+  if ((IS_STICKY_ACROSS_PAGES(cfgpacket)) && !sticky_page)
+    return 0;
+
+  if ((IS_STICKY_ACROSS_DESKS(cfgpacket)) && !sticky_desk)
     return 0;
 
   if (!XGetWindowAttributes(dpy, cfgpacket->w, &xwa))
@@ -290,7 +293,7 @@ void tile_windows(void)
     }
     wdiv = (maxx - ofsx + 1) / count;
 
-    for (i = 0; w && (i < count); ++i)	{
+    for (i = 0; w && (i < count); ++i)  {
       for (j = 0; w && (j < maxnum); ++j) {
 	int nw = wdiv - w->bw * 2;
 	int nh = hdiv - w->bw * 2 - w->th;
@@ -324,7 +327,7 @@ void tile_windows(void)
     }
     hdiv = (maxy - ofsy + 1) / count;
 
-    for (i = 0; w && (i < count); ++i)	{
+    for (i = 0; w && (i < count); ++i)  {
       for (j = 0; w && (j < maxnum); ++j) {
 	int nw = wdiv - w->bw * 2;
 	int nh = hdiv - w->bw * 2 - w->th;
@@ -405,7 +408,8 @@ void parse_args(char *s, int argc, char *argv[], int argi)
     else if (!strcmp(argv[argi], "-a")) {
       all = untitled = transients = maximized = 1;
       if (FvwmCascade)
-	sticky = 1;
+	sticky_page = 1;
+	sticky_desk = 1;
     }
     else if (!strcmp(argv[argi], "-r")) {
       reversed = 1;
@@ -438,7 +442,14 @@ void parse_args(char *s, int argc, char *argv[], int argi)
       maximized = 1;
     }
     else if (!strcmp(argv[argi], "-s")) {
-      sticky = 1;
+      sticky_page = 1;
+      sticky_desk = 1;
+    }
+    else if (!strcmp(argv[argi], "-sp")) {
+      sticky_page = 1;
+    }
+    else if (!strcmp(argv[argi], "-sd")) {
+      sticky_desk = 1;
     }
     else if (!strcmp(argv[argi], "-mn") && ((argi + 1) < argc)) {
       maxnum = atoi(argv[++argi]);
