@@ -301,8 +301,8 @@ int IsItemIndexSticky(List *list, int i)
   Item *temp;
   for(temp=list->head;temp!=NULL && temp->count!=i;temp=temp->next);
   if (temp==NULL) return -1;
-  return (IS_STICKY_ON_DESK(temp) ||
-	  (IS_ICONIFIED(temp) && IS_ICON_STICKY_ON_DESK(temp)));
+  return (IS_STICKY_ACROSS_DESKS(temp) ||
+	  (IS_ICONIFIED(temp) && IS_ICON_STICKY_ACROSS_DESKS(temp)));
 }
 
 /******************************************************************************
