@@ -158,6 +158,12 @@ int ReliefWidth = 2;
 long CurrentDesk = 0;
 int ShowCurrentDesk = 0;
 
+struct
+{
+	unsigned is_shaded : 1;
+	unsigned needs_resize_after_unshade : 1;
+} flags = { 0, 0 };
+
 static RETSIGTYPE TerminateHandler(int sig);
 
 static char *AnimCommand = NULL;
@@ -175,6 +181,33 @@ static RETSIGTYPE TerminateHandler(int sig)
   fvwmSetTerminate(sig);
 }
 
+static void window_shade(unsigned long *body, int do_shade)
+{
+	Window w;
+
+	w = (Window)(body[0]);
+	if (w != win)
+	{
+		return;
+	}
+	if (do_shade)
+	{
+		flags.is_shaded = 1;
+		flags.needs_resize_after_unshade = 0;
+	}
+	else
+	{
+		flags.is_shaded = 0;
+		if (flags.needs_resize_after_unshade)
+		{
+			AdjustWindow(True);
+			flags.needs_resize_after_unshade = 0;
+		}
+	}
+
+	return;
+}
+
 /******************************************************************************
   Main - Setup the XConnection,request the window list and loop forever
     Based on main() from FvwmIdent:
@@ -296,7 +329,8 @@ int main(int argc, char **argv)
 		 M_CONFIGURE_WINDOW | M_ADD_WINDOW | M_DESTROY_WINDOW |
 		 M_VISIBLE_NAME | M_DEICONIFY |
 		 M_ICONIFY | M_END_WINDOWLIST | M_NEW_DESK | M_FOCUS_CHANGE |
-		 M_CONFIG_INFO | M_SENDCONFIG);
+		 M_CONFIG_INFO | M_SENDCONFIG | M_WINDOWSHADE |
+		 M_DEWINDOWSHADE);
   /* extended messages */
   SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME | MX_PROPERTY_CHANGE);
 
@@ -403,15 +437,16 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       {
 	flagitem=ItemFlags(&windows,cfgpacket->w);
 	if(UpdateItemDesk(&windows, cfgpacket) > 0 ||
-	   IS_STICKY(cfgpacket) != IS_STICKY(flagitem) ||
+	   IS_STICKY_ACROSS_DESKS(cfgpacket) !=
+	   IS_STICKY_ACROSS_DESKS(flagitem) ||
 	   DO_SKIP_WINDOW_LIST(cfgpacket) != DO_SKIP_WINDOW_LIST(flagitem))
 	{
 	  UpdateItemGSFRFlags(&windows, cfgpacket);
 	  UpdateButtonDeskFlags(&buttons, i, cfgpacket->desk,
-				IS_STICKY(cfgpacket),
+				IS_STICKY_ACROSS_DESKS(cfgpacket),
 				DO_SKIP_WINDOW_LIST(cfgpacket));
 	  AdjustWindow(False);
-	  RedrawWindow(True, True);
+	  RedrawWindow(True, True, NULL);
 	}
 	else
 	  UpdateItemGSFRFlags(&windows, cfgpacket);
@@ -463,7 +498,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	AddButton(&buttons, name, NULL, 1);
 	UpdateButtonSet(&buttons, i, (IS_ICONIFIED(flagitem)?1:0));
 	UpdateButtonDeskFlags(&buttons,i,ItemDesk(&windows, body[0]),
-			      IS_STICKY(flagitem),
+			      IS_STICKY_ACROSS_DESKS(flagitem),
 			      DO_SKIP_WINDOW_LIST(flagitem));
       }
       free(name);
@@ -538,14 +573,20 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       if(ShowCurrentDesk)
       {
 	AdjustWindow(False);
-	RedrawWindow(True, True);
+	RedrawWindow(True, True, NULL);
       }
       break;
+    case M_WINDOWSHADE:
+      window_shade(body, 1);
+      break;
+    case M_DEWINDOWSHADE:
+      window_shade(body, 0);
+      break;
   case MX_PROPERTY_CHANGE:
     if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND && body[2] == 0 &&
 	WindowState &&  win_bg == ParentRelative)
     {
-      RedrawWindow(True, True);
+      RedrawWindow(True, True, NULL);
     }
     break;
     case M_CONFIG_INFO:
@@ -553,16 +594,16 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       break;
   }
 
-  if (redraw && WindowState==1) RedrawWindow(False, True);
+  if (redraw && WindowState==1) RedrawWindow(False, True, NULL);
 }
 
 
 /******************************************************************************
   RedrawWindow - Update the needed lines and erase any old ones
 ******************************************************************************/
-void RedrawWindow(Bool force, Bool clear_bg)
+void RedrawWindow(Bool force, Bool clear_bg, XEvent *evp)
 {
-  DrawButtonArray(&buttons, force, clear_bg);
+  DrawButtonArray(&buttons, force, clear_bg, evp);
   XFlush(dpy); /** Put here for VMS; ifdef if causes performance problem **/
   if (FQLength(dpy) && !force) LoopOnEvents();
 }
@@ -746,16 +787,17 @@ ParseConfigLine(char *tline)
 	/* set the window background */
 	win_bg = None;
 	for (i = 0; i != MAX_COLOUR_SETS; i++)
-	    if (colorset[i] >= 0 &&
-		Colorset[colorset[i]].pixmap == ParentRelative)
+	{
+	    if (CSET_IS_TRANSPARENT_PR(colorset[i]))
 	      win_bg = ParentRelative;
+	}
 	if (old_win_bg != win_bg)
 	  XSetWindowBackgroundPixmap(dpy, win, win_bg);
 	/* refresh if this colorset is used */
 	for (i = 0; i != MAX_COLOUR_SETS; i++) {
 	  if (colorset[i] == cset) {
 	    AdjustWindow(True);
-	    RedrawWindow(True, True);
+	    RedrawWindow(True, True, NULL);
 	    break;
 	  }
 	}
@@ -845,15 +887,25 @@ void LoopOnEvents(void)
 	Pressed=1;
 	break;
       case Expose:
-	/* have to clear each button with its background,
-	 * let the server compute the intersections */
-	do
-	  ExposeAllButtons(&buttons, &Event);
-	while (FCheckTypedWindowEvent(dpy, win, Expose, &Event));
-	/* No more expose events in the queue draw all icons,
-	 * text and shadows, but not the background */
-	RedrawWindow(True, False);
-	break;
+      {
+	      int ex = Event.xexpose.x;
+	      int ey = Event.xexpose.y;
+	      int ex2 = Event.xexpose.x + Event.xexpose.width;
+	      int ey2 = Event.xexpose.y + Event.xexpose.height;
+	      while (FCheckTypedWindowEvent(dpy, win, Expose, &Event))
+	      {
+		      ex = min(ex, Event.xexpose.x);
+		      ey = min(ey, Event.xexpose.y);
+		      ex2 = max(ex2, Event.xexpose.x + Event.xexpose.width);
+		      ey2 = max(ey2 , Event.xexpose.y + Event.xexpose.height);
+	      }
+	      Event.xexpose.x = ex;
+	      Event.xexpose.y = ey;
+	      Event.xexpose.width = ex2 - ex;
+	      Event.xexpose.height = ey2 - ey;
+	      RedrawWindow(True, True, &Event);
+	      break;
+      }
       case ConfigureNotify:
 	{
 	  XEvent event;
@@ -872,13 +924,14 @@ void LoopOnEvents(void)
 	    Event.xconfigure.y = event.xconfigure.y;
 	    Event.xconfigure.send_event = True;
 	  }
-	}
-	if (Event.xconfigure.send_event && (win_x != Event.xconfigure.x
-	    || win_y != Event.xconfigure.y || win_bg == ParentRelative)) {
-	  win_x = Event.xconfigure.x;
-	  win_y = Event.xconfigure.y;
-	  if (win_bg == ParentRelative)
-	    RedrawWindow(True, True);
+	  if (Event.xconfigure.send_event &&
+	      (win_x != Event.xconfigure.x ||
+	       win_y != Event.xconfigure.y || win_bg == ParentRelative))
+	  {
+		  win_x = Event.xconfigure.x;
+		  win_y = Event.xconfigure.y;
+		  DrawTransparentButtonArray(&buttons);
+	  }
 	}
 	break;
       case KeyPress:
@@ -1001,12 +1054,23 @@ void AdjustWindow(Bool force)
       if (cset >= 0) {
 	if (pixmap[i])
 	  XFreePixmap(dpy, pixmap[i]);
-	if (Colorset[cset].pixmap) {
-	  pixmap[i] = CreateBackgroundPixmap(dpy, win, new_width, buttonheight,
-					     &Colorset[cset], Pdepth,
-					     background[i], False);
-	  XSetTile(dpy, background[i], pixmap[i]);
-	  XSetFillStyle(dpy, background[i], FillTiled);
+	if (Colorset[cset].pixmap && !CSET_IS_TRANSPARENT_PR(cset)) {
+		if (CSET_IS_TRANSPARENT_ROOT(cset))
+		{
+			pixmap[i] = CreateBackgroundPixmap(
+				dpy, win, new_width, win_height,
+				&Colorset[cset], Pdepth,
+				background[i], False);
+		}
+		else
+		{
+			pixmap[i] = CreateBackgroundPixmap(
+				dpy, win, win_width, buttonheight,
+				&Colorset[cset], Pdepth,
+				background[i], False);
+		}
+		XSetTile(dpy, background[i], pixmap[i]);
+		XSetFillStyle(dpy, background[i], FillTiled);
 	} else {
 	  pixmap[i] = None;
 	  XSetFillStyle(dpy, background[i], FillSolid);
@@ -1014,7 +1078,9 @@ void AdjustWindow(Bool force)
       }
     }
   }
-  if (WindowState && (new_height!=win_height || new_width!=win_width))
+  if (WindowState &&
+      (new_height!=win_height || new_width!=win_width ||
+       (!flags.is_shaded && flags.needs_resize_after_unshade)))
   {
 #ifdef COMPLEX_WINDOW_PLACEMENT
     if (Anchor)
@@ -1046,14 +1112,19 @@ void AdjustWindow(Bool force)
       }
       /* XMoveResizeWindow(dpy, win, win_x, win_y, new_width, new_height); */
     }
-    if (Anchor && WindowState == 2)
+    if (flags.is_shaded)
+      flags.needs_resize_after_unshade = 1;
+    else if (Anchor && WindowState == 2)
       XMoveResizeWindow(dpy, win, win_x, win_y, new_width, new_height);
     else
       XResizeWindow(dpy, win, new_width, new_height);
     /* The new code is sooo much simpler :-) */
 #else
-    /* This relies on fvwm honouring South/East gemoetry when resizing. */
-    XResizeWindow(dpy, win, new_width, new_height);
+    if (flags.is_shaded)
+      flags.needs_resize_after_unshade = 1;
+    else
+      /* This relies on fvwm honouring South/East gemoetry when resizing. */
+      XResizeWindow(dpy, win, new_width, new_height);
 #endif
   }
   UpdateArray(&buttons,new_width);
@@ -1237,11 +1308,13 @@ void MakeMeWindow(void)
    * the colorsets is transparent which requires help from the server */
   win_bg = None;
   for (i = 0; i != MAX_COLOUR_SETS; i++)
-    if (colorset[i] >= 0 && Colorset[colorset[i]].pixmap == ParentRelative)
-    {
-      win_bg = ParentRelative;
-      break;
-    }
+  {
+	  if (CSET_IS_TRANSPARENT_PR(colorset[i]))
+	  {
+		  win_bg = ParentRelative;
+		  break;
+	  }
+  }
   attr.background_pixmap = win_bg;
   attr.border_pixel = 0;
   attr.colormap = Pcmap;
@@ -1329,13 +1402,25 @@ void MakeMeWindow(void)
     gcmask=GCForeground;
     background[i]=fvwmlib_XCreateGC(dpy,win,gcmask,&gcval);
 
-    if ((colorset[i] >= 0) && Colorset[colorset[i]].pixmap
-	&& Colorset[colorset[i]].pixmap != ParentRelative) {
-      pixmap[i] = CreateBackgroundPixmap(dpy, win, win_width, buttonheight,
-					 &Colorset[colorset[i]], Pdepth,
-					 background[i], False);
-      XSetTile(dpy, background[i], pixmap[i]);
-      XSetFillStyle(dpy, background[i], FillTiled);
+    if ((colorset[i] >= 0) && Colorset[colorset[i]].pixmap &&
+	!CSET_IS_TRANSPARENT_PR(colorset[i]))
+    {
+	    if (CSET_IS_TRANSPARENT_ROOT(colorset[i]))
+	    {
+		    pixmap[i] = CreateBackgroundPixmap(
+			    dpy, win, win_width, win_height,
+			    &Colorset[colorset[i]], Pdepth,
+			    background[i], False);
+	    }
+	    else
+	    {
+		    pixmap[i] = CreateBackgroundPixmap(
+			    dpy, win, win_width, buttonheight,
+			    &Colorset[colorset[i]], Pdepth,
+			    background[i], False);
+	    }
+	    XSetTile(dpy, background[i], pixmap[i]);
+	    XSetFillStyle(dpy, background[i], FillTiled);
     }
   }
   AdjustWindow(True);
@@ -1393,6 +1478,7 @@ void StartMeUp_I(void)
       XDisplayName(""));
     exit (1);
   }
+XSynchronize(dpy, 1);
   PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
